/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999, 2000, 2001 Sebastien Nicoud <seb@eseb.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#include "common.h"
#include "cgame.h"
#include "csettings.h"
#include "chof.h"

const gchar *CHof::str_version = "version";
const gchar *CHof::str_id = "id";
const gchar *CHof::str_score = "score";
const gchar *CHof::str_level = "level";
const gchar *CHof::str_layers = "layers";
const gchar *CHof::str_timer = "time";
const gchar *CHof::str_name = "name";
const gchar *CHof::str_date = "date";
const gchar *CHof::str_eog = "$";

CHof::CHof()
{
	if (debug) g_print("CHof::CHof()\n");
	_rank = 0;
	memset(_version,0,MAX_HOF_S_LEN);
	_id = 0;
	_difficulty = 0;
	_d = 0;
	_w = 0;
	_h = 0;
	_score = 0;
	_layers = 0;
	_level = 0;
	_timer = 0;
	memset(_date,0,MAX_HOF_S_LEN);
	memset(_name,0,MAX_NAME_LEN);
}

CHof::CHof(const CHof &ind)
{
	if (debug) g_print("CHof::CHof(CHof &)\n");
	_rank = ind._rank;
	if (ind._version != NULL) strcpy(_version, ind._version);
	_id = ind._id;
	_difficulty = ind._difficulty;
	_d = ind._d;
	_w = ind._w;
	_h = ind._h;
	_score = ind._score;
	_layers = ind._layers;
	_level = ind._level;
	_timer = ind._timer;
	if (ind._date != NULL) strcpy(_date, ind._date);
	if (ind._name != NULL) strcpy(_name, ind._name);
}

CHof::CHof(const CGame &game)
{
	if (debug) g_print("CHof::CHof(CGame &)\n");
	_rank = 0;
	strcpy(_version,GNO3DTET_VERSION);
	_id = game._id;
	if (game._settings != NULL) {
		_difficulty = game._settings->_difficulty;
		_d = game._settings->_d;
		_w = game._settings->_w;
		_h = game._settings->_h;
	}
	_score = game._score;
	_layers = game._layers;
	_level = game._level;
	_timer = game._timer;
	if (game._date != NULL) strncpy(_date, game._date, MAX_HOF_S_LEN);
	if (game._name != NULL) strncpy(_name, game._name, MAX_NAME_LEN);
}

CHof::~CHof(void)
{
	if (debug) g_print("CHof::~CHof()\n");
}

CHof &CHof::operator= (const CHof &ind)
{
	if (debug > 2) g_print("CHof::operator=\n");
	_rank = ind._rank;
	if (ind._version != NULL) strcpy(_version, ind._version);
	_id = ind._id;
	_difficulty = ind._difficulty;
	_d = ind._d;
	_w = ind._w;
	_h = ind._h;
	_score = ind._score;
	_layers = ind._layers;
	_level = ind._level;
	_timer = ind._timer;
	if (ind._date != NULL) strcpy(_date, ind._date);
	if (ind._name != NULL) strcpy(_name, ind._name);
	return *this;
}

ostream &operator<< (ostream &os, CHof &ind)
{
//	if (debug > 2) g_print("CHof::operator<<\n");
	os	 << "hof record v."<< ind._version
		 << ": id=" << ind._id
		 << ", rank=" << ind._rank
	     << ", diff=" << ind._difficulty
		 << ", d=" << ind._d
		 << ", w=" << ind._w
		 << ", h=" << ind._h
		 << ", score=" << ind._score
		 << ", layers=" << ind._layers
		 << ", level=" << ind._level
		 << ", timer=" << ind._timer
		 << ", date=" << ind._date
		 << ", name=" << ind._name
		 << endl;
	return os;
}

gchar *CHof::compute_date(void)
{
	if (debug) g_print("CGame::compute_date\n");
	time_t td;
	time(&td);
	strcpy(_date, ctime(&td));
	if (_date[strlen(_date)-1] == '\n') _date[strlen(_date)-1] = '\0';
	if (debug) g_print("  date = >%s<\n",_date);
	return _date;
}

/* is_better_than
 * tells if a game is better than another
 * version > score > level > layers > depth
 * <-- most important    less -->
*/
gboolean CHof::is_better_than(CHof *g) const
{
	if (debug > 2) g_print("CHof::is better than\n");
	if (g == NULL) return TRUE; // p1 is better than nobody
	if (quantify_version() > g->quantify_version()) return TRUE;
	else if (quantify_version() < g->quantify_version()) return FALSE;
	if (_score > g->_score) return TRUE;
	else if (_score < g->_score) return FALSE;
	if (_level > g->_level) return TRUE;
	else if (_level < g->_level) return FALSE;
	if (_layers > g->_layers) return TRUE;
	else if (_layers < g->_layers) return FALSE;
	if (_d < g->_d) return TRUE;
	return FALSE;
}

gboolean CHof::is_version_current(gchar *s) const
{
	if (debug > 2) g_print("CHof::is_version_current\n");
    return (quantify_version(s) == CURRENT);
}

/* return a quantified value of the version string (ex. "1.5.0")
 * if s == "current" return current value
 * if s != NULL, s is processed instead of the _version attribute
 * CURRENT is defined in chof.h
 */
gint CHof::quantify_version(gchar *s) const
{
    // versions:
    //  0.0.0 -> 1.4.99 => 1
    //  1.5.0 -> 1.5.99 => 2
    //  1.6.0 -> or more => 3 <- CURRENT=3
	if (debug > 2) g_print("CHof::quantify_version %s\n",_version);
	if (s != NULL) {
		if (!strcmp(s,"current")) return CURRENT;
		if (strcmp(s,"1.6.0") >= 0) return CURRENT; // s is greater than 1.6.0: for ex: 1.6.3
		if (strcmp(s,"1.5.0") >= 0) return 2; // s is greater than 1.5.0 (but smaller than 1.6.0 <- preceeding line)
		return 1; // anything else (smaller than 1.5.0) is version 1
	}
	// to add versioning quantification, copy and uncomment the following line:
	//if (!strcmp(_version,"x.y.z")) return <current vers.>;
	if (strcmp(_version,"1.6.0") >= 0) return CURRENT;
	if (strcmp(_version,"1.5.0") >= 0) return 2;
	if (!strcmp(_version,"")) return 1; // no version = version 1
	return 1; // anything else is version 1 (that point shouldn't be reached)
}

gchar *CHof::stringify_date(void) const
{
	if (debug > 2) g_print("CHof::stringify_date\n");
	gchar *str;
	if (_date == NULL) str = g_strdup(_("unknown"));
	else str = g_strdup(_date);
	return str;
}

gchar *CHof::stringify_depth(void) const
{
	if (debug > 2) g_print("CHof::stringify_depth\n");
	gchar *str;
	if (_d < MIN_WORLD_DEPTH
	 || _d > MAX_WORLD_DEPTH) str = g_strdup(_("?"));
	else str = g_strdup_printf("%d",_d);
	return str;
}

gchar *CHof::stringify_difficulty(void) const
{
	if (debug > 2) g_print("CHof::stringify_difficulty\n");
	gchar *str;
	if (_difficulty < CSettings::diff_EASY
	 || _difficulty > CSettings::diff_HARD)
		str = g_strdup(_("?"));
	else str = g_strdup(CSettings::diff_descriptors[_difficulty]);
	return str;
}

gchar *CHof::stringify_height(void) const
{
	if (debug > 2) g_print("CHof::stringify_height\n");
	gchar *str;
	if (_h < MIN_WORLD_HEIGHT
	 || _h > MAX_WORLD_HEIGHT)
		str = g_strdup(_("?"));
	else str = g_strdup_printf("%d",_h);
	return str;
}

gchar *CHof::stringify_layers(void) const
{
	if (debug > 2) g_print("CHof::stringify_layers\n");
	gchar *str;
	if (_layers == -1) str = g_strdup(_("?"));
	else str = g_strdup_printf("%d",_layers);
	return str;
}

gchar *CHof::stringify_level(void) const
{
	if (debug > 2) g_print("CHof::stringify_level\n");
	gchar *str;
	if (_level == -1) str = g_strdup(_("?"));
	else str = g_strdup_printf("%d",_level);
	return str;
}

gchar *CHof::stringify_name(void) const
{
	if (debug > 2) g_print("CHof::stringify_name\n");
	gchar *str;
	if (_name == NULL) str = g_strdup(_("Anonymous"));
	else str = g_strdup(_name);
	return str;
}

gchar *CHof::stringify_rank(void) const
{
	if (debug > 2) g_print("CHof::stringify_rank\n");
	gchar *str;
	if (_rank < 0
	 || _h > MAX_NHOFL+1)
		str = g_strdup(_("(?)"));
	else if (is_version_current())
		str = g_strdup_printf("%d",_rank);
	else 
		str = g_strdup_printf("(%d)",_rank);
	return str;
}

gchar *CHof::stringify_score(void) const
{
	if (debug > 2) g_print("CHof::stringify_score\n");
	gchar *str;
	if (_score == -1) str = g_strdup(_("?"));
	else str = g_strdup_printf("%d",_score);
	return str;
}

gchar *CHof::stringify_timer(void) const
{
//	if (debug > 2) g_print("CHof::stringify time\n");
	if (_timer > (999*60+59)*60+59) return g_strdup("888:88:88");
	gint seconds = div((int)_timer,60).rem;
	int minutes = div((int)_timer,60).quot;
	gint heures = div(minutes,60).quot;
	minutes = div(minutes,60).rem;
	return g_strdup_printf("%03u:%02u:%02u",heures,minutes,seconds);
}

gchar *CHof::stringify_width(void) const
{
	if (debug > 2) g_print("CHof::stringify_width\n");
	gchar *str;
	if (_w < MIN_WORLD_WIDTH
	 || _w > MAX_WORLD_WIDTH)
		str = g_strdup(_("?"));
	else str = g_strdup_printf("%d",_w);
	return str;
}

// IO STUFF ///////////////////////////////////////////////////////////////////

void CHof::dialog_error_reading(ifstream &s, const gchar *name, gchar *m) {
	if (debug) g_print("CHof::dialog error reading\n");
	if (s.is_open()) s.close();
	gchar *msg = g_strdup_printf(_(
		"An error occured while reading data from file:\n \
		%s:\n \
		%s.\n \
		Operation aborted"),
		name, m);
	GtkWidget *dialog = gnome_error_dialog(msg);
	gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
	g_free(msg);
}

void CHof::dialog_error_writing(ofstream &s, const gchar *name, gchar *m) {
	if (debug) g_print("CHof::dialog error wrinting\n");
	if (s.is_open()) s.close();
	gchar *msg = g_strdup_printf(_(
		"An error occured while writing data to file:\n \
		%s:\n \
		%s.\n \
		Operation aborted"),
		name, m);
	GtkWidget *dialog = gnome_error_dialog(msg);
	gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
	g_free(msg);
}

/* read_game
 * RET: 0 -> ok
 *      1 -> empty file / EOF
 *		3 -> safe counter failure
 *		4 -> read invalid value
 *		5 -> read unknown key
 *      6 -> error opening
 *		- -> other
*/
gint CHof::read_game(const gchar *filename,
	gboolean pos_beginning_or_not_current)
{
	if (pos_beginning_or_not_current) CHof::_pos = 0L;
	long pos = CHof::_pos;
	if (debug) g_print("CHof::read_game (>%s<,%d) @ pos = %d\n",\
		filename,pos_beginning_or_not_current,pos);
	gint ret = read_at(&pos, filename);
	_pos = pos;
	if (debug) g_print("CHof::read_game: exit elvl %d\n",ret);	
	return ret;
}

gint CHof::read_at(long *pos, const gchar *filename)
{
	if (debug > 2) g_print("CHof::read_at >%s< %d\n",filename,*pos);

	// open file
	ifstream fin(filename);
	if (!fin.is_open()) {
		dialog_error_reading(fin, filename, _("File opening failed"));
		return 6;
	}
	const gint MAX_BUF = 1024;
	gchar buf[MAX_BUF] = "";		
	gchar buf2[MAX_BUF];
	const gint safe_counter_init = 13; // increment if fields are added
	gint safe_counter = safe_counter_init;
	fin.seekg(*pos);

	// check for EOF
	fin >> buf;
	if (fin.eof()) {
		if (debug) g_print("CHof::read_at: EOF\n");
		return 1;
	}
	else fin.seekg(*pos); // rewind

	// setup default version in case it isn't recorded
	strcpy(_version,"0.0.0");
	
	// read record
	fin.seekg(*pos); // rewind
	safe_counter = safe_counter_init;
	fin >> buf;
	while (fin.good()) {
		if (--safe_counter < 0) return 3;
		if (debug > 2) g_print("  buffer read = >%s<\n",buf);
		if (!g_strcasecmp(str_id, buf)) {
			fin >> _id;
			if (debug > 2) g_print("  id found: read = %d\n",_id);		
		} else if (!g_strcasecmp(str_version, buf)) {
			fin.getline(buf2, MAX_BUF);
			gint offset = 0;
			while (buf2[offset] == ' ') { offset++; };
			if (buf2[offset] == '\0') offset = 0;
			strncpy(_version, &buf2[offset], MAX_HOF_S_LEN);
			if (debug > 2) g_print("  version found: read = >%s<\n",_version);
		} else if (!g_strcasecmp(str_name, buf)) {
			fin.getline(buf2, MAX_BUF);
			gint offset = 0;
			while (buf2[offset] == ' ') { offset++; };
			if (buf2[offset] == '\0') offset = 0;
			strncpy(_name, &buf2[offset], MAX_NAME_LEN);
			if (debug > 2) g_print("  name found: read = >%s<\n",_name);		
		} else if (!g_strcasecmp(str_timer, buf)) {
			fin >> _timer;
			if (debug > 2) g_print("  time found: read = %d\n",_timer);		
		} else if (!g_strcasecmp(str_date, buf)) {
			fin.getline(buf2, MAX_BUF);
			gint offset = 0;
			while (buf2[offset] == ' ') { offset++; };
			if (buf2[offset] == '\0') offset = 0;
			strncpy(_date, &buf2[offset], MAX_HOF_S_LEN);
			if (debug > 2) g_print("  date found: read = >%s<\n",_date);		
		} else if (!g_strcasecmp(str_level, buf)) {
			fin >> _level;
			if (debug > 2) g_print("  level found: read = %d\n",_level);		
		} else if (!g_strcasecmp(str_score, buf)) {
			fin >> _score;
			if (debug > 2) g_print("  score found: read = %d\n",_score);		
		} else if (!g_strcasecmp(str_layers, buf)) {
			fin >> _layers;
			if (debug > 2) g_print("  layers found: read = %d\n",_layers);		
		} else if (!g_strcasecmp(CSettings::str_diff, buf)) {
			fin >> _difficulty;
			if (debug > 0) g_print("  settings difficulty found: read = %d\n",
				_difficulty);		
		} else if (!g_strcasecmp(CSettings::str_width, buf)) {
			fin >> _w;
			if (debug > 2) g_print("  settings width found: read = %d\n",
				_w);		
		} else if (!g_strcasecmp(CSettings::str_height, buf)) {
			fin >> _h;
			if (debug > 2) g_print("  settings height found: read = %d\n",
				_h);		
		} else if (!g_strcasecmp(CSettings::str_depth, buf)) {
			fin >> _d;
			if (debug > 2) g_print("  settings depth found: read = %d\n",
				_d);		
		} else if (!g_strncasecmp(str_eog, buf, strlen(str_eog))) {
			if (debug > 2) g_print("  EOG found\n");		
			break; // exit while loop
		} else {
			if (debug) g_print("Read an invalid key : >%s<",buf);
			gchar *msg = g_strdup_printf(_("The file containing the Hall Of Fame is corrupted"));
			dialog_error_reading(fin, filename, msg);
			g_free(msg);
			return 5;
		}
		fin >> buf;
	}
	*pos = fin.tellg();
	return 0;
}

gint CHof::write_game(const gchar* filename, gboolean trunc_or_not_app)
{
	if (debug) g_print("CHof::write_game >%s< , %d\n",
		filename,trunc_or_not_app);

	ofstream fout;
	if (trunc_or_not_app) {
		fout.open(filename, ios::out | ios::trunc);
	} else {
		fout.open(filename, ios::out | ios::app);
	}
	g_assert(fout.is_open());
	// data
	fout << str_id << " " << _id << endl;
	if (!fout.good()) {
		if (debug) {
			g_print("CHof:write_game: something wrong:");
			if ((fout).eof()) g_print(" EOF");
			if ((fout).fail()) g_print(" FAIL");
			if ((fout).bad()) g_print(" BAD");
			g_print("; exit elvl 3\n");
		}
		dialog_error_writing(fout, filename, _("Writing operation failed"));
		return 3;			
	}
	if (_version != NULL) fout << str_version << " " << _version << endl;
	fout << CSettings::str_diff; 
	fout << " " << _difficulty << endl;
	fout << CSettings::str_depth  << " " << _d << endl;
	fout <<	CSettings::str_width  << " " << _w << endl;
	fout << CSettings::str_height << " " << _h << endl;
	fout <<	str_score  << " " << _score  << endl;
	fout <<	str_level  << " " << _level  << endl;
	fout <<	str_layers << " " << _layers << endl;	
	fout << str_timer  << " " << _timer  << endl;
	if (_date != NULL) fout << str_date << " " << _date << endl;
	if (_name != NULL) fout << str_name << " " << _name << endl;
	// end of record
	fout << str_eog << endl;
	if (fout.is_open()) fout.close();
	if (debug) g_print("CHof::write_game: exit elvl 0\n");
	return 0;
}

