/* gnommind: Master Mind game for GNOME
 * Copyright (C) 1999, 2000, 2001 Sebastien Nicoud <seb@eseb.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#ifndef _CHOF
#define _CHOF

#include "common.h"

#define CURRENT		3

class CHof
{
public:
	static const gchar *str_version;
	static const gchar *str_id;
	static const gchar *str_level;
	static const gchar *str_score;
	static const gchar *str_layers;
	static const gchar *str_timer;
	static const gchar *str_name;
	static const gchar *str_date;
	static const gchar *str_eog;

// attributes
private:
	long		 _pos;
public:
	gint		 _rank;
	gchar		 _version[MAX_HOF_S_LEN]; // gno3dtet version MM.mm.ff[.bb]
	gint32		 _id; // game id
	gint		 _difficulty;
	gint		 _d; // world depth
	gint		 _w; // world width
	gint		 _h; // world height
	gint32		 _score; // score
	gint		 _layers; // layers completed
	gint		 _level; // game level reached
	gint32		 _timer; // clock
	gchar		 _date[MAX_HOF_S_LEN]; // current date
	gchar		 _name[MAX_NAME_LEN]; // player name
	
// contructors
public:
	CHof(); // constructeur par defaut
	CHof(const CHof &ind); // constructeur de copy
	CHof(const CGame &game); // constructeur de copy
	
// destructor
public:	
	~CHof();

// operators
public:
	CHof &operator= (const CHof &ind); // affectation de recopie
	friend ostream &operator<< (ostream &os, CHof &ind); // output stream

// methods
public:
	gchar *compute_date(void);
	gboolean is_better_than(CHof *g) const;
	gboolean is_version_current(gchar *s= NULL) const;
	gint quantify_version(gchar *s = NULL) const;
	gchar *stringify_date(void) const;
	gchar *stringify_depth(void) const;
	gchar *stringify_difficulty(void) const;
	gchar *stringify_height(void) const;
	gchar *stringify_layers(void) const;
	gchar *stringify_level(void) const;
	gchar *stringify_name(void) const;
	gchar *stringify_rank(void) const;
	gchar *stringify_score(void) const;
	gchar *stringify_timer(void) const;
	gchar *stringify_width(void) const;

// i/o methods
public:
	gint read_game(const gchar* filename,
		gboolean pos_beginning_or_not_current = TRUE);
	gint write_game(const gchar* filename, gboolean trunc_or_not_app = TRUE);
private:
	void dialog_error_reading(ifstream &s, const gchar *name, gchar *m);
	void dialog_error_writing(ofstream &s, const gchar *name, gchar *m);
	gint read_at(long *pos, const gchar* filename);
};

#endif // _CHOF


