/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999, 2000, 2001 Sebastien Nicoud <seb@eseb.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

/* Indent TAB = 4
*/

#ifndef _COMMON
#define _COMMON

// includes
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <gnome.h>
#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include "config.h"

// general constants
#define GNO3DTET_NAME           "Gnome 3D Tetris"
#define GNO3DTET_AUTHOR		    "Sebastien Nicoud"
#define GNO3DTET_EMAIL		    "<seb@eseb.net>"
#define GNO3DTET_URL			"http://gno3dtet.eseb.net"
#define GNO3DTET_DATECREATED	"October 25 1999"
#define GNO3DTET_DATEMODIFIED	"May 2 2001"
	 // constants defined in config.h, created by configure
#define GNO3DTET_EXENAME 	    PACKAGE
#define GNO3DTET_VERSION		VERSION
	 // constants defined in the Makefile, passed with gcc -D :
	 // HOFDIR = 				"$(prefix)/var/games/"
	 // HOFFILE = 				"gno3dtet.hof"
#define GNO3DTET_HOFABSFILE 	HOFDIR HOFFILE
	 // constant defined in the Makefile, passed with gcc -D :
	 // SOUNDDIR = 				"$(prefix)/share/sounds/gno3dtet/"
#define LICENSE_FILE    		"copying.html"
#define SAVED_GAME				".gno3dtet.saved"
#define SAVED_SESSION			".gno3dtet.session"
#define CONFIG_PREFERENCES		"Preferences"
#define CONFIG_NHOFL			"Nb of lines in HOF"
#define CONFIG_SHOW_OLD_SCORE	"Show old scores in HOF"
#define CONFIG_DIFF				"Difficulty"
#define CONFIG_WIDTH			"World width"
#define CONFIG_HEIGHT			"World height"
#define CONFIG_DEPTH			"World depth"
#define CONFIG_SHOW_TOOLBAR		"Show toolbar"
#define CONFIG_SHOW_SCALE		"Show scale"
#define CONFIG_SHOW_SCORE		"Show score"
#define CONFIG_SHOW_STATS		"Show stats"
#define CONFIG_SHOW_NEXT_PIECE	"Show next piece"
#define CONFIG_KEY_XNEG			"Key for negative rotation on X axis"
#define CONFIG_KEY_XPOS			"Key for positive rotation on X axis"
#define CONFIG_KEY_YNEG			"Key for negative rotation on Y axis"
#define CONFIG_KEY_YPOS			"Key for positive rotation on Y axis"
#define CONFIG_KEY_ZNEG			"Key for negative rotation on Z axis"
#define CONFIG_KEY_ZPOS			"Key for positive rotation on Z axis"
#define CONFIG_KEY_LEFT			"Key for translation left"
#define CONFIG_KEY_RIGHT		"Key for translation right"
#define CONFIG_KEY_DOWN			"Key for translation down"
#define CONFIG_KEY_UP			"Key for translation up"
#define CONFIG_KEY_PAUSE		"Key for pause"
#define CONFIG_KEY_TOGGLENP		"Key for next piece"
#define CONFIG_KEY_DROP			"Key for piece drop"
#define CONFIG_SPEED			"Speed of moves"
#define CONFIG_WINDOWWIDTH		"Window width"
#define CONFIG_WINDOWHEIGHT		"Window height"
#define CONFIG_SOUNDSUPPORT		"Sound support"
#define CONFIG_PAUSE_LOSTFOCUS	"Pause on lost focus"
#define CONFIG_RESUME_FOCUS		"Resume on focus"

#define MAX_UINT64 G_GINT64_CONSTANT(0xFFFFFFFFFFFFFFFFU)
#define MAX_FILENAME_LENGTH		128

// application constants
#define MAIN_TIMER				30
#define CUBE_PARCOURU_EN_1K_MSEC 0.0005
#define INCREMENT_LEVEL_EVERY_N_LAYERS	4
#define	MIN_WORLD_WIDTH			3
#define DEF_WORLD_WIDTH			5
#define	MAX_WORLD_WIDTH			10
#define	MIN_WORLD_HEIGHT		3
#define DEF_WORLD_HEIGHT		5
#define	MAX_WORLD_HEIGHT		10
#define	MIN_WORLD_DEPTH			5
#define DEF_WORLD_DEPTH			10
#define	MAX_WORLD_DEPTH			20
#define DEF_DIFFICULTY			2
#define	DEF_SPEED				1
#define MAX_CUBES_IN_PIECE		5
#define MIN_NHOFL				1
#define MAX_NHOFL 				50
#define DFLT_NHOFL				10
#define MAX_NAME_LEN			32
#define MAX_HOF_S_LEN			32
#define SCALE_WIDTH				20
#define STATS_WIDTH				30
#define NB_SCORE_DIGITS			9
#define NB_LEVEL_DIGITS			3
#define NB_LAYERS_DIGITS		4
#define NB_BLOCS_DIGITS			5
#define NB_CUBES_DIGITS			6
#define	BIG_TEXT_HEIGHT			12
#define SMALL_TEXT_HEIGHT		7
#define SCORE_TEXT_WIDTH		41
#define	LEVEL_TEXT_WIDTH		41
#define	STATS_TEXT_WIDTH		25
#define	BIG_DIGIT_HEIGHT		22
#define	BIG_DIGIT_WIDTH			12
#define	BIG_COMMA_WIDTH			4
#define SMALL_DIGIT_HEIGHT		13
#define SMALL_DIGIT_WIDTH		7
#define	SMALL_COMMA_WIDTH		3
#define	DFLT_KEY_BINDING		"aqswde4628 pn"
#define	DFLT_KEY_BINDING_FR		"qaszde4628 pn"
#define DFLT_WINDOWWIDTH		400
#define DFLT_WINDOWHEIGHT		460

// debug stuff
#define TRACE { if (debug) printf("trace %s @ %u\n",__FILE__,__LINE__); }
#define DBG_CB					1

// classes
class CSettings;
class CWSettings;
class CHof;
class CWHof;
class CPiece;
class CGame;
class CWMain;
class CSound;
enum key {
		key_XNEG,
		key_XPOS,
		key_YNEG,
		key_YPOS,
		key_ZNEG,
		key_ZPOS,
		key_LEFT,
		key_RIGHT,
		key_DOWN,
		key_UP,
		key_DROP,
		key_PAUSE,
		key_TOGGLENP,
		key_NONE // used to create key arrays
};

// common global vars
extern gint debug; // debug level
extern CGame *mds; // main data structure
extern CWMain *mwin; // main window
extern CSettings *settings; // official settings = saved settings
extern CWSettings *wsets; // preferences window
extern CWHof *whof; // hall of fame window
extern CSound sounds[]; // sound database

// common functions

#endif //_COMMON
