/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999, 2000, 2001 Sebastien Nicoud <seb@eseb.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#include "common.h"
#include "cpiece.h"

CPiece::CPiece()
{
	if (debug > 1) g_print("CPiece::CPiece()\n");
	_pd = NULL;
	_class = 0;
	_dx = 0;
	_dy = 0;
	_dz = 0;
	_nb_cubes = 0;
	_cubes = NULL;
	_nb_pointes = 0;
	_pointes = NULL;
	_nb_arretes = 0;
	_arretes = NULL;
	_tx = 0.0;
	_ty = 0.0;
	_tz = 0.0;
}

CPiece::CPiece(const CPiece &ind)
{
	if (debug) g_print("CPiece::CPiece(CPiece & = >%s<)\n",ind._pd);
	free_memory();
	if (ind._pd) _pd = g_strdup(ind._pd);
	else _pd = NULL;
	_class = ind._class;
	_dx = ind._dx;
	_dy = ind._dy;
	_dz = ind._dz;
	_tx = ind._tx;
	_ty = ind._ty;
	_tz = ind._tz;
	gint i;
	
	_nb_cubes = ind._nb_cubes;
	_cubes = g_new0(t_triplet*,_nb_cubes);
	for (i=0; i<_nb_cubes; i++) {
		_cubes[i] = g_new0(t_triplet,1);
		_cubes[i]->x = ind._cubes[i]->x;
		_cubes[i]->y = ind._cubes[i]->y;
		_cubes[i]->z = ind._cubes[i]->z;
	}
	_nb_pointes = ind._nb_pointes;
	_pointes = g_new0(t_triplet*,_nb_pointes);
	for (i=0; i<_nb_pointes; i++) {
		_pointes[i] = g_new0(t_triplet,1);
		_pointes[i]->x = ind._pointes[i]->x;
		_pointes[i]->y = ind._pointes[i]->y;
		_pointes[i]->z = ind._pointes[i]->z;
	}
	_nb_arretes = ind._nb_arretes;
	_arretes = g_new0(t_arrete*,_nb_arretes);
	for (i=0; i<_nb_arretes; i++) {
		_arretes[i] = g_new0(t_arrete,1);
		_arretes[i]->a = ind._arretes[i]->a;
		_arretes[i]->b = ind._arretes[i]->b;
	}
}

CPiece::~CPiece()
{
	if (debug > 1) g_print("CPiece::~CPiece\n");
	free_memory();
}

CPiece &CPiece::operator= (const CPiece &ind)
{
	if (debug > 2) g_print("CPiece::operator= pd=>%s<\n",ind._pd);
	free_memory();
	if (ind._pd) _pd = g_strdup(ind._pd);
	else _pd = NULL;
	_class = ind._class;
	_dx = ind._dx;
	_dy = ind._dy;
	_dz = ind._dz;
	_tx = ind._tx;
	_ty = ind._ty;
	_tz = ind._tz;
	gint i;
	
	_nb_cubes = ind._nb_cubes;
	_cubes = g_new0(t_triplet*,_nb_cubes);
	for (i=0; i<_nb_cubes; i++) {
		_cubes[i] = g_new0(t_triplet,1);
		_cubes[i]->x = ind._cubes[i]->x;
		_cubes[i]->y = ind._cubes[i]->y;
		_cubes[i]->z = ind._cubes[i]->z;
	}
	_nb_pointes = ind._nb_pointes;
	_pointes = g_new0(t_triplet*,_nb_pointes);
	for (i=0; i<_nb_pointes; i++) {
		_pointes[i] = g_new0(t_triplet,1);
		_pointes[i]->x = ind._pointes[i]->x;
		_pointes[i]->y = ind._pointes[i]->y;
		_pointes[i]->z = ind._pointes[i]->z;
	}
	_nb_arretes = ind._nb_arretes;
	_arretes = g_new0(t_arrete*,_nb_arretes);
	for (i=0; i<_nb_arretes; i++) {
		_arretes[i] = g_new0(t_arrete,1);
		_arretes[i]->a = ind._arretes[i]->a;
		_arretes[i]->b = ind._arretes[i]->b;
	}
	return *this;
}

ostream &operator<< (ostream &os, CPiece &ind)
{
//	if (debug > 2) g_print("CPiece::operator<<\n");
	gint i = 0;
	os << "piece: pd=";
	if (ind._pd == NULL) os << "(null)"; else os << ind._pd;
	os << ", class " << ind._class;
	os << endl;
	os << "  maxes x=" << ind._dx << ", y=" << ind._dy
		 << ", z=" << ind._dz << endl; 
	os << "  tx=" << ind._tx << ", ty=" 
		 << ind._ty << ", tz=" << ind._tz << endl;
	os << "  cubes n=" << ind._nb_cubes << " :";
	gint ncubes = ind._nb_cubes;	
	if (ind._cubes == NULL) {
		os << "(null)" << endl;
	} else {
		if (ncubes > 0 && ncubes <= CPIECE_MAX_CUBES) {
			for (i=0; i<ncubes; i++) {
				os << " " << i << "(";
				if (ind._cubes[i] == NULL) os << "null)";
				else os << ind._cubes[i]->x << "," << ind._cubes[i]->y
					 	  << "," << ind._cubes[i]->z << ")";
			}
			os << endl;
		} else {
			os << " *" << endl;
	}	}

	os << "  pointes n=" << ind._nb_pointes << " :";
	gint nptes = ind._nb_pointes;
	if (ind._pointes == NULL) {
		os << "(null)" << endl;
	} else {
		if (nptes > 0 && nptes <= ncubes*8 && nptes <= CPIECE_MAX_CUBES*8) {
			for (i=0; i<nptes; i++) {
				os << " " << i << "(";
				if (ind._pointes[i] == NULL) os << "null)";
				else os << ind._pointes[i]->x << "," << ind._pointes[i]->y
					 	  << "," << ind._pointes[i]->z << ")";
			}
			os << endl;
		} else {
			os << " *" << endl;
	}	}
	
	os << "  edges n=" << ind._nb_arretes << " :";
	gint narr = ind._nb_arretes;
	if (ind._arretes == NULL) {
		os << "(null)" << endl;
	} else {
		if (narr > 0 && narr <= ncubes*12 && narr <= CPIECE_MAX_CUBES*12) {
			for (i=0; i<narr; i++) {
				os << " " << i << "(";
				if (ind._arretes[i] == NULL) os << "null)";
				else os << ind._arretes[i]->a
						  << "," << ind._arretes[i]->b << ")";
			}
			os << endl;
		} else {
			os << " *" << endl;
	}	}
	return os;
}

void CPiece::center_piece_to_weight_center(void)
{
	if (debug > 1) g_print("CPiece::center_piece_to_weight_center\n");	
	gfloat x, y, z;
	compute_offsets_to_weight_center(&x, &y, &z);
	_tx -= x;
	_ty -= y;
	_tz -= z;
	apply_translations();
}

void CPiece::compute_offsets_to_weight_center(gfloat *x, gfloat *y, gfloat *z)
{
	if (debug > 1) g_print("CPiece::compute_offsets_to_weight_center\n");	
	gint i;
	gfloat somme = 0;
	for (i=0; i<_nb_cubes; i++)	somme += _cubes[i]->x;
	*x = somme/_nb_cubes;
	somme = 0;
	for (i=0; i<_nb_cubes; i++)	somme += _cubes[i]->y;
	*y = somme/_nb_cubes;
	somme = 0;
	for (i=0; i<_nb_cubes; i++)	somme += _cubes[i]->z;
	*z = somme/_nb_cubes;
	if (debug) g_print("  dx=%.2f, dy=%.2f, dz=%.2f\n", *x,*y,*z);
}

/* create
 * create a piece according to the piece descriptor (dp)
 * the first digit is the piece difficulty
 * RET: piece difficulty
*/
gint CPiece::create(const gchar *pd)
{
	if (debug > 2) g_print("CPiece::create, pd = >%s<\n",pd);	
	if (pd == NULL) return 0;
	_pd = g_strdup(pd);
	_tx = 0.0;
	_ty = 0.0;
	_tz = 0.0;
	
	gint i,j,k;
	t_3darray allcubes;
	t_3darray allpointes;
	t_3darray allpointesindex;
	// init temp tables
	for (i=0; i<CPIECE_MAX_CUBES+1; i++)
		for (j=0; j<CPIECE_MAX_CUBES+1; j++)
			for (k=0; k<CPIECE_MAX_CUBES+1; k++) {
				allcubes[i][j][k] = 0; // actually only 0->MAX_CUBES size needed
				allpointes[i][j][k] = 0;
				allpointesindex[i][j][k] = -1;
	}	
	// create first cube, located at (0,0,0)
	_nb_cubes = 0;
	piece_tables_update(allcubes, allpointes, 0,0,0);

	// parse the piece descriptor
	gint pdl = strlen(pd);
	gchar c = 0;
	gint x = 0; _dx = 0;
	gint y = 0; _dy = 0;
	gint z = 0; _dz = 0;
	_class = pd[0] - '1' + 1;
	for (gint ipdl=1; ipdl<pdl; ipdl++) {
		c = pd[ipdl];
		switch (c) {
			case 'r': x++;
			if (x > _dx) _dx = x;
			break;
			case 'l': x--;
			if (x < 0) {
				x = 0;
				piece_tables_deplace(allcubes, allpointes, 'x');
			}
			break;
			case 'u': y++;
			if (y > _dy) _dy = y;
			break;
			case 'd': y--;
			if (y < 0) {
				y = 0;
				piece_tables_deplace(allcubes, allpointes, 'y');
			}
			break;
			case 'f': z++;
			if (z > _dz) _dz = z;
			break;
			case 'b': z--;
			if (z < 0) {
				z = 0;
				piece_tables_deplace(allcubes, allpointes, 'z');
			}
			break;
		} // switch c
		piece_tables_update(allcubes, allpointes, x, y, z);
	}
	// compute offset of the 3d center of the piece
	_dx++;
	_dy++;
	_dz++;
	// dX	 1	 2	 3	 4	 5	 6
	// ddX	-.5	-.5	-1.	-1.	-1.5 -1.5
	// cddX	 0	 0	-1	-1	-2	-2
	gint cddx = (gint)(-div(_dx-1,2).quot);
	gint cddy = (gint)(-div(_dy-1,2).quot);
	gint cddz = (gint)(-div(_dz-1,2).quot);
	gfloat ddx = cddx - 0.5;
	gfloat ddy = cddy - 0.5;
	gfloat ddz = cddz - 0.5;
	if (debug > 2)
		g_print("  dx=%d, ddx=%.2f, dx=%d, ddx=%.2f, dx=%d, ddx=%.2f\n",
			_dx, ddx, _dy, ddy, _dz, ddz);
	
	// compute _cubes
	g_assert(_nb_cubes > 0);
	g_assert(_nb_cubes < CPIECE_MAX_CUBES+1);
	_cubes = g_new0(t_triplet*,_nb_cubes); // alloc array of ptrs
	for (i=0; i<_nb_cubes; i++) _cubes[i] = NULL;
	gint ic = 0;
	for (i=0; i<_dx; i++)
		for (j=0; j<_dy; j++)
			for (k=0; k<_dz; k++)
				if (allcubes[i][j][k]) {
					_cubes[ic] = g_new0(t_triplet,1);
					_cubes[ic]->x = i+cddx;
					_cubes[ic]->y = j+cddy;
					_cubes[ic]->z = k+cddz;
					ic++;
	}
	if (debug > 1) g_print("CPiece::create, found %d cubes\n",_nb_cubes);

	// get number of visible pointes
	_nb_pointes = 0;
	for (i=0; i<_dx+1; i++)
		for (j=0; j<_dy+1; j++)
			for (k=0; k<_dz+1; k++)
				if (allpointes[i][j][k] % 2) {
					 allpointes[i][j][k] = 1;
					 _nb_pointes++;
				} else allpointes[i][j][k] = 0;
	if (debug > 1) g_print("CPiece::create, found %d pointes\n",_nb_pointes);

	// compute pointes
	_pointes = g_new0(t_triplet*,_nb_pointes); // alloc array of ptrs
	for (i=0; i<_nb_pointes; i++) _pointes[i] = NULL;
	gint ip = 0;
	for (i=0; i<_dx+1; i++)
		for (j=0; j<_dy+1; j++)
			for (k=0; k<_dz+1; k++) 
				if (allpointes[i][j][k]) {
					_pointes[ip] = g_new0(t_triplet,1);
					_pointes[ip]->x = i+ddx;
					_pointes[ip]->y = j+ddy;
					_pointes[ip]->z = k+ddz;
					allpointesindex[i][j][k] = ip;
					ip++;
					// if (debug > 2) g_print("CPiece::create, found pointe %d=
					//	(%d,%d,%d)\n",ip-1,i,j,k);
	}
		
	// get number of arretes
	_nb_arretes = 0;
	gint nb_on_this_line = 0;
	// x-axed edges
	for (j=0; j<_dy+1; j++)
		for (k=0; k<_dz+1; k++) {
			nb_on_this_line = 0;
			for (i=0; i<_dx+1; i++)
				if (allpointes[i][j][k]) nb_on_this_line++;
			// the nb of pointes on the line must be 2*n
			if (nb_on_this_line % 2) g_print("**ERROR**: CPiece::create nb of \
pointe on X line (y=%d,z=%d) = %d\n",j,k,nb_on_this_line);
			_nb_arretes += nb_on_this_line/2;
	}
	// y-axed edges
	for (i=0; i<_dx+1; i++)
		for (k=0; k<_dz+1; k++) {
			nb_on_this_line = 0;
			for (j=0; j<_dy+1; j++)
				if (allpointes[i][j][k]) nb_on_this_line++;
			// the nb of pointes on the line must be 2*n
			if (nb_on_this_line % 2) g_print("**ERROR**: CPiece::create nb of \
pointe on Y line (x=%d,z=%d) = %d\n",i,k,nb_on_this_line);
			_nb_arretes += nb_on_this_line/2;
	}
	// x-axed edges
	for (i=0; i<_dx+1; i++)
		for (j=0; j<_dy+1; j++) {
			nb_on_this_line = 0;
			for (k=0; k<_dz+1; k++)
				if (allpointes[i][j][k]) nb_on_this_line++;
			// the nb of pointes on the line must be 2*n
			if (nb_on_this_line % 2) g_print("**ERROR**: CPiece::create nb of \
pointe on Z line (x=%d,y=%d) = %d\n",i,j,nb_on_this_line);
			_nb_arretes += nb_on_this_line/2;
	}
	if (debug > 1) g_print("CPiece::create, found %d arretes\n",_nb_arretes);
	
	// compute arretes
	_arretes = g_new0(t_arrete*, _nb_arretes);
	for (i=0; i<_nb_arretes; i++) _arretes[i] = NULL;
	gint ia = -1;
	gboolean a_or_not_b = TRUE;
	// x-axed edges
	for (j=0; j<CPIECE_MAX_CUBES+1; j++)
		for (k=0; k<CPIECE_MAX_CUBES+1; k++) {
			a_or_not_b = TRUE; // a first
			for (i=0; i<CPIECE_MAX_CUBES+1; i++)
				if (allpointes[i][j][k]) {
					// pointe found
					if (a_or_not_b) {
						ia++;
						_arretes[ia] = g_new0(t_arrete, 1);
					}
					g_assert(allpointesindex[i][j][k] != -1);
					if (a_or_not_b)	_arretes[ia]->a = allpointesindex[i][j][k];
					else _arretes[ia]->b = allpointesindex[i][j][k];
					a_or_not_b = !a_or_not_b;
	}		}
	// y-axed edges
	for (i=0; i<CPIECE_MAX_CUBES+1; i++)
		for (k=0; k<CPIECE_MAX_CUBES+1; k++) {
			a_or_not_b = TRUE; // a first
			for (j=0; j<CPIECE_MAX_CUBES+1; j++)
				if (allpointes[i][j][k]) {
					// pointe found
					if (a_or_not_b) {
						ia++;
						_arretes[ia] = g_new0(t_arrete, 1);
					}
					g_assert(allpointesindex[i][j][k] != -1);
					if (a_or_not_b)	_arretes[ia]->a = allpointesindex[i][j][k];
					else _arretes[ia]->b = allpointesindex[i][j][k];
					a_or_not_b = !a_or_not_b;
	}		}
	// z-axed edges
	for (i=0; i<CPIECE_MAX_CUBES+1; i++)
		for (j=0; j<CPIECE_MAX_CUBES+1; j++) {
			a_or_not_b = TRUE; // a first
			for (k=0; k<CPIECE_MAX_CUBES+1; k++)
				if (allpointes[i][j][k]) {
					// pointe found
					if (a_or_not_b) {
						ia++;
						_arretes[ia] = g_new0(t_arrete, 1);
					}
					g_assert(allpointesindex[i][j][k] != -1);
					if (a_or_not_b)	_arretes[ia]->a = allpointesindex[i][j][k];
					else _arretes[ia]->b = allpointesindex[i][j][k];
					a_or_not_b = !a_or_not_b;
	}		}	
	if (debug > 2) cout << *this << endl;
	return (pd[0]-'0');
}

void CPiece::free_memory(void)
{
	if (debug > 1) {
		g_print("CPiece::free_memory for >");
		if (_pd) g_print("%s<\n",_pd); else g_print("(null)<\n");
	}
	if (_pd) {
		g_free(_pd);
		_pd = NULL;
	}
	gint i;
	if (_cubes) {
		for (i=0; i<_nb_cubes; i++) if (_cubes[i]) {
			g_free(_cubes[i]);
			_cubes[i] = NULL;
		}
		g_free(_cubes);
		_cubes = NULL;
	}
	_nb_cubes = 0;
	if (_pointes) {
		for (i=0; i<_nb_pointes; i++) if (_pointes[i]) {
			g_free(_pointes[i]);
			_pointes[i] = NULL;
		}
		g_free(_pointes);
		_pointes = NULL;
	}
	_nb_pointes = 0;
	if (_arretes) {
		for (i=0; i<_nb_arretes; i++) if (_arretes[i]) {
			g_free(_arretes[i]);
			_arretes[i] = NULL;
		}
		g_free(_arretes);
		_arretes = NULL;
	}
	_nb_arretes = 0;
}

void CPiece::piece_tables_deplace(t_3darray &cubes, t_3darray &pointes,
	gchar to_where)
{
	if (debug > 2) g_print("CPiece::piece_tables_deplace to %c\n",to_where);
	gint ix, iy, iz;
	switch (to_where) {
		case 'x':
		for (ix=CPIECE_MAX_CUBES; ix>0; ix--)
			for (iy = 0; iy < CPIECE_MAX_CUBES+1; iy++)
				for (iz = 0; iz < CPIECE_MAX_CUBES+1; iz++) {
					cubes[ix][iy][iz] = cubes[ix-1][iy][iz];
					pointes[ix][iy][iz] = pointes[ix-1][iy][iz];
		}
		for (iy = 0; iy < CPIECE_MAX_CUBES+1; iy++)
			for (iz = 0; iz < CPIECE_MAX_CUBES+1; iz++) {
				cubes[0][iy][iz] = 0;
				pointes[0][iy][iz] = 0;
		}
		_dx++;
		break;
		case 'y':
		for (iy=CPIECE_MAX_CUBES; iy>0; iy--)
			for (ix = 0; ix < CPIECE_MAX_CUBES+1; ix++)
				for (iz = 0; iz < CPIECE_MAX_CUBES+1; iz++) {
					cubes[ix][iy][iz] = cubes[ix][iy-1][iz];
					pointes[ix][iy][iz] = pointes[ix][iy-1][iz];
		}
		for (ix = 0; ix < CPIECE_MAX_CUBES+1; ix++)
			for (iz = 0; iz < CPIECE_MAX_CUBES+1; iz++) {
				cubes[ix][0][iz] = 0;
				pointes[ix][0][iz] = 0;
		}
		_dy++;
		break;
		case 'z':
		for (iz=CPIECE_MAX_CUBES; iz>0; iz--)
			for (ix = 0; ix < CPIECE_MAX_CUBES+1; ix++)
				for (iy = 0; iy < CPIECE_MAX_CUBES+1; iy++) {
					cubes[ix][iy][iz] = cubes[ix][iy][iz-1];
					pointes[ix][iy][iz] = pointes[ix][iy][iz-1];
		}
		for (ix = 0; ix < CPIECE_MAX_CUBES+1; ix++)
			for (iy = 0; iy < CPIECE_MAX_CUBES+1; iy++) {
				cubes[ix][iy][0] = 0;
				pointes[ix][iy][0] = 0;
		}
		_dz++;
		break;
		default:
		g_print("ERROR CPiece::piece_tables_deplace: to_where = %c\n",to_where);
	}
}

void CPiece::piece_tables_update(t_3darray &cubes, t_3darray &pointes,
	gint x, gint y, gint z)
{
	if (debug > 2) g_print("CPiece::piece_tables_update (%d,%d,%d), \
cube nb=%d\n",x,y,z,_nb_cubes);
	g_assert(x+1 < CPIECE_MAX_CUBES+1);
	g_assert(y+1 < CPIECE_MAX_CUBES+1);
	g_assert(z+1 < CPIECE_MAX_CUBES+1);
	if (cubes[x][y][z]) return;
	_nb_cubes++;
	cubes[x][y][z]++;
	pointes[x][y][z]++;
	pointes[x+1][y][z]++;
	pointes[x][y+1][z]++;
	pointes[x][y][z+1]++;
	pointes[x+1][y+1][z]++;
	pointes[x+1][y][z+1]++;
	pointes[x][y+1][z+1]++;
	pointes[x+1][y+1][z+1]++;
}

// x : y' = y cos a - z sin a
//     z' = z cos a + y sin a
void CPiece::rotate_x(gfloat dt, CPiece *p2, 
	gboolean do_pointes, gboolean do_cubes)
{
	if (debug > 2) g_print("CPiece::rotate_x %.2f\n",dt);
	gint i;
	gboolean bp2 = p2 != NULL;
	gfloat ty, tz;
	gboolean final = FALSE;
	if (dt == 1.0 || dt == -1.0) final = TRUE;
	// transform pointes
	if (do_pointes) {
		gfloat gap = dt * HALF_OF_PI;
		gfloat cosap = cos(gap);
		gfloat sinap = sin(gap);
		for (i=0; i<_nb_pointes; i++) {
			ty = _pointes[i]->y;
			tz = _pointes[i]->z;
			if (final) {
				_pointes[i]->y = -tz*dt; 
				_pointes[i]->z = ty*dt;
				//ROUND05(_pointes[i];
				if (bp2) {
					p2->_pointes[i]->y = -tz*dt;
					p2->_pointes[i]->z = ty*dt;
				}				
			} else {
				_pointes[i]->y = ty*cosap - tz*sinap; 
				_pointes[i]->z = tz*cosap + ty*sinap; 
	}	}	}
	// transform cubes if transfo is complete
	if (final && do_cubes) for (i=0; i<_nb_cubes; i++) {
		ty = _cubes[i]->y;
		tz = _cubes[i]->z;
		_cubes[i]->y = -tz*dt;
		_cubes[i]->z = ty*dt;
		if (bp2) {
			p2->_cubes[i]->y = -tz*dt;
			p2->_cubes[i]->z = ty*dt;
	}	}
}

// y : x' = x cos a - z sin a
//     z' = z cos a + x sin a
void CPiece::rotate_y(gfloat dt, CPiece *p2,
	gboolean do_pointes, gboolean do_cubes)
{
	if (debug > 2) g_print("CPiece::rotate_y %.2f\n",dt);
	gint i;
	gboolean bp2 = p2 != NULL;
	gfloat tx, tz;
	gboolean final = FALSE;
	if (dt == 1.0 || dt == -1.0) final = TRUE;
	if (do_pointes) {
		gfloat gap = dt * HALF_OF_PI;
		gfloat cosap = cos(gap);
		gfloat sinap = sin(gap);
		for (i=0; i<_nb_pointes; i++) {
			tx = _pointes[i]->x;
			tz = _pointes[i]->z;
			if (final) {
				_pointes[i]->x = -tz*dt; 
				_pointes[i]->z = tx*dt;
				//ROUND05(_pointes[i];
				if (bp2) {
					p2->_pointes[i]->x = -tz*dt;
					p2->_pointes[i]->z = tx*dt;
				}				
			} else {
				_pointes[i]->x = tx*cosap - tz*sinap; 
				_pointes[i]->z = tz*cosap + tx*sinap; 
	}	}	}
	// transform cubes if transfo is complete
	if (final && do_cubes) for (i=0; i<_nb_cubes; i++) {
		tx = _cubes[i]->x;
		tz = _cubes[i]->z;
		_cubes[i]->x = -tz*dt;
		_cubes[i]->z = tx*dt;
		if (bp2) {
			p2->_cubes[i]->x = -tz*dt;
			p2->_cubes[i]->z = tx*dt;
	}	}
}

// z : x' = x cos a + y sin a
//     y' = y cos a - x sin a
void CPiece::rotate_z(gfloat dt, CPiece *p2,
	gboolean do_pointes, gboolean do_cubes)
{
	if (debug > 2) g_print("CPiece::rotate_z %.2f\n",dt);
	gint i;
	gboolean bp2 = p2 != NULL;
	gfloat tx, ty;
	gboolean final = FALSE;
	if (dt == 1.0 || dt == -1.0) final = TRUE;
	if (do_pointes) {
		gfloat gap = dt * HALF_OF_PI;
		gfloat cosap = cos(gap);
		gfloat sinap = sin(gap);
		for (i=0; i<_nb_pointes; i++) {
			tx = _pointes[i]->x;
			ty = _pointes[i]->y;
			if (final) {
				_pointes[i]->x = ty*dt; 
				_pointes[i]->y = -tx*dt;
				//ROUND05(_pointes[i];
				if (bp2) {
					p2->_pointes[i]->x = ty*dt;
					p2->_pointes[i]->y = -tx*dt;
				}				
			} else {
				_pointes[i]->x = tx*cosap + ty*sinap; 
				_pointes[i]->y = ty*cosap - tx*sinap; 
	}	}	}
	// transform cubes if transfo is complete
	if (final && do_cubes) for (i=0; i<_nb_cubes; i++) {
		tx = _cubes[i]->x;
		ty = _cubes[i]->y;
		_cubes[i]->x = ty*dt;
		_cubes[i]->y = -tx*dt;
		if (bp2) {
			p2->_cubes[i]->x = ty*dt;
			p2->_cubes[i]->y = -tx*dt;
	}	}
}

void CPiece::apply_translations(gboolean do_pointes, gboolean do_cubes)
{
	if (debug > 2) g_print("CPiece::apply_translations\n");
	if (do_pointes) for (gint i=0; i<_nb_pointes; i++) {
		_pointes[i]->x += _tx;
		_pointes[i]->y += _ty;
		_pointes[i]->z += _tz;
	}
	if (do_cubes) for (gint i=0; i<_nb_cubes; i++) {
		_cubes[i]->x += _tx;
		_cubes[i]->y += _ty;
		_cubes[i]->z += _tz;
}	}

void CPiece::translate_x(gfloat dt,	CPiece *p2,
	gboolean do_pointes, gboolean do_cubes)
{
	if (debug > 2) g_print("CPiece::translate_x %.2f\n",dt);
	gboolean final = FALSE;
	if (dt == 1.0 || dt == -1.0) final = TRUE;
	gboolean bp2 = p2 != NULL;
	if (final) {
		_tx += dt;
		if (bp2) p2->_tx += dt;
	}
	if (!final && do_pointes)
		for (gint i=0; i<_nb_pointes; i++) _pointes[i]->x += dt;
}	

void CPiece::translate_y(gfloat dt,	CPiece *p2,
	gboolean do_pointes, gboolean do_cubes)
{
	if (debug > 2) g_print("CPiece::translate_y %.2f\n",dt);
	gboolean final = FALSE;
	if (dt == 1.0 || dt == -1.0) final = TRUE;
	gboolean bp2 = p2 != NULL;
	if (final) {
		_ty += dt;
		if (bp2) p2->_ty += dt;
	}
	if (!final && do_pointes)
		for (gint i=0; i<_nb_pointes; i++) _pointes[i]->y += dt;
}	

void CPiece::translate_z(gfloat dt,	CPiece *p2,
	gboolean do_pointes, gboolean do_cubes)
{
	if (debug > 2) g_print("CPiece::translate_z %.2f\n",dt);
	gboolean final = FALSE;
	if (dt == 1.0 || dt == -1.0) final = TRUE;
	gboolean bp2 = p2 != NULL;
	if (final) {
		_tz += dt;
		if (bp2) p2->_tz += dt;
	}
	if (!final && do_pointes)
		for (gint i=0; i<_nb_pointes; i++) _pointes[i]->z += dt;
}	
