/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999, 2000, 2001 Sebastien Nicoud <seb@eseb.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#ifndef _CPIECE
#define _CPIECE

#include "common.h"

#define CPIECE_MAX_CUBES MAX_CUBES_IN_PIECE
#define ROUND05(x)	((gfloat)(((gint)(x*2))/2.0))
#define HALF_OF_PI	1.57079633

typedef struct _triplet {
	gfloat x;
	gfloat y;
	gfloat z;
} t_triplet;

typedef struct _arrete {
	gint a;
	gint b;
} t_arrete;

typedef gint t_3darray[CPIECE_MAX_CUBES+1][CPIECE_MAX_CUBES+1]
						[CPIECE_MAX_CUBES+1];

class CPiece
{
// typedefs, consts
public:

// attributes
public:
	gchar *_pd;
	gint _class; // 1,2,3 = _pd[0] value
	gint _nb_cubes;
	t_triplet **_cubes; // array of ptrs to t_triplet
	gint _nb_arretes;
	t_arrete **_arretes;
	gint _dx;
	gint _dy;
	gint _dz;
	gint _nb_pointes;
	t_triplet **_pointes;
	gfloat _tx;
	gfloat _ty;
	gfloat _tz;
	
// contructor
public:
	CPiece(); // constructeur par defaut
	CPiece(const CPiece &ind); // constructeur de copy
	
// destructor
	~CPiece();

// operators
public:
	CPiece &operator= (const CPiece &ind); // affectation de recopie
	friend ostream &operator<< (ostream &os, CPiece &ind); // output stream

// methods
public:
	void center_piece_to_weight_center(void);
	void compute_offsets_to_weight_center(gfloat *x, gfloat *y, gfloat *z);
	gint create(const gchar *pd);
	void free_memory(void);
	void rotate_x(gfloat dt, CPiece *p2,
			gboolean do_pointes = TRUE, gboolean do_cubes = TRUE);
	void rotate_y(gfloat dt, CPiece *p2,
			gboolean do_pointes = TRUE, gboolean do_cubes = TRUE);
	void rotate_z(gfloat dt, CPiece *p2,
			gboolean do_pointes = TRUE, gboolean do_cubes = TRUE);
	void apply_translations(
			gboolean do_pointes = TRUE, gboolean do_cubes = TRUE);
	void translate_x(gfloat dt, CPiece *p2,
			gboolean do_pointes = TRUE, gboolean do_cubes = TRUE);
	void translate_y(gfloat dt, CPiece *p2,
			gboolean do_pointes = TRUE, gboolean do_cubes = TRUE);
	void translate_z(gfloat dt, CPiece *p2,
			gboolean do_pointes = TRUE, gboolean do_cubes = TRUE);
	
private:
	void piece_tables_deplace(t_3darray &cubes, t_3darray &pointes,
		gchar to_where);
	void piece_tables_update(t_3darray &cubes, t_3darray &pointes,
		gint x, gint y, gint z);
};

#endif // _CPIECE
