/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999, 2000, 2001 Sebastien Nicoud <seb@eseb.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#include "common.h"
#include "csettings.h"

const gchar *CSettings::str_width  = "width";
const gchar *CSettings::str_height = "height";
const gchar *CSettings::str_depth  = "depth";
const gchar *CSettings::str_diff   = "difficulty";

const gchar *CSettings::diff_descriptors[5] = {
	"easiest", // unused
	"Easy",
	"Medium",
	"Hard",
	"hardest" // unused
};

const gchar *CSettings::speed_descriptors[5] = {
	"slowest", //unused
	"Normal ",
	" Fast  ",
	"Instant",
	"fastest" // unused
};
	
CSettings::CSettings()
{
	if (debug) g_print("CSettings::CSettings()\n");
	_w = DEF_WORLD_WIDTH;
	_h = DEF_WORLD_HEIGHT;
	_d = DEF_WORLD_DEPTH;
	compute_total_cubes();
	_difficulty = diff_MEDIUM;
	gchar *dflt_key_string = DFLT_KEY_BINDING;
	for (gint i=0; i<key_NONE; i++) _keys[i] = dflt_key_string[i];
	_speed = speed_NORMAL;
	_show_np    = TRUE;
	_show_scale = TRUE;	
	_show_score = TRUE;
	_show_stats = FALSE;
	_sound_support = TRUE;
	_pause_on_nofocus = FALSE;
	_resume_on_focus = FALSE;
	_nhofl = DFLT_NHOFL;
	_show_old_score = TRUE;
}

CSettings::CSettings(const CSettings &ind)
{
	if (debug) g_print("CSettings::CSettings(CSettings &)\n");
	_w = ind._w;
	_h = ind._h;
	_d = ind._d;
	compute_total_cubes();
	_difficulty = ind._difficulty;
	for (gint i=0; i<key_NONE; i++) _keys[i] = ind._keys[i];
	_speed = ind._speed;
	_show_np = ind._show_np;
	_show_scale = ind._show_scale;	
	_show_score = ind._show_score;
	_show_stats = ind._show_stats;
	_sound_support = ind._sound_support;
	_pause_on_nofocus = ind._pause_on_nofocus;
	_resume_on_focus = ind._resume_on_focus;
	_nhofl = ind._nhofl;
	_show_old_score = ind._show_old_score;
}

CSettings::~CSettings()
{
	if (debug) g_print("CSettings::~CSettings\n");
}

CSettings &CSettings::operator= (const CSettings &ind)
{
	if (debug > 2) g_print("CSettings::operator=\n");
	_w = ind._w;
	_h = ind._h;
	_d = ind._d;
	compute_total_cubes();
	_difficulty = ind._difficulty;
	for (gint i=0; i<key_NONE; i++) _keys[i] = ind._keys[i];
	_speed = ind._speed;
	_show_np = ind._show_np;
	_show_scale = ind._show_scale;	
	_show_score = ind._show_score;
	_show_stats = ind._show_stats;
	_sound_support = ind._sound_support;
	_pause_on_nofocus = ind._pause_on_nofocus;
	_resume_on_focus = ind._resume_on_focus;
	_nhofl = ind._nhofl;
	_show_old_score = ind._show_old_score;
	return *this;
}

ostream &operator<< (ostream &os, CSettings &ind)
{
	if (debug > 2) g_print("CSettings::operator<<\n");
	os << "settings: ";
	os <<	CSettings::str_width << " = " << ind._w;
	os << ", " << CSettings::str_height << " = " << ind._h;
	os << ", " << CSettings::str_depth << " = " << ind._d;
	os << ", total_cubes = " << ind._total_cubes << endl;

	os << "  " << CSettings::str_diff << " = ";
	os << CSettings::diff_descriptors[ind._difficulty] << endl;

	os << "  keys: x axis = " << ind._keys[key_XNEG]
			            << ", " << ind._keys[key_XPOS];
	os << "; y axis = " << ind._keys[key_YNEG]
			      << ", " << ind._keys[key_YPOS];
	os << "; z axis = " << ind._keys[key_ZNEG]
			      << ", " << ind._keys[key_ZPOS] << endl;

	os << "  left = " << ind._keys[key_LEFT]
		<< ", right = " << ind._keys[key_RIGHT]
		<< ", down = " << ind._keys[key_DOWN]
		<< ", up = " << ind._keys[key_UP] << endl;
	os << "  drop = " << ind._keys[key_DROP]
		<< ", pause = " << ind._keys[key_PAUSE]
		<< ", toggle next piece = " << ind._keys[key_TOGGLENP] << endl;
	
	os << "  speed = " << CSettings::speed_descriptors[ind._speed] << endl;

	os << "  view: show next piece = " << ind._show_np
		<< ", show scale = " << ind._show_scale
		<< ", show score = " << ind._show_score
		<< ", show stats = " << ind._show_stats << endl;
	
	os << "  sound support = " << ind._sound_support << endl;

	os << "  nb of lines in hof = " << ind._nhofl
		<< ", show old scores = " << ind._show_old_score << endl;
	
	os << "  pause on lost focus = " << ind._pause_on_nofocus
		<< ", resume on focus = " << ind._resume_on_focus << endl;
	return os;
}
