/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999, 2000, 2001 Sebastien Nicoud <seb@eseb.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#include "common.h"
#include "csound.h"


// class definition
CSound::CSound()
{
	if (debug > 1) g_print("CSound::CSound()\n");
	_played = FALSE;
	memset(_event,0,MAX_EVENT_LEN);
	memset(_note,0,MAX_NOTE_LEN);
	memset(_file,0,MAX_FILENAME_LENGTH);
	_valid = FALSE;
	memset(_default,0,MAX_FILENAME_LENGTH);
}

CSound::CSound(const CSound &ind)
{
	if (debug > 1) g_print("CSound::CSound()\n");
	_played = ind._played;
	memcpy(_event,ind._event,MAX_EVENT_LEN);
	memcpy(_note,ind._note,MAX_NOTE_LEN);
	memcpy(_file,ind._file,MAX_FILENAME_LENGTH);
	_valid = ind._valid;
	memcpy(_default,ind._default,MAX_FILENAME_LENGTH);	
}

CSound::~CSound()
{
	if (debug > 1) g_print("CSound::~CSound()\n");
}

ostream &operator<< (ostream &os, CSound &ind)
{
//	if (debug > 2) g_print("CSound::operator<<\n");
	os	<< "sound: "
		<< "  event=>" << ind._event << "<, "
		<< " note=>" << ind._note << "<, "
		<< " played=" << ind._played << endl;
	os	<< "  file=>" << ind._file << "<, "
		<< " valid=" << ind._valid << ", "
		<< " default=>" << ind._default << "<" << endl;
	return os;
}

gint CSound::init(gchar *event, gchar *note, gchar *def)
{
	if (debug) g_print("CSound::init (%s)\n",event);
    gchar config_str[128];
	// played
	if (!strcmp(event, "Move")) { // this event not played by default
		// read from config file, = FALSE if not found
		sprintf(config_str,"%s/%s/Enable %s sound=false",
			GNO3DTET_EXENAME, CONFIG_PREFERENCES, event);
		_played = gnome_config_get_bool_with_default(config_str, NULL);
	} else {
		// read from config file, = TRUE if not found
		sprintf(config_str,"%s/%s/Enable %s sound=true",
			GNO3DTET_EXENAME, CONFIG_PREFERENCES, event);
		_played = gnome_config_get_bool_with_default(config_str, NULL);
	}
	// _event
	strncpy(_event, event, MAX_EVENT_LEN);
	_event[MAX_EVENT_LEN-1] = 0;
	// note
	strncpy(_note, note, MAX_NOTE_LEN);
	_note[MAX_NOTE_LEN-1] = 0;
	// default = SOUNDDIR + def
	sprintf(_default,"%s%s", SOUNDDIR, def);
	_default[MAX_FILENAME_LENGTH-1] = 0;
	// file, read from config file, = _default if not found
	sprintf(config_str,"%s/%s/Filename %s sound=%s",
		GNO3DTET_EXENAME, CONFIG_PREFERENCES,
		event, _default);
	gchar *tmp_str = gnome_config_get_string_with_default(config_str,NULL);
	strncpy(_file, tmp_str, MAX_FILENAME_LENGTH);
	g_free(tmp_str);
	_file[MAX_FILENAME_LENGTH-1] = 0;
	// valid
	// FIXME - compute validity here
	_valid = TRUE;
	return (_valid?0:1);
}

gint CSound::restore_default(void)
{
	if (debug) g_print("CSound::restore_default\n");
	strcpy(_file, _default);
	// FIXME - compute validity here
	_valid = TRUE;
	return (_valid?0:1);	
}

void CSound::save(gboolean synchronize)
{
	if (debug) g_print("CSound::save\n");
    gchar config_str[128];
	// played
	sprintf(config_str,"%s/%s/Enable %s sound",
		GNO3DTET_EXENAME, CONFIG_PREFERENCES, _event);
	gnome_config_set_bool(config_str, _played);
	// file
	sprintf(config_str,"%s/%s/Filename %s sound",
		GNO3DTET_EXENAME, CONFIG_PREFERENCES, _event);
	gnome_config_set_string(config_str, _file);

	if (synchronize) gnome_config_sync();
}

// standalone functions ///////////////////////////////////////////////////
gint csound_get_event_number(gchar *event)
{
	if (debug>1) g_print("csound: csound_get_event_number >%s<: ", event);
	for (gint i=0; i<NUMBER_SOUND_EVENT; i++)
		if (!strcmp(sounds[i]._event,event)) {
			if (debug>1) g_print("found at %d\n", i);
			return i; // found it
		}
	if (debug>1) g_print("not found !\n");
	return -1; // not found
}

/* csound_init_sounds
 *	initialize the sound database
 */
void csound_init_sounds(void)
{
	if (debug) g_print("csound: csound_init_sounds\n");
	// define new sounds here, then update NUMBER_SOUND_EVENT
	// the 2nd field should be <= 80 chars
	sounds[CSound::snd_SCORE].init(
		_("Big Score"),
		_("you score big"),
		  "score.wav");
	sounds[CSound::snd_DANGER].init(
		_("Danger"),
		_("the top cube is dangerously high"),
		  "danger.wav");
	sounds[CSound::snd_FALL].init(
		_("Fall"),
		_("a piece is dropped"),
		  "fall.wav");
	sounds[CSound::snd_HOF].init(
		_("Hall Of Fame"),
		_("you enter the Hall Of Fame"),
		  "hof.wav");
	sounds[CSound::snd_LAND].init(
		_("Landing piece"),
		_("a piece lands"),
		  "land.wav");
	sounds[CSound::snd_SCREWD].init(
		_("Bad Landing"),
		_("a piece lands and creates holes underneath"),
		  "screwd.wav");
	sounds[CSound::snd_MOVE].init(
		_("Move"),
		_("the current piece is moved around"),
		  "move.wav");
	sounds[CSound::snd_LEVEL].init(
		_("Next level"),
		_("a new speed level is reached"),
		  "level.wav");
	sounds[CSound::snd_NEW].init(
		_("New Game"),
		_("a new game starts"),
		  "new.wav");
	sounds[CSound::snd_LAYER].init(
		_("Completed layer"),
		_("a layer is completed"),
		  "layer.wav");
	sounds[CSound::snd_PAUSE].init(
		_("Pause"),
		_("the game is paused"),
		  "pause.wav");
	sounds[CSound::snd_EOL].init("EOL", "End Of List - unused", "");
}

void csound_play_for_event(gint snd_event, gint snd_event_otherwise)
{
	if (debug > 1) g_print("csound: csound_play_for_event %d\n", snd_event);
	if (snd_event >= NUMBER_SOUND_EVENT) return;
	if (sounds[snd_event]._played) {
    	if (debug > 1) g_print("csound: playing >%s<\n",sounds[snd_event]._file);
		gnome_sound_play(sounds[snd_event]._file);
    }
	else if (snd_event_otherwise != -1 && sounds[snd_event_otherwise]._played) {
    	if (debug > 1) g_print("csound: playing o >%s<\n",sounds[snd_event]._file);
		gnome_sound_play(sounds[snd_event_otherwise]._file);
}   }
	
