/* gnommind: Master Mind game for GNOME
 * Copyright (C) 1999, 2000, 2001 Sebastien Nicoud <seb@eseb.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#ifndef _CWHOF
#define _CWHOF

#include "common.h"

class CWHof
{
// statics
public:
	static gboolean	_viewed; // true when hof is viewed
	static const gint _nc = 11; // nb of columns in table
	
// constants
private:
	enum {
		state_VIEW_ONLY, // view only
		state_NOT_GOOD_ENOUGH, // score is too low
		state_NEW_HIDDEN_ENTRY, // new entry, but not shown
		state_NEW_ENTRY, // new entry in the top shown scores
		state_ALREADY_RECORDED // score already recorded (same game ids)
	};
// attributes
private: // GUI stuff
	GtkWidget	*_window; // main window
	GtkWidget	*_table; // table
	GSList		*_slist; // list containing strings
	gint		 _wh; // window height
	
private: // data
	CHof		*_data[MAX_NHOFL+2];
	CHof		*_game;
	gint		 _state;
	gint		 _nhofl;
	gint		 _game_index;
	gint		 _already_recorded_game_index;
public:
	gboolean 	 _show_old_score;
	gint	 	 _nhofl_to_display;

// contructors
public:
	CWHof(); // constructeur par defaut
	
// destructor
public:	
	~CWHof();

// operators
public:
	friend ostream &operator<< (ostream &os, CWHof &ind);
	
// methods
private:
	gboolean append_game_to_hof(void);
	gint does_player_enter_hof(void);
	void free_memory(void);
	void sort_hof(void);
	void table_add_row(gint rank, gint index=-1);
	void window_hof(void);
	void window_name_entry(void);
	
public:
	void close_hof(void);
	void hof(gboolean update=FALSE, gint show_old_score=-1, gint nhofl=-1);
	
// i/o methods
private:
	guint read_hof(void);
	gint write_hof(void);
	
// callbacks
private:
	static void cb_window_hof_destroy(GtkObject *widget, gpointer data);
	static void cb_entry_name_changed(GtkObject *widget, gpointer data);
	static void cb_window_name_entry_destroy(GtkObject *widget, gpointer data);
	static void cb_window_name_entry_key_press_event(GtkWidget *widget,
		GdkEvent *event, gpointer data);
};

#endif // _WHOF


