/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999, 2000, 2001 Sebastien Nicoud <seb@eseb.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#include "common.h"
#include "csound.h"
#include "cgame.h"
#include "csettings.h"
#include "cwsettings.h"
#include "cwhof.h"
#include "cwmain.h"
#include "score.xpm"
#include "level.xpm"
#include "layers.xpm"
#include "blocs.xpm"
#include "cubes.xpm"
#include "digitssmall.xpm"
#include "digitsbig.xpm"

// statics
GdkPixmap		*CWMain::_pixmap_board = NULL;
GdkPixmap		*CWMain::_pixmap_world = NULL;
GdkGC 			*CWMain::_gc_board = NULL;

const gint CWMain::_k = 4; // drawing constant - tells at which depth the
	// cube looks twice smaller

const gchar *CWMain::_color_descriptors[MAX_WORLD_DEPTH] = {
	"red",
	"orange",
	"yellow3",
	"chartreuse3",
	"forest green",
	"cyan4",
	"blue",
	"dark violet",
	"violet",
	"indian red",
	"dark goldenrod",
	"gold2",
	"pink3",
	"grey50",
	"red4",
	"salmon2",
	"green",
	"dark violet",
	"sienna",
	"white"
};

CWMain::CWMain()
{
	if (debug) g_print("CWMain::CWMain - create main window\n");
	_timer_tag = FALSE;
	_x0 = 0;
	_y0 = 0;
	_tx = settings->_w/2.0;
	_ty = settings->_h/2.0;
	_no_clipping = TRUE;
	_clip_x1 = 0;
	_clip_y1 = 0;
	_clip_x2 = -1;
	_clip_y2 = -1;
	_frames = 0;
	_ms_timer = 0;
	_ms_elapsed = 0;
	_show_scale = TRUE;
	_paused_on_lostfocus = FALSE;

	_wmain = gnome_app_new(GNO3DTET_EXENAME,GNO3DTET_NAME);
	// main window handlers
//	gtk_signal_connect(GTK_OBJECT(_wmain), "delete_event",
//		GTK_SIGNAL_FUNC(cb_wmain_delete_event), NULL);
	gtk_signal_connect(GTK_OBJECT(_wmain), "destroy",
		GTK_SIGNAL_FUNC(cb_wmain_delete_event), NULL);
	gtk_signal_connect(GTK_OBJECT(_wmain), "key_press_event",
		GTK_SIGNAL_FUNC(cb_key_press_event),this);
	gtk_signal_connect(GTK_OBJECT(_wmain), "focus-out-event",
		GTK_SIGNAL_FUNC(cb_wmain_focus_out_event),this);
	gtk_signal_connect(GTK_OBJECT(_wmain), "focus-in-event",
		GTK_SIGNAL_FUNC(cb_wmain_focus_in_event),this);
	gtk_widget_realize(_wmain);

	// load images
	load_images();
		
	// create menus
	if (debug) g_print("CWMain::CWMain: create menus\n");
	GnomeUIInfo game_menu[] = {
	GNOMEUIINFO_MENU_NEW_GAME_ITEM(cb_menu_game_new_activate, this),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_PAUSE_GAME_ITEM(cb_menu_game_pause_activate, this),
	GNOMEUIINFO_MENU_SCORES_ITEM(cb_menu_game_hof_activate, this),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM(cb_menu_game_exit_activate, this),
	GNOMEUIINFO_END
	};
	GnomeUIInfo settings_menu[] = {
	GNOMEUIINFO_MENU_PREFERENCES_ITEM(cb_menu_settings_preferences_activate,
		this),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_TOGGLEITEM_DATA(N_("Show Tool Bar"),
		N_("Toggle display of the toolbar"),
		cb_menu_settings_toolbar_check_toggled, this, NULL),
	GNOMEUIINFO_TOGGLEITEM_DATA(N_("Enable sounds"),
		N_("Toggle sound support"),
		cb_menu_settings_soundsupport_check_toggled, this, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_TOGGLEITEM_DATA(N_("Show Next Piece"),
		N_("Toggle display of the next piece"),
		cb_menu_settings_next_piece_check_toggled, this, NULL),
	GNOMEUIINFO_TOGGLEITEM_DATA(N_("Show Depth Scale"),
		N_("Toggle display of the scale"),
		cb_menu_settings_scale_check_toggled, this, NULL),
	GNOMEUIINFO_TOGGLEITEM_DATA(N_("Show Score"),
		N_("Toggle display of the score"),
		cb_menu_settings_score_check_toggled, this, NULL),
	GNOMEUIINFO_TOGGLEITEM_DATA(N_("Show Stats"),
		N_("Toggle display of the stats"),
		cb_menu_settings_stats_check_toggled, this, NULL),
	GNOMEUIINFO_ITEM_DATA(N_("Show All"),
		N_("Show everything"),
		cb_menu_settings_show_all_activate, this, NULL),
	GNOMEUIINFO_ITEM_DATA(N_("Hide All"),
		N_("Hide everything"),
		cb_menu_settings_hide_all_activate, this, NULL),
	GNOMEUIINFO_END
	};	
	GnomeUIInfo help_menu[] = {
	GNOMEUIINFO_HELP((void *)GNO3DTET_EXENAME),
	GNOMEUIINFO_ITEM(N_("Show License"),N_("Show GNU Public License"),
		cb_menu_help_license_activate, NULL),
	GNOMEUIINFO_MENU_ABOUT_ITEM(cb_menu_help_about_activate, this),
	GNOMEUIINFO_END
	}
	;
	GnomeUIInfo menu[] = {
	GNOMEUIINFO_MENU_GAME_TREE(game_menu),
	GNOMEUIINFO_MENU_SETTINGS_TREE(settings_menu),
	GNOMEUIINFO_MENU_HELP_TREE(help_menu),
	GNOMEUIINFO_END
	};
	gnome_app_create_menus(GNOME_APP(_wmain), menu);
	_menu_item_toolbar = settings_menu[2].widget;
	_menu_item_sound_support = settings_menu[3].widget; // [4] is a separator;
	_menu_item_np = settings_menu[5].widget;
	_menu_item_scale = settings_menu[6].widget;
	_menu_item_score = settings_menu[7].widget;
	_menu_item_stats = settings_menu[8].widget;

	// create status bar
	create_status_bar(); 

	// create tool bar
	if (debug) g_print("CWMain::CWMain: create tool bar\n");
	GnomeUIInfo toolbar_uiinfo[] = {
	{GNOME_APP_UI_ITEM, N_("New"), N_("Start a new game"),
	 (gpointer)cb_menu_game_new_activate, this, NULL,
	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_NEW, 0,
	 (GdkModifierType)0, NULL},
	{GNOME_APP_UI_ITEM, N_("Pause"), N_("Pause the game"),
	 (gpointer)cb_menu_game_pause_activate, this, NULL,
	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_TIMER_STOP, 0,
	 (GdkModifierType)0, NULL},
	{GNOME_APP_UI_ITEM, N_("Prefs"), N_("Call Preferences window"),
	 (gpointer)cb_menu_settings_preferences_activate, this, NULL,
	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PREFERENCES, 0,
	 (GdkModifierType)0, NULL},
	{GNOME_APP_UI_ITEM, N_("Scores"), N_("Show the Hall Of Fame"),
	 (gpointer)cb_menu_game_hof_activate, this, NULL,
	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_SCORES, 0,
	 (GdkModifierType)0, NULL},
	{GNOME_APP_UI_ITEM, N_("Exit"), N_("Exit the program"),
	 (gpointer)cb_menu_game_exit_activate, this, NULL, GNOME_APP_PIXMAP_STOCK,
	 GNOME_STOCK_PIXMAP_EXIT, 0, (GdkModifierType)0, NULL},
	GNOMEUIINFO_END
	};
	gnome_app_create_toolbar(GNOME_APP(_wmain), toolbar_uiinfo);
	_toolbar_gdi = gnome_app_get_dock_item_by_name(
		GNOME_APP(_wmain), GNOME_APP_TOOLBAR_NAME);
	_toolbar = gnome_dock_item_get_child (_toolbar_gdi);
	
    // enable hints. They appear in the status bar.
    gnome_app_install_menu_hints(GNOME_APP(_wmain), menu);

	// create the main container box & drawing area
	if (debug) g_print("CWMain::CWMain: create board\n");
	GtkWidget *main_vbox = gtk_vbox_new (FALSE, 0);
	gnome_app_set_contents(GNOME_APP(_wmain),main_vbox);

	_w = 100;
	_h = 100;
	_board = gtk_drawing_area_new();
	_pixmap_board = NULL;
	gtk_box_pack_start(GTK_BOX(main_vbox),_board,TRUE,TRUE,0);
	gtk_drawing_area_size(GTK_DRAWING_AREA(_board), _w, _h);
	gtk_signal_connect(GTK_OBJECT(_board), "expose_event",
		GTK_SIGNAL_FUNC(cb_board_expose_event), this);
	gtk_signal_connect(GTK_OBJECT(_board), "configure_event",
		GTK_SIGNAL_FUNC(cb_board_configure_event), this);

	// color stuff
	if (debug) g_print("CWMain::CWMain: create colors\n");
	_colormap = gdk_colormap_get_system();
	for (gint i=0; i<MAX_WORLD_DEPTH; i++) {	
		if (!gdk_color_parse(_color_descriptors[i], &_colors[i][1])) {
			if (debug) g_print("** ERROR **: CWmain::CWmain() color lookup failed\n");
			_colors[i][1].red   = 0xFFFF;
			_colors[i][1].green = 0x0000;
			_colors[i][1].blue  = 0x0000;
		}
		if (!gdk_colormap_alloc_color(_colormap, &_colors[i][1], FALSE, TRUE))
			g_print("** ERROR **: CWmain::CWmain() color alloc failed\n");
		// compute lighter color
		_colors[i][0].red = _colors[i][1].red + (0xFFFF-_colors[i][1].red)/2;
		_colors[i][0].green = _colors[i][1].green + (0xFFFF-_colors[i][1].green)/2;
		_colors[i][0].blue = _colors[i][1].blue + (0xFFFF-_colors[i][1].blue)/2;
		if (!gdk_colormap_alloc_color(_colormap, &_colors[i][0], FALSE, TRUE))
			g_print("** ERROR **: CWmain::CWmain() color alloc failed\n");
		// compute darker color
		_colors[i][2].red = _colors[i][1].red/2;
		_colors[i][2].green = _colors[i][1].green/2;
		_colors[i][2].blue = _colors[i][1].blue/2;		
		if (!gdk_colormap_alloc_color(_colormap, &_colors[i][2], FALSE, TRUE))
			g_print("** ERROR **: CWmain::CWmain() color alloc failed\n");
		if (debug > 1)
			g_print("  color %d: R[%x-%x-%x] G[%x-%x-%x] B[%x-%x-%x]\n", i,
				_colors[i][0].red, _colors[i][1].red, _colors[i][2].red,
				_colors[i][0].green, _colors[i][1].green, _colors[i][2].green,
				_colors[i][0].blue, _colors[i][1].blue, _colors[i][2].blue);
	}	
	// create GC
	_gc_board = gdk_gc_new(_wmain->window);
	_black.red = 0x0000;
	_black.green = 0x0000;
	_black.blue = 0x0000;
	if (!gdk_colormap_alloc_color(_colormap, &_black, FALSE, TRUE))
		g_print("** ERROR **: CWmain::CWmain() color alloc failed\n");
	_white.red = 0xFFFF;
	_white.green = 0xFFFF;
	_white.blue = 0xFFFF;
	if (!gdk_colormap_alloc_color(_colormap, &_white, FALSE, TRUE))
		g_print("** ERROR **: CWmain::CWmain() color alloc failed\n");
	_color_world.red = 0x0000;
	_color_world.green = 0xFFFF;
	_color_world.blue = 0x0000;
	if (!gdk_colormap_alloc_color(_colormap, &_color_world, FALSE, TRUE))
		g_print("** ERROR **: CWmain::CWmain() color alloc failed\n");
	gdk_gc_set_foreground(_gc_board, &_color_world);
	_color_piece.red = 0xFFFF;
	_color_piece.green = 0xFFFF;
	_color_piece.blue = 0xFFFF;
	if (!gdk_colormap_alloc_color(_colormap, &_color_piece, FALSE, TRUE))
		g_print("** ERROR **: CWmain::CWmain() color alloc failed\n");
	_color_scale.red = 0x2424;
	_color_scale.green = 0x4040;
	_color_scale.blue = 0x3333;
	if (!gdk_colormap_alloc_color(_colormap, &_color_scale, FALSE, TRUE))
		g_print("** ERROR **: CWmain::CWmain() color alloc failed\n");

	// timers
	if (debug) g_print("CWMain::CWMain: start timer (1 sec)\n");
	gtk_label_set(GTK_LABEL(_lbl_timer),"888:88:88");
	gtk_timeout_add(1000, cb_timer1000_called, this);

	gtk_widget_show_all(_wmain);
}

CWMain::~CWMain()
{
	if (debug) g_print("CWMain::~CWMain - destroy main window\n");
	free_memory();
}

ostream &operator<< (ostream &os, CWMain &ind)
{
	if (debug > 2) g_print("CWMain::operator<<\n");
	os << "main window: ";
	// FIXME
	return os;
}

void CWMain::compute_drawing_values(void)
{
        if (debug) g_print("CWMain::compute_drawing_values\n");
        if (mds == NULL) return;
        if (mds->_settings == NULL) return;
		gint sw = mds->_settings->_w;
		gint sh = mds->_settings->_h;
		gint sd = mds->_settings->_d;
        if (debug) g_print(
			"  world=(%dx%dx%d), show score=%d, stats=%d, scale=%d\n",
			sw,sh,sd, _show_score, _show_stats, _show_scale);

		_tx = sw/2.0;
		_ty = sh/2.0;
        gint cwu = _w / sw; // cube width up
        gint chu = _h / sh; // cube height down
        _cu = (cwu>chu)?chu:cwu; // cote cube on top

		gint adjusted_w = _w;
		gint needed_w = 0;
		if (_show_scale) needed_w += SCALE_WIDTH;
		if (_show_stats) needed_w += STATS_TEXT_WIDTH;
		gint free_w = _w - _cu*sw;
		if (free_w < needed_w) adjusted_w = adjusted_w - needed_w;

		gint adjusted_h = _h;
		gint needed_h = 0;
		if (_show_score) needed_h += BIG_TEXT_HEIGHT;
		gint free_h = _h - _cu*sh;
		if (free_h < needed_h) adjusted_h = adjusted_h - needed_h;

        cwu = adjusted_w / sw;
        chu = adjusted_h / sh;
        _cu = (cwu>chu)?chu:cwu;
	
		// middle of the world
        _x0 = _w - _cu*sw/2-1;
		if (_show_scale) _x0 -= SCALE_WIDTH;
        _y0 = _h - _cu*sh/2-1;
		
		// scale cube height
        _scale_ch = adjusted_h / sd;
        if (_scale_ch > SCALE_WIDTH) _scale_ch = SCALE_WIDTH;
		
		// next piece size
		_cnp = _cu/4;
		
        _clip_x1 = 0;
        _clip_y1 = 0;
        _clip_x2 = -1;
        _clip_y2 = -1;
        if (debug) g_print("  w=%d, h=%d, adjw=%d, adjh=%d,\
x0=%d, y0=%d, cu=%d\n", _w, _h, adjusted_w, adjusted_h, _x0, _y0, _cu);
}

/* compute_numbers
 * creates a table of the digits composing a number
 * IN: n the number to process
 * INOUT: d = ptr to the digit table
 * IN: nbd = d[] length
 * RET: the actual number of digits
 * Exemple compute_numbers(2456, d[], 6) returns 4 and d[10,10,2,4,5,6]
*/
gint CWMain::compute_numbers(gint32 n, gint *d, gint nbd)
{
    if (debug > 2) g_print("CWMain::compute_numbers\n");
	gchar str[16];
	for (gint i=0; i<16; i++) str[i] = 'x';
	gchar *strnb = g_strdup_printf("%d",n);
	gint strnb_len = strlen(strnb);
	gint offset = 16 - strnb_len;
	for (gint i=0; i<strnb_len; i++) str[offset+i] = strnb[i];
	offset = 16 - nbd;
	gint ret = 0;
	for (gint i=0; i<nbd; i++)
		if (str[offset+i] == 'x') d[i] = 10;
		else {
			d[i] = (gint)(str[offset+i]-'0');
			ret++;
		}
    g_free(strnb);
	return ret;
}

/* create_status_bar
 * this function creates the status bar with a timer
*/
void CWMain::create_status_bar(void)
{
    if (debug) g_print("CWMain::create_status_bar\n");

	// create status bar
	_status_bar = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_USER);

	// create timer box
	_lbl_timer = gtk_label_new("000:00:00");
	gtk_box_pack_end(GTK_BOX(_status_bar),_lbl_timer,FALSE,FALSE,3);
	gtk_widget_show(_lbl_timer);

	// create timer box
	_lbl_frames = gtk_label_new("0 f/sec");
	gtk_box_pack_end(GTK_BOX(_status_bar),_lbl_frames,FALSE,FALSE,3);
	gtk_widget_show(_lbl_frames);

	// create settings info
	gchar ss[16];
	sprintf(ss,"%c.%dx%dx%d",
		(CSettings::diff_descriptors[settings->_difficulty])[0],
		settings->_w, settings->_h, settings->_d);
	_lbl_settings = gtk_label_new(ss);
	gtk_box_pack_end(GTK_BOX(_status_bar),_lbl_settings,FALSE,FALSE,3);
	gtk_widget_show(_lbl_settings);

	// set status bar in Gnome app
	gnome_appbar_set_default (GNOME_APPBAR(_status_bar),
		_("Seb's Gnome 3D Tetris"));
	gnome_app_set_statusbar(GNOME_APP(_wmain),_status_bar);
}

/* create_timer(timer in msec = delay between 2 frames)
 * function called by main()
*/
void CWMain::create_timer(gint timer)
{
	if (debug) g_print("CWMain::create_timer, start main timer (%d msec)\n",
		timer);
	if (!_timer_tag)
		gtk_timeout_add(timer, cb_timer_called, this);
	_timer_tag = TRUE;
}

/* draw_all
 * this function is only called when the whole screen needs to be refreshed
 * outside the gaming loop.
 * callers: CWSettings::apply()
*/
void CWMain::draw_all(gboolean need_cleaning)
{
	if (debug > 1) g_print("CWMain::draw_all\n");
	if (mds->_state == CGame::state_PLAYING) return;
	if (mds->_state != CGame::state_PAUSED) {
		// can't init when paused (which is playing)
		// because on-going game vars will be re-initialized
		if (need_cleaning) {
			mds->_state = CGame::state_NEW;
			mds->init(TRUE/*keep current settings*/);
		}
	}
	// update views according to settings global var
	toggle_np(toggle_UPDATE, FALSE); // updates _show_np
	toggle_scale(toggle_UPDATE, FALSE); 
	toggle_score(toggle_UPDATE, FALSE); 
	toggle_stats(toggle_UPDATE, FALSE); 
	compute_drawing_values();
	draw_world();
	if (mds->_state == CGame::state_PAUSED) draw_piece();
	if (mds->_state == CGame::state_PAUSED && _show_np) draw_next_piece(0);
	redraw(); // flush
	// status bar
	set_status_bar_message();
	set_status_bar_settings();
	if (need_cleaning) gtk_label_set(GTK_LABEL(_lbl_frames),"0 f/sec");
	if (need_cleaning) gtk_label_set(GTK_LABEL(_lbl_timer),"000:00:00");
}

void CWMain::draw_flat_rectangle(gfloat xa, gfloat ya, gfloat za,
	gfloat w, gfloat h)
{
    if (debug > 2) g_print("CWMain::draw_flat_rectangle \
(%.2f,%.2f,%.2f,%.2f,%.2f)\n", xa, ya, za, w ,h);
	draw_line(xa, ya, za, xa+w, ya, za, FALSE, FALSE);
	draw_line(xa+w, ya, za, xa+w, ya+h, za, FALSE, FALSE);
	draw_line(xa+w, ya+h, za, xa, ya+h, za, FALSE, FALSE);
	draw_line(xa, ya+h, za, xa, ya, za, FALSE, FALSE);
}

void CWMain::draw_next_piece(gfloat dt)
{
	if (debug > 2) g_print("CWMain::draw_next_piece\n");
	_next_piece.rotate_y(dt/4, NULL, TRUE, FALSE);
	t_triplet *pointeA;
	t_triplet *pointeB;
	gint xoffset = (gint)(_np_offset*_cnp);
	gint yoffset = (gint)(_np_offset*_cnp)+40;
	gint xsa;	gint ysa;
	gint xsb;	gint ysb;
	gdk_gc_set_foreground(_gc_board, &_white);
	for (gint nb_arretes=0; nb_arretes<_next_piece._nb_arretes; nb_arretes++) {
		pointeA = _next_piece._pointes[_next_piece._arretes[nb_arretes]->a];
		pointeB = _next_piece._pointes[_next_piece._arretes[nb_arretes]->b];
		xsa = (gint)(pointeA->x*_cnp+xoffset);
		ysa = (gint)(pointeA->y*_cnp+yoffset);
		xsb = (gint)(pointeB->x*_cnp+xoffset);
		ysb = (gint)(pointeB->y*_cnp+yoffset);
		gdk_draw_line(_pixmap_board, _gc_board,	xsa, ysa, xsb, ysb);
	}
}

/* draw_number
 * IN: n -> number to draw
 * IN: x,y -> coords in the screen
 * IN: nb_of_digits -> used only when IN:format > 0
 * IN: format = 0 -> print number only
 *            = 1 -> print number and pad with 0 to reach number of digits
 *            = 2 -> print number and pad with empty digits to reach number of digits
 * a digit pixmap is formatted: "0123456789_,"
*/
void CWMain::draw_number(gint n, gint x, gint y,
	gint nb_of_digits, 
	gint format,
	gboolean big_or_not_small_digits,
	gboolean print_commas,
	gboolean right_or_not_left)
{
    if (debug > 2) g_print("CWMain::draw_number %d (%d,%d) d=%d, f=%d\n",
		n, x, y, nb_of_digits, format);
	gint i;
	gint xoffset = x;
	gint yoffset = y;
	gint nbd[16];
	gint digit = 0;
	gint next_digit = 0;
	gint actual_nb_of_digits = compute_numbers(n, (gint *)nbd, nb_of_digits);
	if (format == 1)
		// pad with 0's
		for (i=0; i<nb_of_digits; i++) if (nbd[i] == 10) nbd[i] = 0;
	if (right_or_not_left) {
		// going to the right
		for (i=0; i<nb_of_digits; i++) {
			digit = nbd[i];
			if (i+1<nb_of_digits) next_digit = nbd[i+1];
			else next_digit = 10;
			if (format == 0 && digit == 10) continue; // jump to for(i)
			if (big_or_not_small_digits) {
				// draw big digits
				gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_digitsbig,
					digit*BIG_DIGIT_WIDTH, 0,
					xoffset,yoffset,
					BIG_DIGIT_WIDTH, BIG_DIGIT_HEIGHT);
				xoffset += BIG_DIGIT_WIDTH;
				if (print_commas && !div(nb_of_digits-i-1,3).rem
				 && i+1 != nb_of_digits
				 && digit != 10 && next_digit != 10) {
					// draw comma
					gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_digitsbig,
						11*BIG_DIGIT_WIDTH, 0,
						xoffset,yoffset,
						BIG_COMMA_WIDTH, BIG_DIGIT_HEIGHT);
					xoffset += BIG_COMMA_WIDTH;
				}
			} else {
				// draw small digits
				gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_digitssmall,
					digit*SMALL_DIGIT_WIDTH, 0,
					xoffset,yoffset,
					SMALL_DIGIT_WIDTH, SMALL_DIGIT_HEIGHT);
				xoffset += SMALL_DIGIT_WIDTH;
				if (print_commas && !div(nb_of_digits-i-1,3).rem
				 && i+1 != nb_of_digits
				 && digit != 10 && next_digit != 10) {
					// draw comma
					gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_digitssmall,
						11*SMALL_DIGIT_WIDTH, 0,
						xoffset,yoffset,
						SMALL_COMMA_WIDTH, SMALL_DIGIT_HEIGHT);
					xoffset += SMALL_COMMA_WIDTH;
		}	}	}						
	} else {
		// going to the left
		for (i=nb_of_digits-1; i>=0; i--) {
			digit = nbd[i];
			if (i-1>=0) next_digit = nbd[i-1];
			else next_digit = 10;
			if (format == 0 && digit == 10) break; // exit for(i)
			if (big_or_not_small_digits) {
				// draw big digits
				xoffset -= BIG_DIGIT_WIDTH;
				gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_digitsbig,
					digit*BIG_DIGIT_WIDTH, 0,
					xoffset,yoffset,
					BIG_DIGIT_WIDTH, BIG_DIGIT_HEIGHT);
				if (print_commas && !div(nb_of_digits-i,3).rem
				 && actual_nb_of_digits > 3
				 && digit != 10 && next_digit != 10) {
					// draw comma
					xoffset -= BIG_COMMA_WIDTH;
					gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_digitsbig,
						11*BIG_DIGIT_WIDTH, 0,
						xoffset,yoffset,
						BIG_COMMA_WIDTH, BIG_DIGIT_HEIGHT);
				}
			} else {
				// draw small digits
				xoffset -= SMALL_DIGIT_WIDTH;
				gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_digitssmall,
					digit*SMALL_DIGIT_WIDTH, 0,
					xoffset,yoffset,
					SMALL_DIGIT_WIDTH, SMALL_DIGIT_HEIGHT);
				if (print_commas && !div(nb_of_digits-i,3).rem
				 && actual_nb_of_digits > 3
				 && digit != 10 && next_digit != 10) {
					// draw comma
					xoffset -= SMALL_COMMA_WIDTH;
					gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_digitssmall,
						11*SMALL_DIGIT_WIDTH, 0,
						xoffset,yoffset,
						SMALL_COMMA_WIDTH, SMALL_DIGIT_HEIGHT);
}	}	}	}	}
			
void CWMain::draw_line(gfloat xa, gfloat ya, gfloat za,
	gfloat xb, gfloat yb, gfloat zb,
	gboolean board_drawing, gboolean update_clip)
{
	if (_pixmap_board == NULL) return;
	if (debug > 2) g_print("CWMain::draw_line (%.2f,%.2f,%.2f -> %.2f,%.2f,%.2f)\n",		xa, ya, za, xb, yb, zb);
	gint xsa = screen_x(xa, za);
	gint ysa = screen_y(ya, za);
	gint xsb = screen_x(xb, zb);
	gint ysb = screen_y(yb, zb);
//	if (debug) g_print("CWMain::draw_line (%.2f,%.2f,%.2f) => (%d,%d)",xa, ya, za, xsa, ysa);
//	if (debug) g_print(" -to-> (%.2f,%.2f,%.2f) => (%d,%d)\n",xb, yb, zb, xsb, ysb);
	if (update_clip) {
		if (xsa < _clip_x1) _clip_x1 = xsa;
		else if (xsa > _clip_x2) _clip_x2 = xsa;
		if (ysa < _clip_y1) _clip_y1 = ysa;
		else if (ysa > _clip_y2) _clip_y2 = ysa;
		if (xsb < _clip_x1) _clip_x1 = xsb;
		else if (xsb > _clip_x2) _clip_x2 = xsb;
		if (ysb < _clip_y1) _clip_y1 = ysb;
		else if (ysb > _clip_y2) _clip_y2 = ysb;
	}
	if (board_drawing)
		gdk_draw_line(_pixmap_board, _gc_board,	xsa, ysa, xsb, ysb);
	else
		gdk_draw_line(_pixmap_world, _gc_board,	xsa, ysa, xsb, ysb);
}

void CWMain::draw_piece(void)
{
	if (debug > 2) g_print("CWMain::draw_piece\n");
	gint nb_arretes;
	gdk_gc_set_foreground(_gc_board, &_color_piece);
	_clip_x1 = _w;
	_clip_y1 = _h;
	_clip_x2 = 0;
	_clip_y2 = 0;
	t_triplet *pointeA;
	t_triplet *pointeB;
	for (nb_arretes=0; nb_arretes<mds->_xing_piece._nb_arretes; nb_arretes++) {
		pointeA = mds->_xing_piece._pointes[
			mds->_xing_piece._arretes[nb_arretes]->a];
		pointeB = mds->_xing_piece._pointes[
			mds->_xing_piece._arretes[nb_arretes]->b];
		draw_line(
			pointeA->x,pointeA->y,pointeA->z,
			pointeB->x,pointeB->y,pointeB->z);
	}
	_clip_x1--;
	_clip_y1--;
	_clip_x2+=2;
	_clip_y2+=2;
}

void CWMain::draw_set_piece(gint x, gint y, gint z)
{
	if (debug > 2) g_print("CWMain::draw_set_piece (%d,%d,%d)\n", x,y,z);
	// draw top
	gint sd = mds->_settings->_d; // depth, in cubes
	gint sw = mds->_settings->_w;
	gint sh = mds->_settings->_h;
	gint ct = screen_cote(z-1)+1;
	gint xt = screen_x(x,z-1);
	gint yt = screen_y(y,z-1);
	gint cb = screen_cote(z)+1;
	gint xb = screen_x(x,z);
	gint yb = screen_y(y,z);
	gint wx = x+1;
	gint wy = y+1;
	gint wz = sd+1-z;
	gboolean t = mds->_world[wx][wy][wz+1];
	gboolean u = mds->_world[wx][y][wz];
	gboolean r = mds->_world[wx+1][wy][wz];
	gboolean d = mds->_world[wx][wy+1][wz];
	gboolean l = mds->_world[x][wy][wz];
   
	GdkPoint points[4];
	if (!u && 2*y > sh) {
		points[0].x = xb; points[0].y = yb;
		points[1].x = xb+cb; points[1].y = yb;
		points[2].x = xt+ct; points[2].y = yt;
		points[3].x = xt; points[3].y = yt;
		gdk_gc_set_foreground(_gc_board, &(_colors[(gint)(sd-z)][0]));
		gdk_draw_polygon(_pixmap_world, _gc_board, TRUE, (GdkPoint *)points, 4);
	}
	if (!r && 2*x+1 < sw) {
		points[0].x = xb+cb; points[0].y = yb;
		points[1].x = xb+cb; points[1].y = yb+cb;
		points[2].x = xt+ct; points[2].y = yt+ct;
		points[3].x = xt+ct; points[3].y = yt;
		gdk_gc_set_foreground(_gc_board, &(_colors[(gint)(sd-z)][2]));
		gdk_draw_polygon(_pixmap_world, _gc_board, TRUE, (GdkPoint *)points, 4);
		gdk_gc_set_foreground(_gc_board, &(_colors[(gint)(sd-z)][1]));
		if (wz>1 && mds->_world[wx+1][wy][wz-1]) {
			gdk_gc_set_foreground(_gc_board, &(_colors[(gint)(sd-z-1)][2]));
			gdk_draw_line(_pixmap_world, _gc_board, xb+cb,yb, xb+cb,yb+cb);
	}	}
	if (!d && 2*y+1 < sh) {
		points[0].x = xb; points[0].y = yb+cb;
		points[1].x = xb+cb; points[1].y = yb+cb;
		points[2].x = xt+ct; points[2].y = yt+ct;
		points[3].x = xt; points[3].y = yt+ct;
		gdk_gc_set_foreground(_gc_board, &(_colors[(gint)(sd-z)][2]));
		gdk_draw_polygon(_pixmap_world, _gc_board, TRUE, (GdkPoint *)points, 4);
		if (wz>1 && mds->_world[wx][wy+1][wz-1]) {
			gdk_gc_set_foreground(_gc_board, &(_colors[(gint)(sd-z-1)][2]));
			gdk_draw_line(_pixmap_world, _gc_board, xb,yb+cb, xb+cb,yb+cb);
	}	}
	if (!l && 2*x > sw) {
		points[0].x = xb; points[0].y = yb;
		points[1].x = xb; points[1].y = yb+cb;
		points[2].x = xt; points[2].y = yt+ct;
		points[3].x = xt; points[3].y = yt;
		gdk_gc_set_foreground(_gc_board, &(_colors[(gint)(sd-z)][0]));
		gdk_draw_polygon(_pixmap_world, _gc_board, TRUE, (GdkPoint *)points, 4);
	}
	if (!t) {
		gdk_gc_set_foreground(_gc_board, &(_colors[(gint)(sd-z)][1]));
		gdk_draw_rectangle(_pixmap_world, _gc_board,
			TRUE, xt, yt, ct, ct);
		gdk_gc_set_foreground(_gc_board, &_white);
		if (!u && !mds->_world[wx][y][wz+1])
			gdk_draw_line(_pixmap_world, _gc_board, xt,yt, xt+ct,yt);
		if (!l && !mds->_world[x][wy][wz+1])
			gdk_draw_line(_pixmap_world, _gc_board, xt,yt, xt,yt+ct);
	}
	if (!u && 2*y>sh && !l && 2*x>sw && !mds->_world[x][y][wz]) {
		gdk_gc_set_foreground(_gc_board, &_white);
		gdk_draw_line(_pixmap_world, _gc_board, xt,yt, xb,yb);		
	}
	if ((!l && !d || l && d)
	 && 2*x>sw && 2*y+1<sh && !mds->_world[x][wy+1][wz]) {
		gdk_gc_set_foreground(_gc_board, &(_colors[(gint)(sd-z)][1]));
		gdk_draw_line(_pixmap_world, _gc_board, xt,yt+ct, xb,yb+cb);		
	}
	if (!r && 2*y+1<sh && 2*x+1<sw && mds->_world[wx+1][y][wz]) {
		gdk_gc_set_foreground(_gc_board, &_black);
		gdk_draw_line(_pixmap_world, _gc_board, xt+ct,yt, xb+cb,yb);		
	}
	if ((!r && !u || r && u)
	 && 2*x+1<sw && 2*y>sh && !mds->_world[wx+1][y][wz]) {
		gdk_gc_set_foreground(_gc_board, &(_colors[(gint)(sd-z)][1]));
		gdk_draw_line(_pixmap_world, _gc_board, xt+ct,yt, xb+cb,yb);		
	}
}

void CWMain::draw_world(void)
{	
	if (debug) g_print("CWMain::draw_world, x0=%d, y0=%d, cu=%d, k=%d, tx=%.2f,\
 ty=%.2f, highst cube=%d\n",_x0,_y0,_cu,_k,_tx,_ty, mds->_highest_cube);
	// background
	gdk_gc_set_foreground(_gc_board, &_black);
	gdk_draw_rectangle(_pixmap_world, _gc_board/*_board->style->black_gc*/,
		TRUE, 0, 0, _w, _h);
	gint i,j,k;
	gint sw = mds->_settings->_w;
	gint sh = mds->_settings->_h;
	gint sd = mds->_settings->_d;
	gdk_gc_set_foreground(_gc_board, &_color_world);
	for (i=0; i<=sd; i++)
		draw_flat_rectangle(0,0,i,sw,sh);
	for (i=0; i<=sw; i++) {
		draw_line(i,0,0,i,0,sd, FALSE, FALSE);
		draw_line(i,0,sd,i,sh,sd, FALSE, FALSE);
		draw_line(i,sh,sd,i,sh,0, FALSE, FALSE);
	}
	for (i=0; i<=sh; i++) {
		draw_line(0,i,0,0,i,sd, FALSE, FALSE);
		draw_line(0,i,sd,sw,i,sd, FALSE, FALSE);
		draw_line(sw,i,sd,sw,i,0, FALSE, FALSE);
	}
	// scale
	if (_show_scale) {
		gint x = _w-SCALE_WIDTH;
		gint y;
		gint w = SCALE_WIDTH-1;
		gint h = _scale_ch-1;
		for (gint i=0; i<mds->_settings->_d; i++) {
			y = _h-_scale_ch*(i+1);
			if (i < mds->_highest_cube) {
			gdk_gc_set_foreground(_gc_board, &(_colors[i][1]));
			gdk_draw_rectangle(_pixmap_world, _gc_board, TRUE, x, y, w, h);
			gdk_gc_set_foreground(_gc_board, &(_colors[i][2]));
			gdk_draw_line(_pixmap_world, _gc_board, x+w, y, x+w, y+h);
			gdk_draw_line(_pixmap_world, _gc_board, x+w, y+h, x, y+h);
			gdk_gc_set_foreground(_gc_board, &(_colors[i][0]));
			gdk_draw_line(_pixmap_world, _gc_board, x, y+h, x, y);
			gdk_draw_line(_pixmap_world, _gc_board, x, y, x+w, y);
			} else {
			gdk_gc_set_foreground(_gc_board, &_color_scale);
			gdk_draw_rectangle(_pixmap_world, _gc_board, TRUE, x+1, y+1,
				w-2, h-2);			
	}	}	}
	// set pieces
	gint msw = (gint)(sw/2);
	gint msh = (gint)(sh/2);
	if (div(sw,2).rem) msw++;
	if (div(sh,2).rem) msh++;
	gint sz = 0;
	// draw from the bottom to the top
	for (k=1; k</*mds->_settings->_d*/mds->_highest_cube+1; k++) {
		sz = sd+1-k; 
		// draw from the outer ring to the inner ring
		for (j=1; j<=msh; j++)
			for (i=1; i<=msw; i++) {
				if (mds->_world[i][j][k]) draw_set_piece(i-1, j-1, sz);
				if (mds->_world[sw+1-i][j][k]) draw_set_piece(sw-i, j-1, sz);
				if (mds->_world[i][sh+1-j][k]) draw_set_piece(i-1, sh-j, sz);
				if (mds->_world[sw+1-i][sh+1-j][k])
					draw_set_piece(sw-i, sh-j, sz);
	}		}
	// score
	gint nbd[12];
	gint x_offset = 0;
	gint y_offset = 0;
	if (_show_score) {
		x_offset = _w-_cu*sw;
		if (_show_scale) x_offset -= SCALE_WIDTH;
		if (x_offset < 0) x_offset = 0;
		y_offset = _h-_cu*sh-BIG_TEXT_HEIGHT;
		if (y_offset < 0) y_offset = 0;
		// level
		gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_level,
			0, 0, x_offset, y_offset, -1, -1);
		// nb level
		y_offset += BIG_TEXT_HEIGHT;
		draw_number(mds->_level, x_offset, y_offset,
			NB_LEVEL_DIGITS/*nb_of_digits*/, 0/*format*/,
			TRUE/*big_or_not_small_digits*/, TRUE/*print_commas*/,
			TRUE/*right_or_not_left*/);
		// score
		x_offset = _w-SCALE_WIDTH;
		y_offset -= BIG_TEXT_HEIGHT;		
		gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_score, 0,0,
			x_offset-SCORE_TEXT_WIDTH,y_offset, SCORE_TEXT_WIDTH,-1);
		// nb score
		y_offset += BIG_TEXT_HEIGHT;
		draw_number(mds->_score, x_offset, y_offset,
			NB_SCORE_DIGITS/*nb_of_digits*/, 0/*format*/,
			TRUE/*big_or_not_small_digits*/, TRUE/*print_commas*/,
			FALSE/*right_or_not_left*/);
	}	
	// stats
	if (_show_stats) {
		gint xtext_offset = _w-_cu*sw-STATS_TEXT_WIDTH;
		if (_show_scale) xtext_offset -= SCALE_WIDTH;
		x_offset = xtext_offset + 2;
		// nb cubes
		y_offset = _h - SMALL_DIGIT_HEIGHT-2;
		draw_number(mds->_cubes, x_offset, y_offset,
			NB_CUBES_DIGITS/*nb_of_digits*/, 0/*format*/,
			FALSE/*big_or_not_small_digits*/, TRUE/*print_commas*/,
			TRUE/*right_or_not_left*/);
		// cubes
		y_offset -= SMALL_TEXT_HEIGHT+2;
		gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_cubes, 0, 0,
			xtext_offset, y_offset, -1,-1);
		// nb blocs
		y_offset -= SMALL_DIGIT_HEIGHT+8;
		draw_number(mds->_blocs, x_offset, y_offset,
			NB_BLOCS_DIGITS/*nb_of_digits*/, 0/*format*/,
			FALSE/*big_or_not_small_digits*/, TRUE/*print_commas*/,
			TRUE/*right_or_not_left*/);
		// cubes
		y_offset -= SMALL_TEXT_HEIGHT+2;
		gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_blocs, 0, 0,
			xtext_offset,y_offset, -1,-1);
		// nb layers
		y_offset -= SMALL_DIGIT_HEIGHT+8;
		draw_number(mds->_layers, x_offset, y_offset,
			NB_LAYERS_DIGITS/*nb_of_digits*/, 0/*format*/,
			FALSE/*big_or_not_small_digits*/, TRUE/*print_commas*/,
			TRUE/*right_or_not_left*/);
		// layers
		y_offset -= SMALL_TEXT_HEIGHT+2;
		gdk_draw_pixmap(_pixmap_world, _gc_board, xpm_layers, 0, 0,
			xtext_offset,y_offset, -1,-1);
	}
	// update main pixmap
	gdk_draw_pixmap(_pixmap_board, _gc_board, _pixmap_world, 
		0,0, 0,0, -1,-1); // copy whole rectangle
	_no_clipping = TRUE;
}

/* exit game
 * calls the confirmation dialog
*/
void CWMain::exit(void)
{
	if (debug > DBG_CB) g_print("CWMain::exit\n");
	GtkWidget *box;
	box = gnome_message_box_new (_("Do you really want to quit?"),
		GNOME_MESSAGE_BOX_QUESTION,
		GNOME_STOCK_BUTTON_YES,
		GNOME_STOCK_BUTTON_NO,
		NULL);
	gnome_dialog_set_parent (GNOME_DIALOG(box), GTK_WINDOW(_wmain));
	gnome_dialog_set_default (GNOME_DIALOG(box), 0);
	gtk_window_set_modal (GTK_WINDOW (box), TRUE);  
	gtk_signal_connect (GTK_OBJECT (box), "clicked",
		(GtkSignalFunc)cb_dialog_quit_game_clicked, this);
	gtk_widget_show (box);
}

void CWMain::free_memory(void)
{
	if (debug > 2) g_print("CWMain::free_memory\n");
	// timer
	if (_ms_timer) g_timer_destroy(_ms_timer);
	// images
	if (xpm_score) gdk_pixmap_unref(xpm_score);
	if (mask_score) gdk_pixmap_unref(mask_score);
	if (xpm_level) gdk_pixmap_unref(xpm_level);
	if (mask_level) gdk_bitmap_unref(mask_level);
	if (xpm_layers) gdk_pixmap_unref(xpm_layers);
	if (mask_layers) gdk_bitmap_unref(mask_layers);
	if (xpm_blocs) gdk_pixmap_unref(xpm_blocs);
	if (mask_blocs) gdk_bitmap_unref(mask_blocs);
	if (xpm_cubes) gdk_pixmap_unref(xpm_cubes);
	if (mask_cubes) gdk_bitmap_unref(mask_cubes);
	if (xpm_digitssmall) gdk_pixmap_unref(xpm_digitssmall);
	if (xpm_digitsbig) gdk_pixmap_unref(xpm_digitsbig);
	// top-level widgets
//	if (_wmain) gtk_object_unref(GTK_OBJECT(_wmain));
	if (_gc_board) gdk_gc_unref(_gc_board);
	if (_pixmap_board) gdk_pixmap_unref(_pixmap_board);
	if (_pixmap_world) gdk_pixmap_unref(_pixmap_world);
	if (_colormap) gdk_colormap_unref(_colormap);

}

void CWMain::get_next_piece(void)
{
	if (debug > 1) g_print("CWMain::get_next_piece: %d\n", mds->_next_piece);
	_next_piece = *(mds->pieces[mds->_next_piece]);
	_next_piece.center_piece_to_weight_center();
	// apply rotations
	_next_piece.rotate_x(0.5, NULL, TRUE, FALSE);
	_next_piece.rotate_z(0.5, NULL, TRUE, FALSE);
	// compute offset
	_np_offset = 0;
	for (gint i=0; i<_next_piece._nb_pointes; i++) {
		if (_next_piece._pointes[i]->x > _np_offset)
			_np_offset =  _next_piece._pointes[i]->x;
		if (_next_piece._pointes[i]->y > _np_offset)
			_np_offset =  _next_piece._pointes[i]->y;
	}
}

/* key_pressed
 * IN: key pressed
*/
void CWMain::key_pressed(gint key)
{
	if (debug > 1) g_print("CWMain::key_pressed: >%c<\n",key);
	if (mds->_state == CGame::state_PAUSED) {
		// check for unpause from kb
		if (key == mds->_settings->_keys[key_PAUSE])
		pause(pause_TOGGLE);
		return;
	}
	if (mds->_state != CGame::state_PLAYING) return;
	// put key in lowercase
	if (key >= 'A' && key <='Z') key = key - 'A' + 'a';
	guint *keys =  mds->_settings->_keys;
	if (key == keys[key_UP] || key == GDK_Up)
		mds->add_transfo(CGame::trans_U);
	else if (key == keys[key_RIGHT] || key == GDK_Right)
		mds->add_transfo(CGame::trans_R);
	else if (key == keys[key_DOWN] || key == GDK_Down)
		mds->add_transfo(CGame::trans_D);
	else if (key == keys[key_LEFT] || key == GDK_Left)
		mds->add_transfo(CGame::trans_L);
	else if (key == keys[key_XNEG])
		mds->add_transfo(CGame::trans_XN);
	else if (key == keys[key_XPOS])
		mds->add_transfo(CGame::trans_XP);
	else if (key == keys[key_YNEG])
		mds->add_transfo(CGame::trans_YN);
	else if (key == keys[key_YPOS])
		mds->add_transfo(CGame::trans_YP);
	else if (key == keys[key_ZNEG])
		mds->add_transfo(CGame::trans_ZN);
	else if (key == keys[key_ZPOS])
		mds->add_transfo(CGame::trans_ZP);
	else if (key == keys[key_DROP])
		mds->fast_fall();
	else if (key == keys[key_PAUSE])
		pause(pause_TOGGLE);
	else if (key == keys[key_TOGGLENP])
		toggle_np(toggle_TOGGLE);
}

void CWMain::load_images(void)
{
    if (debug) g_print("CWMain::load_images\n");
	xpm_score = gdk_pixmap_create_from_xpm_d(_wmain->window, &mask_score,
		NULL, score_xpm);
	xpm_level = gdk_pixmap_create_from_xpm_d(_wmain->window, &mask_level,
		NULL, level_xpm);
	xpm_layers = gdk_pixmap_create_from_xpm_d(_wmain->window, &mask_layers,
		NULL, layers_xpm);
	xpm_blocs = gdk_pixmap_create_from_xpm_d(_wmain->window, &mask_blocs,
		NULL, blocs_xpm);
	xpm_cubes = gdk_pixmap_create_from_xpm_d(_wmain->window, &mask_cubes,
		NULL, cubes_xpm);
	xpm_digitssmall = gdk_pixmap_create_from_xpm_d(_wmain->window, NULL,
		NULL, digitssmall_xpm);
	xpm_digitsbig = gdk_pixmap_create_from_xpm_d(_wmain->window, NULL,
		NULL, digitsbig_xpm);
}

/* main loop
*/
void CWMain::loop(void)
{
	if (debug > 2) g_print("CWMain::loop, timer tag=%d\n",_timer_tag);
	if (!_timer_tag) return;

	// do something only if game on-going
	if (mds->_state == CGame::state_NEW
	 || mds->_state == CGame::state_DONE
	 || mds->_state == CGame::state_PAUSED)
		return;
	if (mds->_state == CGame::state_LOST) {
		mds->_state = CGame::state_DONE;
		set_status_bar_message();
		draw_world();
		redraw();
		if (debug > 1)
			g_print(" loop: game done, calling HOF (update required)\n");
		// sound
		if (mds->_settings->_sound_support)
			csound_play_for_event(CSound::snd_HOF);
		whof->hof(TRUE/*update hof*/);
		return;
	}
	// count frames
	_frames++;
		
	// timing
	gulong ms = 0;
	gdouble ms_elapsed_now = g_timer_elapsed(_ms_timer, &ms)*1000;
	ms_elapsed_now += ms/1000;
	gdouble intervale = ms_elapsed_now - _ms_elapsed ;
	_ms_elapsed = ms_elapsed_now;
	if (intervale < MAIN_TIMER) intervale = MAIN_TIMER;
	else if (intervale > 500) intervale = 500;
	//cout << "TIME = " << _ms_elapsed << " INTERVALE = " << intervale << endl; 
	
	// move and redraw
	mds->compute_transfos(intervale*CUBE_PARCOURU_EN_1K_MSEC);
	if (mds->fall() || _no_clipping) {
		// a piece froze
		draw_world(); // update the background
		get_next_piece();
	} else {
		// falling piece rectangle
		gdk_draw_pixmap(_pixmap_board, _gc_board, _pixmap_world, 
			_clip_x1,_clip_y1, _clip_x1,_clip_y1,
			_clip_x2-_clip_x1, _clip_y2-_clip_y1);
		// next piece rectangle
		if (_show_np)
			gdk_draw_pixmap(_pixmap_board, _gc_board, _pixmap_world, 
				0,30, 0,30,
				(gint)(_np_offset*_cnp*2)+20, (gint)(_np_offset*_cnp*2)+20);
	}			
	gint x01 = _clip_x1;
	gint y01 = _clip_y1;
	gint x02 = _clip_x2;
	gint y02 = _clip_y2;
	draw_piece(); // updates clipping rectangle
	if (_clip_x1 < x01) x01 = _clip_x1;
	if (_clip_y1 < y01) y01 = _clip_y1;
	if (_clip_x2 > x02) x02 = _clip_x2;
	if (_clip_y2 > y02) y02 = _clip_y2;
	if (_show_np)
		// draw next piece
		draw_next_piece(intervale*CUBE_PARCOURU_EN_1K_MSEC);
	if (_no_clipping) {
		// redraw the whole world
		redraw();
		_no_clipping = FALSE;
	} else {
		// redraw falling piece rectangle
		redraw(x01,y01,x02,y02);
		// redraw next piece rectangle
		if (_show_np) redraw(0,30,
				(gint)(_np_offset*_cnp*2)+20,60+(gint)(_np_offset*_cnp*2));
	}
}

/* new_game
 * called when the NEW button or menu item is selected
*/
void CWMain::new_game()
{
	if (debug) g_print("CWMain::new_game\n");
	if (mds->_state == CGame::state_PLAYING
	 || mds->_state == CGame::state_PAUSED) {
		// a game is on-going
		// ask if the player really wants a new game
		stop_timer();
		GtkWidget *box;
		box = gnome_message_box_new (
			_("Do you really want to start a new game ?"),
			GNOME_MESSAGE_BOX_QUESTION,
			GNOME_STOCK_BUTTON_YES,
			GNOME_STOCK_BUTTON_NO,
			NULL);
		gnome_dialog_set_parent (GNOME_DIALOG(box), GTK_WINDOW(_wmain));
		gnome_dialog_set_default (GNOME_DIALOG(box), 0);
		gtk_window_set_modal (GTK_WINDOW (box), TRUE);  
		gtk_signal_connect (GTK_OBJECT (box), "clicked",
			(GtkSignalFunc)cb_dialog_new_game_clicked, this);
 		gtk_widget_show(box);
	} else
		// start without asking
		start();
}

/* pause
 * called when pause button/menu item clicked
*/
void CWMain::pause(gint action)
{
	if (debug) g_print("CWMain::pause, action = %d\n",action);
	// makes sens only if there's a running game
	if (mds->_state != CGame::state_PLAYING
	 && mds->_state != CGame::state_PAUSED) return;
	// sound
	if (mds->_settings->_sound_support)
		csound_play_for_event(CSound::snd_PAUSE);
	// setup new state
	if (action == pause_PLAY)
		// force PLAY
		mds->_state = CGame::state_PLAYING;
	else if (action == pause_PAUSE)
		// force PAUSE
		mds->_state = CGame::state_PAUSED;
	else { // toggle
   		if (mds->_state != CGame::state_PAUSED)
			mds->_state = CGame::state_PAUSED;
		else mds->_state = CGame::state_PLAYING;
	}
	//
	_paused_on_lostfocus = FALSE;
   	if (mds->_state != CGame::state_PAUSED) {
		_no_clipping = TRUE;
	}
	// status bar
	set_status_bar_message();
}

/* pause_onfocus
 * called when window grabs or loses focus
*/
void CWMain::pause_onfocus(gint action)
{
	if (debug) g_print("CWMain::pause_onfocus, action = %d\n",action);
	// makes sens only if there's a running game
	if (mds->_state != CGame::state_PLAYING
	 && mds->_state != CGame::state_PAUSED) return;
	// setup new state
	if (action == pause_PLAY) {
		// resume if game is paused due to lost focus and pref. option enabled
		if (mds->_settings->_resume_on_focus
		 && mds->_state == CGame::state_PAUSED
		 && _paused_on_lostfocus) {
			mds->_state = CGame::state_PLAYING;
			_paused_on_lostfocus = FALSE;
		}
	} else if (action == pause_PAUSE) {
		// pause if game is playing and pref. option enabled
		if (mds->_settings->_pause_on_nofocus
		 && mds->_state == CGame::state_PLAYING) {
			mds->_state = CGame::state_PAUSED;
			_paused_on_lostfocus = TRUE;
			// sound
			if (mds->_settings->_sound_support)
				csound_play_for_event(CSound::snd_PAUSE);
	}	}	
	//
	if (mds->_state != CGame::state_PAUSED) {
		_no_clipping = TRUE;
	}
	// status bar
	set_status_bar_message();
}

void CWMain::redraw(gint cx1, gint cy1, gint cx2, gint cy2)
{
	if (debug > 2) g_print("CWMain::redraw world (%d,%d,%d,%d)\n",
		cx1,cy1,cx2,cy2);
	GdkRectangle redrawrect;
	if (cx1 == -1 || _no_clipping) {
		redrawrect.x = 0;
		redrawrect.y = 0;
		redrawrect.width = _w;
		redrawrect.height = _h;
	} else {
		redrawrect.x = cx1;
		redrawrect.y = cy1;
		redrawrect.width = cx2-cx1;
		redrawrect.height = cy2-cy1;
	}
	gtk_widget_draw(_board, &redrawrect);
}

gint CWMain::screen_cote(gfloat z)
{
	return ((gint)(_cu*_k/(z+_k)));
}

gint CWMain::screen_x(gfloat x, gfloat z)
{
	return ((gint)(_x0 + (x-_tx)*_cu*_k/(z+_k)));
}

gint CWMain::screen_y(gfloat y, gfloat z)
{
	return ((gint)(_y0 + (y-_ty)*_cu*_k/(z+_k)));
}

gint CWMain::seb_max(gint n1, gint n2, gint n3, gint n4, gint n5, gint n6)
{
	gint ret = n1;
	if (n2 > ret) ret = n2;
	if (n3 == -1) return ret;	if (n3 > ret) ret = n3;	
	if (n4 == -1) return ret;	if (n4 > ret) ret = n4;	
	if (n5 == -1) return ret;	if (n5 > ret) ret = n5;	
	if (n6 > ret) return n6;
	return ret;	
}

gint CWMain::seb_min(gint n1, gint n2, gint n3, gint n4, gint n5, gint n6)
{
	gint ret = n1;
	if (n2 < ret) ret = n2;
	if (n3 == -1) return ret;	if (n3 < ret) ret = n3;	
	if (n4 == -1) return ret;	if (n4 < ret) ret = n4;	
	if (n5 == -1) return ret;	if (n5 < ret) ret = n5;	
	if (n6 < ret) return n6;
	return ret;	
}

/* set_status_bar_message
 * displays msg in the status bar
*/
void CWMain::set_status_bar_message(gchar *msg)
{
	if (debug > 2) g_print("CWMain::set_status_bar_message\n");
	gnome_appbar_pop(GNOME_APPBAR(_status_bar));
	if (msg == NULL)
		if (mds->_state == CGame::state_PLAYING)
			gnome_appbar_push(GNOME_APPBAR(_status_bar),_("Playing"));
		else if (mds->_state == CGame::state_PAUSED)
			gnome_appbar_push(GNOME_APPBAR(_status_bar),_("GAME PAUSED"));
		else if (mds->_state == CGame::state_DONE)
			gnome_appbar_push(GNOME_APPBAR(_status_bar),_("Game Over"));
		else if (mds->_state == CGame::state_DEMO)
			gnome_appbar_push(GNOME_APPBAR(_status_bar),_("Demo mode"));
		else gnome_appbar_push(GNOME_APPBAR(_status_bar),_("Ready"));  
	else gnome_appbar_push(GNOME_APPBAR(_status_bar),msg);
}

void CWMain::set_status_bar_settings(void)
{
	if (debug > 2) g_print("CWMain::set_status_bar_settings\n");
	gchar ss[16];
	sprintf(ss,"%c.%dx%dx%d",
		(CSettings::diff_descriptors[mds->_settings->_difficulty])[0],
		mds->_settings->_w,	mds->_settings->_h, mds->_settings->_d);
	gtk_label_set(GTK_LABEL(_lbl_settings),ss);
}
		
void CWMain::set_window_geometry(gint xpos, gint ypos, gint w, gint h)
{
	if (debug) g_print("CWMain::set_window_geometry (x=%d, y=%d, w=%d, h=%d)\n",
		xpos, ypos, w, h);
	if (xpos != 0 && ypos != 0)
		gtk_widget_set_uposition(_wmain, xpos, ypos);
	gtk_window_set_default_size(GTK_WINDOW(_wmain), w, h);
}

void CWMain::save_window_geometry()
{
	// save window geometry in config file
	if (debug) g_print("CWMain::save_window_geometry:");
	gint w = _wmain->allocation.width;
	gint h = _wmain->allocation.height;
	if (debug) g_print("w=%d, h=%d\n",w,h);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_WINDOWWIDTH, w);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_WINDOWHEIGHT, h);
	gnome_config_sync();
}

/* show_about
*/
void CWMain::show_about(void)
{
	if (debug) g_print("CWMain::show_about\n");
	GtkWidget *window_about;
	gchar *author = GNO3DTET_AUTHOR" "GNO3DTET_EMAIL;
	const gchar *authors[] = {
		author,
		NULL
	};
	gchar *comments = g_strdup_printf(_(\
	   "%s - The GNOME 3D Tetris game -\n\
		Date Created:  %s\n\
		Date Modified: %s\n\
		%s WWW Homepage at %s"),
		GNO3DTET_EXENAME,
		GNO3DTET_DATECREATED,
		GNO3DTET_DATEMODIFIED,
		GNO3DTET_EXENAME, GNO3DTET_URL);
	window_about = gnome_about_new(
		GNO3DTET_NAME,
		GNO3DTET_VERSION,
		_("(C) 1999, 2000, 2001 Sebastien Nicoud"),
		(const char **)authors,
		comments,
		NULL);
//	gnome_dialog_set_parent(GNOME_DIALOG(window_about),	GTK_WINDOW(_wmain));
	gtk_widget_show(window_about);
	g_free(comments);
}

/* show _license
*/
void CWMain::show_license(void)
{
	if (debug) g_print("CWMain::show_license\n");
	GnomeHelpMenuEntry help_entry;
	help_entry.name = GNO3DTET_EXENAME;
	help_entry.path = LICENSE_FILE;
	gnome_help_display(NULL/*ignored*/,&help_entry); // call the default browser
}

/* show_settings
*/
void CWMain::show_settings(void)
{
	if (debug) g_print("CWMain::show_settings\n");
	wsets->show(_wmain);
}

void CWMain::stop_timer(void)
{
	if (debug) g_print("CWMain::stop_timer, tag=%d\n",_timer_tag);
	_timer_tag = FALSE;
	g_timer_stop(_ms_timer);
//	if (_timer_tag) gtk_timeout_remove(_timer_tag);
}

void CWMain::start_timer(void)
{
	// for some reason, gtk_timeout_remove-ed timer can't be restarted
	// so the timer is on until the app is killed
	if (debug) g_print("CWMain::start_timer, start main timer (%d msec)\n",
		MAIN_TIMER);
	if (!_timer_tag) {
//		_timer_tag = gtk_timeout_add(MAIN_TIMER, cb_timer_called, this);
//		if (debug) g_print("  gtk_timeout_add returned %d\n",_timer_tag);
	} else if (debug) g_print("  timer already started: tag=%d\n",_timer_tag);
	// start main timer
	_timer_tag = TRUE;
	// start seconds timer
	_ms_timer = g_timer_new();
	g_timer_reset(_ms_timer);
}	

/* start
 * inits data for a new game and start it
 * if load_game != NULL, loads load_game
*/
void CWMain::start(CGame *load_game)
{
	// sound
	if (mds->_settings->_sound_support)
		csound_play_for_event(CSound::snd_NEW);

	gboolean new_game = FALSE;
	if (load_game == NULL) new_game = TRUE;
	if (debug) if (new_game)
		g_print("CWMain::start (new game)\n");
		else g_print("CWMain::start (load game)\n");
		
	// setup game settings & inits everything
	if (new_game) mds->init();
	else *mds = *load_game;

	// update view settings from saved settings
	toggle_all(toggle_UPDATE);
	toggle_sound_support(toggle_UPDATE);
	
	// init next piece
	get_next_piece();
	
	// offsets
	_tx = mds->_hw;
	_ty = mds->_hh;
	if (debug) g_print("  world offsets: tx=%.2f, ty=%.2f\n",_tx,_ty);
	
	// write settings in status bar
	set_status_bar_settings();

	// start game
	mds->start();

	// update status bar message
	set_status_bar_message();

	// redraw world, avoids a bunch of GtkCritical errors
	cb_board_configure_event(_board, NULL, this);
	
	// main loop timer + clock
	start_timer();
}

void CWMain::toggle_all(gint action)
{
	if (debug) g_print("CWMain::toggle_all %d\n", action);
	toggle_np(action, FALSE/* refresh all in once */); 
	toggle_scale(action, FALSE);
	toggle_score(action, FALSE);
	toggle_stats(action, FALSE);
	// redraw screen once (refresh all in once is here)
	compute_drawing_values();
	cb_board_configure_event(_board, NULL, this);
	redraw();
}

void CWMain::toggle_np(gint action, gboolean refresh)
{
	if (debug) g_print("CWMain::toggle_np\n");
	if (action == toggle_ON)
		GTK_CHECK_MENU_ITEM(_menu_item_np)->active = TRUE;
	else if (action == toggle_OFF)
		GTK_CHECK_MENU_ITEM(_menu_item_np)->active = FALSE;
	else if (action == toggle_TOGGLE)
		GTK_CHECK_MENU_ITEM(_menu_item_np)->active =
			!GTK_CHECK_MENU_ITEM(_menu_item_np)->active;
	else if (action == toggle_UPDATE)
		GTK_CHECK_MENU_ITEM(_menu_item_np)->active = settings->_show_np;
	_show_np = GTK_CHECK_MENU_ITEM(_menu_item_np)->active;
	_no_clipping = TRUE;
	if (refresh) {
		cb_board_configure_event(_board, NULL, this);
		redraw();
	}
}

void CWMain::toggle_scale(gint action, gboolean refresh)
{
	if (debug) g_print("CWMain::toggle_scale\n");
	if (action == toggle_ON)
		GTK_CHECK_MENU_ITEM(_menu_item_scale)->active = TRUE;
	else if (action == toggle_OFF)
		GTK_CHECK_MENU_ITEM(_menu_item_scale)->active = FALSE;
	else if (action == toggle_TOGGLE)
		GTK_CHECK_MENU_ITEM(_menu_item_scale)->active =
			!GTK_CHECK_MENU_ITEM(_menu_item_scale)->active;
	else if (action == toggle_UPDATE)
		GTK_CHECK_MENU_ITEM(_menu_item_scale)->active = settings->_show_scale;
	_show_scale = GTK_CHECK_MENU_ITEM(_menu_item_scale)->active;
	_no_clipping = TRUE;
	if (refresh) {
		compute_drawing_values();
		cb_board_configure_event(_board, NULL, this);
		redraw();
	}
}

void CWMain::toggle_score(gint action, gboolean refresh)
{
	if (debug) g_print("CWMain::toggle_score\n");
	if (action == toggle_ON)
		GTK_CHECK_MENU_ITEM(_menu_item_score)->active = TRUE;
	else if (action == toggle_OFF)
		GTK_CHECK_MENU_ITEM(_menu_item_score)->active = FALSE;
	else if (action == toggle_TOGGLE)
		GTK_CHECK_MENU_ITEM(_menu_item_score)->active =
			!GTK_CHECK_MENU_ITEM(_menu_item_score)->active;
	else if (action == toggle_UPDATE)
		GTK_CHECK_MENU_ITEM(_menu_item_score)->active = settings->_show_score;
	_show_score = GTK_CHECK_MENU_ITEM(_menu_item_score)->active;
	_show_level = _show_score;
	_no_clipping = TRUE;
	if (refresh) {
		compute_drawing_values();
		cb_board_configure_event(_board, NULL, this);
		redraw();
	}
}

void CWMain::toggle_sound_support(gint action)
{
	if (debug) g_print("CWMain::toggle_sound_support\n");
	if (action == toggle_ON)
		GTK_CHECK_MENU_ITEM(_menu_item_sound_support)->active = TRUE;
	else if (action == toggle_OFF)
		GTK_CHECK_MENU_ITEM(_menu_item_sound_support)->active = FALSE;
	else if (action == toggle_TOGGLE)
		GTK_CHECK_MENU_ITEM(_menu_item_sound_support)->active =
			!GTK_CHECK_MENU_ITEM(_menu_item_sound_support)->active;
	else if (action == toggle_UPDATE)
		GTK_CHECK_MENU_ITEM(_menu_item_sound_support)->active =
			settings->_sound_support;
	if (mds != NULL && mds->_settings != NULL) // existence is checked
		// because this function is also called from main, before
		// the CGame instantiation
		mds->_settings->_sound_support =
			GTK_CHECK_MENU_ITEM(_menu_item_sound_support)->active;
}

void CWMain::toggle_stats(gint action, gboolean refresh)
{
	if (debug) g_print("CWMain::toggle_stats\n");
	if (action == toggle_ON)
		GTK_CHECK_MENU_ITEM(_menu_item_stats)->active = TRUE;
	else if (action == toggle_OFF)
		GTK_CHECK_MENU_ITEM(_menu_item_stats)->active = FALSE;
	else if (action == toggle_TOGGLE)
		GTK_CHECK_MENU_ITEM(_menu_item_stats)->active =
			!GTK_CHECK_MENU_ITEM(_menu_item_stats)->active;
	else if (action == toggle_UPDATE)
		GTK_CHECK_MENU_ITEM(_menu_item_stats)->active = settings->_show_stats;
	_show_stats = GTK_CHECK_MENU_ITEM(_menu_item_stats)->active;
	_no_clipping = TRUE;
	if (refresh) {
		compute_drawing_values();
		cb_board_configure_event(_board, NULL, this);
		redraw();
	}
}

void CWMain::toggle_toolbar(gint action)
{
	if (debug) g_print("CWMain::toggle_toolbar\n");
	if (action == toggle_ON)
		GTK_CHECK_MENU_ITEM(_menu_item_toolbar)->active = TRUE;
	else if (action == toggle_OFF)
		GTK_CHECK_MENU_ITEM(_menu_item_toolbar)->active = FALSE;
	else if (action == toggle_TOGGLE)
		GTK_CHECK_MENU_ITEM(_menu_item_toolbar)->active =
			!GTK_CHECK_MENU_ITEM(_menu_item_toolbar)->active;
	gboolean state = GTK_CHECK_MENU_ITEM(_menu_item_toolbar)->active;
	if (state)
		gtk_widget_show(GTK_WIDGET(_toolbar_gdi));
	else gtk_widget_hide(GTK_WIDGET(_toolbar_gdi));
	gtk_widget_queue_resize(_wmain);
	// save setting in the config file
	gnome_config_set_bool(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_SHOW_TOOLBAR, state);
	gnome_config_sync();
	compute_drawing_values();
}

void CWMain::update_status_bar(void)
{
	if (debug > 2) g_print("CWMain::update_status_bar\n");
	
	// update frames / sec
	gchar *sframes = g_strdup_printf("%d f/sec",_frames);
	gtk_label_set(GTK_LABEL(_lbl_frames),sframes);
	g_free(sframes);
	_frames = 0;

 	// updated timer
	mds->inc_timer();
	gchar *stime = mds->stringify_timer();
	gtk_label_set(GTK_LABEL(_lbl_timer),stime);
	g_free(stime);
}

/* cb_board_configure_event
 * called when the main window is resized
*/ 
gint CWMain::cb_board_configure_event(GtkWidget *widget,
	GdkEventConfigure *event, gpointer data)
{
	if (debug > 2) g_print("CWMain::cb_board_configure_event\n");

	if (mds == NULL) return TRUE;
	if (mds->_settings == NULL) return TRUE;

	CWMain *window = (CWMain *)data;
	window->_w = widget->allocation.width;
	window->_h = widget->allocation.height;	
	window->compute_drawing_values();

	if (_pixmap_board) gdk_pixmap_unref(_pixmap_board);
	_pixmap_board = gdk_pixmap_new(widget->window, 
		window->_w, window->_h, -1);

	if (_pixmap_world) gdk_pixmap_unref(_pixmap_world);
	_pixmap_world = gdk_pixmap_new(widget->window, 
		window->_w, window->_h, -1);

	window->draw_world();
	// draw stuff if paused. If not, loop() will take care of it
	if (mds->_state == CGame::state_PAUSED) window->draw_piece();
	if (mds->_state == CGame::state_PAUSED && window->_show_np)
		window->draw_next_piece(0);
	
	return TRUE; // don't propagate the signal
}

/* cb_board_expose_event
 * called when the the board needs to be repaint
*/ 
gint CWMain::cb_board_expose_event(GtkWidget *widget,
	GdkEventConfigure *event, gpointer data)
{
	if (debug > 2) g_print("CWMain::cb_board_expose_event\n");
	if (_pixmap_board != NULL) gdk_draw_pixmap(
		widget->window,
		widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
		_pixmap_board,
		event->x, event->y,
		event->x, event->y,
		event->width, event->height);
		return FALSE; // propagate the signal
}

/* dialog_new_game_clicked
 * Dialog Window: Start a new game ?
 * button 0 = YES, button 1 = NO
*/
void CWMain::cb_dialog_new_game_clicked(GtkWidget *widget, gint button
	, gpointer data)
{
	if (debug > DBG_CB) g_print(
		"CWMain::cb_dialog_new_game_clicked: button = %d\n", button);
	if (button == 0) ((CWMain *)data)->start();
	else ((CWMain *)data)->start_timer();
}

/* cb_dialog_quit_game_clicked
 * Dialog Window: Quit the game ?
 * button 0 = YES, button 1 = NO
*/
void CWMain::cb_dialog_quit_game_clicked(GtkWidget *widget, gint button)
{
	if (debug > DBG_CB) g_print("CWMain::cb_dialog_quit_game_clicked: \
button = %d\n", button);
	if (button == 0) gtk_main_quit ();
}

/* cb_key_press_event
 * event callback when a key is pressed
*/ 
gint CWMain::cb_key_press_event(GtkWidget *widget, GdkEvent *event,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWMain::cb_key_press_event\n");
	((CWMain *)data)->key_pressed((gint)(((GdkEventKey *)event)->keyval));
}

/* cb_menu_game_new_activate
 * called when NEW button/menu clicked
*/
void CWMain::cb_menu_game_new_activate(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWMain::cb_menu_game_new_activate\n");
	((CWMain *)data)->new_game();
}

/* cb_menu_game_pause_activate
 * called when PAUSE button/menu clicked
*/
void CWMain::cb_menu_game_pause_activate(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWMain::cb_menu_game_pause_activate, toggled\n");
	((CWMain *)data)->pause(pause_TOGGLE);
}

/* menu_game_hof_activate
 * called when HOF button/menu clicked
*/
void CWMain::cb_menu_game_hof_activate(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWMain::cb_menu_game_hof_activate\n");
	whof->hof(FALSE/*no update required*/);
}

void CWMain::cb_menu_game_exit_activate(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWMain::cb_menu_game_exit_activate\n");
	((CWMain *)data)->exit();
}

void CWMain::cb_menu_settings_hide_all_activate(
	GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB)
		g_print("CWMain::cb_menu_settings_hide_all_activate\n");
	((CWMain *)data)->toggle_all(CWMain::toggle_OFF);
}

void CWMain::cb_menu_settings_next_piece_check_toggled(
	GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB)
		g_print("CWMain::cb_menu_settings_next_piece_check_toggled\n");
	((CWMain *)data)->toggle_np();
}

void CWMain::cb_menu_settings_preferences_activate(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWMain::cb_settings_preferences_activate\n");
//	((CWMain *)data)->pause(pause_PAUSE);
	((CWMain *)data)->show_settings();
}

void CWMain::cb_menu_settings_scale_check_toggled(
	GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB)
		g_print("CWMain::cb_menu_settings_scale_check_activate\n");
	((CWMain *)data)->toggle_scale();
}

void CWMain::cb_menu_settings_score_check_toggled(
	GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB)
		g_print("CWMain::cb_menu_settings_score_check_activate\n");
	((CWMain *)data)->toggle_score();
}

void CWMain::cb_menu_settings_stats_check_toggled(
	GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB)
		g_print("CWMain::cb_menu_settings_stats_check_activate\n");
	((CWMain *)data)->toggle_stats();
}

void CWMain::cb_menu_settings_show_all_activate(
	GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB)
		g_print("CWMain::cb_menu_settings_show_all_activate\n");
	((CWMain *)data)->toggle_all(CWMain::toggle_ON);
}

void CWMain::cb_menu_settings_soundsupport_check_toggled(
	GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB)
		g_print("CWMain::cb_menu_settings_soundsupport_check_toggled\n");
	((CWMain *)data)->toggle_sound_support();
}

void CWMain::cb_menu_settings_toolbar_check_toggled(
	GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB)
		g_print("CWMain::cb_menu_settings_toolbar_check_toggled\n");
	((CWMain *)data)->toggle_toolbar();
}

void CWMain::cb_menu_help_about_activate(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWMain::cb_menu_help_about_activate\n");
	((CWMain *)data)->show_about();
}

void CWMain::cb_menu_help_license_activate(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWMain::cb_menu_help_license_activate\n");
	((CWMain *)data)->show_license();
}	

/* cb_timer_called
 * function called by main timer every MAIN_TIMER msec
*/
gint CWMain::cb_timer_called(gpointer data)
{
	if (mds->_state == CGame::state_PLAYING
	 || mds->_state == CGame::state_DEMO
	 || mds->_state == CGame::state_LOST)
	 	((CWMain *)data)->loop();
	return TRUE; /*TRUE = timer not stopped, FALSE stops timer*/
}

/* cb_timer1000_called
 * function called by timer every 1000 msec
*/
gint CWMain::cb_timer1000_called(gpointer data)
{
	if (mds->_state == CGame::state_PLAYING
	 || mds->_state == CGame::state_DEMO) {
	 	((CWMain *)data)->update_status_bar();
		return TRUE/*timer not stopped*/;
}	}

/* cb_wmain_delete_event
 * quits the main window on WM close event
*/
void CWMain::cb_wmain_delete_event(GtkWidget *widget, GdkEvent *event,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWMain::cb_wmain_delete_event\n");
	gtk_main_quit();
}

/* cb_wmain_focus_*_event
 *  called when wmain grabs or loses the focus, respectively
*/
void CWMain::cb_wmain_focus_in_event(GtkWidget *widget, GdkEvent *event,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWMain::cb_wmain_focus_in_event\n");
	((CWMain *)data)->pause_onfocus(pause_PLAY);
}

void CWMain::cb_wmain_focus_out_event(GtkWidget *widget, GdkEvent *event,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWMain::cb_wmain_focus_out_event\n");
	((CWMain *)data)->pause_onfocus(pause_PAUSE);
}
