/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999, 2000, 2001 Sebastien Nicoud <seb@eseb.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#ifndef _CWMAIN
#define _CWMAIN

#include "common.h"

class CWMain
{
// typedefs, consts
private:
	static const gint _k;
	static const gchar *_color_descriptors[MAX_WORLD_DEPTH];
	static const gchar *_gamebar_name;
public:
	enum {
		pause_PLAY,
		pause_PAUSE,
		pause_TOGGLE
	};
	enum {
		toggle_UPDATE,
		toggle_ON,
		toggle_OFF,
		toggle_TOGGLE,
		toggle_KEEP
	};
	
// attributes
private:
	GtkWidget			*_wmain;
	static GdkGC 		*_gc_board;
	GtkWidget			*_board;
	static GdkPixmap	*_pixmap_board;
	static GdkPixmap	*_pixmap_world;
	GtkWidget			*_status_bar;
	GtkWidget			*_lbl_settings;
	GtkWidget			*_lbl_frames;
	GtkWidget			*_lbl_timer;
	GtkWidget 			*_toolbar;
	GnomeDockItem		*_toolbar_gdi;

	gboolean 			 _timer_tag; // main timer running
	gint				 _w; // board width, in pixels
	gint				 _h; // board height, in pixels
	gint				 _x0; // board center, in pixels
	gint				 _y0; //  :
	gfloat				 _tx; // world center, in cubes
	gfloat				 _ty; //  :
	gint 				 _cu; // top cube cote, in pixels
	gint				 _cnp; // next piece cube cote, in pixels
	gint				 _scale_ch; // scale height, in pixels
	gint32				 _frames; // frames per seconds
	GTimer				*_ms_timer; // microseconds timer
	gdouble 			 _ms_elapsed; // millisecond d-time
	gboolean			 _paused_on_lostfocus;

	GdkColormap 		*_colormap;
	GdkColor 			 _colors[MAX_WORLD_DEPTH][3];
	GdkColor			 _color_world;
	GdkColor			 _color_piece;
	GdkColor			 _color_scale;
	GdkColor			 _white, _black;

	GdkPixmap 			*xpm_score;
	GdkBitmap 			*mask_score;
	GdkPixmap 			*xpm_level;
	GdkBitmap 			*mask_level;
	GdkPixmap 			*xpm_layers;
	GdkBitmap 			*mask_layers;
	GdkPixmap 			*xpm_blocs;
	GdkBitmap 			*mask_blocs;
	GdkPixmap 			*xpm_cubes;
	GdkBitmap 			*mask_cubes;
	GdkPixmap 			*xpm_digitssmall;
	GdkPixmap 			*xpm_digitsbig;
	CPiece				 _next_piece;
	GtkWidget			*_menu_item_toolbar;
	GtkWidget			*_menu_item_sound_support;
	GtkWidget			*_menu_item_np;
	gboolean			 _show_np;
	GtkWidget			*_menu_item_score;
	gboolean			 _show_score;
	gboolean			 _show_level;
	GtkWidget			*_menu_item_scale;
	gboolean			 _show_scale;
	gboolean 			 _no_clipping;
	GtkWidget			*_menu_item_stats;
	gboolean			 _show_stats;
	gfloat				 _np_offset;
	gint				 _clip_x1, _clip_y1;
	gint				 _clip_x2, _clip_y2;
		
// contructor
public:
	CWMain(); // constructeur par defaut
	
// destructor
public:
	~CWMain();

// operators
public:
	friend ostream &operator<< (ostream &os, CWMain &ind); // output stream

// methods
public:
	void create_timer(gint timer = MAIN_TIMER);
	void draw_all(gboolean need_cleaning = FALSE);
	void save_window_geometry(void);
	void set_window_geometry(gint xpos, gint ypos, gint w, gint h);
	void show_about(void);
	void show_license(void);
	void show_settings(void);
	void toggle_all(gint action = toggle_KEEP);
	void toggle_toolbar(gint action = toggle_KEEP);
	void toggle_np(gint action = toggle_KEEP, gboolean refresh = TRUE);
	void toggle_scale(gint action = toggle_KEEP, gboolean refresh = TRUE);
	void toggle_score(gint action = toggle_KEEP, gboolean refresh = TRUE);
	void toggle_stats(gint action = toggle_KEEP, gboolean refresh = TRUE);
	void toggle_sound_support(gint action = toggle_KEEP);
private:
	void compute_drawing_values(void);
	gint compute_numbers(gint32 n, gint *d, gint nbd);
	void create_status_bar(void);
	void draw_next_piece(gfloat dt);
	void draw_number(gint n, gint x, gint y,
			gint nb_of_digits = 7, 
			gint format = 0/*no padding*/,
			gboolean big_or_not_small_digits = TRUE,
			gboolean print_commas = TRUE,
			gboolean right_or_not_left = TRUE);
	void draw_line(gfloat xa, gfloat ya, gfloat za,
			gfloat xb, gfloat yb, gfloat zb,
			gboolean board_drawing = TRUE, gboolean update_clip = TRUE);
	void draw_flat_rectangle(gfloat xa, gfloat ya, gfloat za,
			gfloat w, gfloat h);
	void draw_piece(void);
	void draw_set_piece(gint x, gint y, gint z);
	void draw_world(void);
	void exit(void);
	void free_memory(void);
	void get_next_piece(void);
	void key_pressed(gint key);
	void load_images(void);
	void loop(void);
	void new_game(void);
	void pause(gint action);
	void pause_onfocus(gint action);
	void redraw(gint cx1=-1, gint cy1=0, gint cx2=0, gint cy2=0);
	gint screen_cote(gfloat z);
	gint screen_x(gfloat x, gfloat z);
	gint screen_y(gfloat y, gfloat z);
	gint seb_max(gint n1, gint n2,
			gint n3=-1, gint n4=-1, gint n5=-1, gint n6=-1);
	gint seb_min(gint n1, gint n2,
			gint n3=-1, gint n4=-1, gint n5=-1, gint n6=-1);
	void set_status_bar_message(gchar *msg = NULL);
	void set_status_bar_settings(void);
	void start_timer(void);
	void stop_timer(void);
	void start(CGame *load_game = NULL);
	void update_status_bar(void);
	
private: // callbacks
	static gint cb_board_configure_event(GtkWidget *widget,
					GdkEventConfigure *event, gpointer data);
	static gint cb_board_expose_event(GtkWidget *widget,
					GdkEventConfigure *event, gpointer data);
	static void cb_dialog_new_game_clicked(GtkWidget *widget, gint button,
					gpointer data);
	static void cb_dialog_quit_game_clicked(
					GtkWidget *widget, gint button);
	static gint cb_key_press_event(GtkWidget *widget, GdkEvent *event,
					gpointer data);
	static void cb_menu_game_new_activate(GtkWidget *widget, gpointer data);
	static void cb_menu_game_pause_activate(GtkWidget *widget, gpointer data);
	static void cb_menu_game_hof_activate(GtkWidget *widget, gpointer data);
	static void cb_menu_settings_preferences_activate(GtkWidget *widget, gpointer data);
	static void cb_menu_settings_hide_all_activate(
					GtkWidget *widget, gpointer data);
	static void cb_menu_settings_scale_check_toggled(
					GtkWidget *widget, gpointer data);
	static void cb_menu_settings_score_check_toggled(
					GtkWidget *widget, gpointer data);
	static void cb_menu_settings_stats_check_toggled(
					GtkWidget *widget, gpointer data);
	static void cb_menu_settings_show_all_activate(
					GtkWidget *widget, gpointer data);
	static void cb_menu_settings_soundsupport_check_toggled(
					GtkWidget *widget, gpointer data);
	static void cb_menu_settings_toolbar_check_toggled(
					GtkWidget *widget, gpointer data);
	static void cb_menu_settings_next_piece_check_toggled(
					GtkWidget *widget, gpointer data);
	static void cb_menu_help_about_activate(GtkWidget *widget, gpointer data);
	static void cb_menu_help_license_activate(GtkWidget *widget, gpointer data);
	static void cb_menu_game_exit_activate(
					GtkWidget *widget, gpointer data);
	static gint cb_timer_called(gpointer data);
	static gint cb_timer1000_called(gpointer data);
	static void cb_wmain_delete_event(
					GtkWidget *widget, GdkEvent *event, gpointer data);
	static void cb_wmain_focus_in_event(
					GtkWidget *widget, GdkEvent *event,	gpointer data);
	static void cb_wmain_focus_out_event(
					GtkWidget *widget, GdkEvent *event,	gpointer data);

};

#endif
