/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999, 2000, 2001 Sebastien Nicoud <seb@eseb.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#include "common.h"
#include "csound.h"
#include "csettings.h"
#include "cpiece.h"
#include "cwmain.h"
#include "cgame.h"
#include "cwhof.h"
#include "cwsettings.h"

gboolean CWSettings::_viewed = false;

CWSettings::CWSettings()
{
	if (debug) g_print("CWSettings::CWSettings()\n");
	_parent = NULL;
	_settings = new CSettings();
	_key = key_NONE;
	for (gint i=0; i<key_NONE; i++) _key_buttons[i] = NULL;
	_sound_list = NULL;
	_current_sound = -1;
}

CWSettings::~CWSettings()
{
	if (debug) g_print("CWSettings::~CWSettings()\n");
	free_memory();
}

void CWSettings::apply(void)
{
	if (debug) g_print("CWSettings::apply()\n");
	// permanent settings update - take effect at NEW game
	settings->_w = _settings->_w;
	settings->_h = _settings->_h;
	settings->_d = _settings->_d;
	settings->_difficulty = _settings->_difficulty;
	settings->_speed = _settings->_speed;
	settings->_show_np = _settings->_show_np;
	settings->_show_scale = _settings->_show_scale;
	settings->_show_score = _settings->_show_score;
	settings->_show_stats = _settings->_show_stats;
	for (gint i=0; i<key_NONE; i++)
		settings->_keys[i] = _settings->_keys[i];
	settings->_sound_support = _settings->_sound_support;
	for (gint i=0; i<NUMBER_SOUND_EVENT; i++)
		sounds[i] = _sounds[i];
	settings->_pause_on_nofocus = _settings->_pause_on_nofocus;
	settings->_resume_on_focus = _settings->_resume_on_focus;
	settings->_nhofl = _settings->_nhofl;
	settings->_show_old_score = _settings->_show_old_score;

	// live update - take affect immediately
	if (mds->_state != CGame::state_PLAYING
	 && mds->_state != CGame::state_PAUSED) {
		// update the world settings if there isn't any game going on
		mds->_settings->_w = _settings->_w;
		mds->_settings->_h = _settings->_h;
		mds->_settings->_d = _settings->_d;
		mds->_settings->_difficulty = _settings->_difficulty;
	}	
	mds->_settings->_speed = _settings->_speed;
	mds->_settings->_show_np = _settings->_show_np;
	mds->_settings->_show_scale = _settings->_show_scale;
	mds->_settings->_show_score = _settings->_show_score;
	mds->_settings->_show_stats = _settings->_show_stats;
	mwin->draw_all(TRUE/*clean world*/); // update world if no game on-going
	for (gint i=0; i<key_NONE; i++)
		mds->_settings->_keys[i] = _settings->_keys[i];
	mds->_settings->_sound_support = _settings->_sound_support;
	mwin->toggle_sound_support(CWMain::toggle_UPDATE);
	mds->_settings->_pause_on_nofocus = _settings->_pause_on_nofocus;
	mds->_settings->_resume_on_focus = _settings->_resume_on_focus;
	mds->_settings->_nhofl = _settings->_nhofl;
	mds->_settings->_show_old_score = _settings->_show_old_score;
	
	// save the settings in the init file (~/.gnome/gno3dtet)
	// settings
	gnome_config_set_int(GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_WIDTH,
		settings->_w);
	gnome_config_set_int(GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_HEIGHT,
		settings->_h);
	gnome_config_set_int(GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_DEPTH,
		settings->_d);
	gnome_config_set_int(GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_DIFF,
		settings->_difficulty);
	gnome_config_set_int(GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_SPEED,
		settings->_speed);
	// view
	gnome_config_set_bool(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_SHOW_NEXT_PIECE,
		settings->_show_np);
	gnome_config_set_bool(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_SHOW_SCALE,
		settings->_show_scale);
	gnome_config_set_bool(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_SHOW_SCORE,
		settings->_show_score);
	gnome_config_set_bool(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_SHOW_STATS,
		settings->_show_stats);
	// keys
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_XNEG,
		settings->_keys[key_XNEG]);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_XPOS,
		settings->_keys[key_XPOS]);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_YNEG,
		settings->_keys[key_YNEG]);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_YPOS,
		settings->_keys[key_YPOS]);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_ZNEG,
		settings->_keys[key_ZNEG]);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_ZPOS,
		settings->_keys[key_ZPOS]);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_LEFT,
		settings->_keys[key_LEFT]);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_RIGHT,
		settings->_keys[key_RIGHT]);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_DOWN,
		settings->_keys[key_DOWN]);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_UP,
		settings->_keys[key_UP]);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_DROP,
		settings->_keys[key_DROP]);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_PAUSE,
		settings->_keys[key_PAUSE]);
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_KEY_TOGGLENP,
		settings->_keys[key_TOGGLENP]);
	// sound
	gnome_config_set_bool(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_SOUNDSUPPORT,
		settings->_sound_support);
	for (gint i=0; i<NUMBER_SOUND_EVENT; i++) sounds[i].save();
	// HOF
	gnome_config_set_int(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_NHOFL,
		settings->_nhofl);
	gnome_config_set_bool(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_SHOW_OLD_SCORE,
		settings->_show_old_score);
	// app
	gnome_config_set_bool(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_PAUSE_LOSTFOCUS,
		settings->_pause_on_nofocus);
	gnome_config_set_bool(
		GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_RESUME_FOCUS,
		settings->_resume_on_focus);	
	// flush
	gnome_config_sync();
}

void CWSettings::browse_sound_files(void)
{
	if (debug) g_print("CWSettings::browse_sound_files\n");
}

void CWSettings::change_depth(void)
{
	if (debug > 1) g_print("CWSettings::change_depth\n");
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
	_settings->_d = ((gint)GTK_ADJUSTMENT(_adj_depth)->value);	
}

void CWSettings::change_difficulty(gint d)
{
	if (debug > 1) g_print("CWSettings::change_difficulty to %d\n",d);
	_settings->_difficulty = d;
	update_difficulty_label();
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
}

void CWSettings::change_height(void)
{
	if (debug > 1) g_print("CWSettings::change_height\n");
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
	_settings->_h = ((gint)GTK_ADJUSTMENT(_adj_height)->value);	
}

void CWSettings::change_nhofl(gint value)
{
	if (debug > 1) g_print("CWSettings::change_nhofl to: %d\n", value);
	_settings->_nhofl = value;	
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
}

void CWSettings::change_sound_event(void)
{
	if (debug > 1) g_print("CWSettings::change_sound_event\n");
	// get event number
	_current_sound = csound_get_event_number(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(_combo)->entry)));
	if (_current_sound == -1) {
		// not found, invalid entry
		// number label
		gchar str[8];
		sprintf(str,"? / %d",NUMBER_SOUND_EVENT);
		gtk_label_set_text(GTK_LABEL(_sound_number_lbl), str);
		// note
		gtk_label_set_text(GTK_LABEL(_sound_event_note),
			_("< Invalid event ! >"));
		// enable
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(_check_sound), FALSE);
		gtk_widget_set_sensitive(_check_sound, FALSE);
		// file
		gtk_entry_set_text(GTK_ENTRY(_sound_entry),"");
		gtk_widget_set_sensitive(_sound_entry, FALSE);
		// buttons
		gtk_widget_set_sensitive(_browse_sound_button, FALSE);
		gtk_widget_set_sensitive(_play_sound_button, FALSE);
		gtk_widget_set_sensitive(_default_sound_button, FALSE);
	} else {
		// number label
		gchar str[8];
		sprintf(str,"%d / %d",_current_sound+1, NUMBER_SOUND_EVENT);
		gtk_label_set_text(GTK_LABEL(_sound_number_lbl), str);
		// note
		gtk_label_set_text(GTK_LABEL(_sound_event_note),
			_sounds[_current_sound]._note);
		// enable
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_check_sound),
			_sounds[_current_sound]._played);
		gtk_widget_set_sensitive(_check_sound, TRUE);
		// file
		gtk_entry_set_text(GTK_ENTRY(_sound_entry),
			_sounds[_current_sound]._file);
		gtk_widget_set_sensitive(_sound_entry, TRUE);
		// buttons
		gtk_widget_set_sensitive(_browse_sound_button, TRUE);
		gtk_widget_set_sensitive(_play_sound_button, TRUE);
		gtk_widget_set_sensitive(_default_sound_button, TRUE);
	}
}

void CWSettings::change_sound_file(void)
{
	if (debug) g_print("CWSettings::change_sound_file (%d)\n", _current_sound);
	if (_current_sound == -1) return; // invalid event
	strcpy(_sounds[_current_sound]._file,
		gtk_entry_get_text(GTK_ENTRY(_sound_entry)));
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
}

void CWSettings::change_speed(gint d)
{
	if (debug > 1) g_print("CWSettings::change_speed to %d\n",d);
	_settings->_speed = d;
	update_speed_label();
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
}

void CWSettings::change_width(void)
{
	if (debug > 1) g_print("CWSettings::change_width\n");
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
	_settings->_w = ((gint)GTK_ADJUSTMENT(_adj_width)->value);	
}

gint CWSettings::clean_all_scores(gboolean confirmed)
{
	if (debug > 1) g_print("CWSettings::clean_all_scores confirmed=%d\n", confirmed);
	if (confirmed) {
		// do it
		ofstream fout;
		gchar *filename = g_strdup(GNO3DTET_HOFABSFILE);
		fout.open(filename, ios::out | ios::trunc);
		if (!fout.is_open()) {
			if (debug) g_print("  opening >%s< failed\n",filename);
			// try current dir
			filename = g_strdup(HOFFILE);
			fout.open(filename, ios::out | ios::trunc);
			if (!fout.is_open()) {
				if (debug) g_print("  opening >%s< failed\n",filename);
				// try home dir
				filename = g_strdup_printf("%s%s%s",
					g_get_home_dir(), G_DIR_SEPARATOR_S, HOFFILE);
				fout.open(filename, ios::out | ios::trunc);
				if (!fout.is_open()) {
					if (debug) g_print("  opening >%s< failed\n",filename);
					g_free(filename);
					return 1;
		}	}	}
		if (fout.is_open())
			// close without writin anything
			fout.close();
		// refresh HOF if open
		if (whof->_viewed) whof->hof(FALSE);
	} else {
		// ask confirmation
		GtkWidget *box;
		box = gnome_message_box_new (
			_("Do you really want to clean all scores ?"),
			GNOME_MESSAGE_BOX_QUESTION,
			GNOME_STOCK_BUTTON_YES,
			GNOME_STOCK_BUTTON_NO,
			NULL);
		gnome_dialog_set_parent (GNOME_DIALOG(box), GTK_WINDOW(_window));
		gnome_dialog_set_default (GNOME_DIALOG(box), 0);
		gtk_window_set_modal (GTK_WINDOW (box), TRUE);  
		gtk_signal_connect (GTK_OBJECT (box), "clicked",
			(GtkSignalFunc)cb_dialog_clean_all_scores_clicked, this);
		gtk_widget_show(box);
	}
	return 0;
}

void CWSettings::enable_sounds(gboolean action)
{
	if (debug) g_print("CWSettings::enable_sounds %d\n", action);
	for (gint i=0; i<NUMBER_SOUND_EVENT; i++)
		_sounds[i]._played = action;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_check_sound), action);
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
}

void CWSettings::free_memory(void)
{
	if (debug) g_print("CWSettings::free_memory\n");
	delete _settings;
	g_list_free(_sound_list);
}

void CWSettings::key_pressed(gint key)
{
	if (debug) g_print("CWSettings::key_pressed(%c=%d)\n",key,key);
	if (_key == key_NONE) return;
	_settings->_keys[_key] = key;
	// change label of button
	GtkWidget *lbl;
	gchar st[32];
	lbl = gtk_label_new(stringify_key(key, st));
	gtk_widget_destroy(GTK_BIN(_key_buttons[_key])->child);
	gtk_container_add (GTK_CONTAINER (_key_buttons[_key]), lbl);
	gtk_widget_show(lbl);
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
	// make apply button insensitive if same key choosed twice
	for (gint i=0; i<key_NONE; i++)
		if (i != _key && _settings->_keys[i] == key) {
			gnome_property_box_set_state(
				GNOME_PROPERTY_BOX(_window), FALSE);
			return;
}	}

void CWSettings::play_sound(void)
{
	if (debug) g_print("CWSettings::play_sound\n");
	if (_current_sound == -1) return; // invalid event
	gnome_sound_play(_sounds[_current_sound]._file);
}

void CWSettings::restore_default_sound(void)
{
	if (debug) g_print("CWSettings::restore_default_sound\n");
	if (_current_sound == -1) return; // not a valid sound event
	gtk_entry_set_text(GTK_ENTRY(_sound_entry),
		_sounds[_current_sound]._default); // calls entry changed event
}

void CWSettings::restore_default_sounds(void)
{
	if (debug) g_print("CWSettings::restore_default_sounds\n");
	for (gint i=0; i<NUMBER_SOUND_EVENT; i++)
		strcpy(_sounds[i]._file, _sounds[i]._default);
	// reset list
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(_combo)->entry),
		_sounds[0]._event);
	change_sound_event();
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
}

void CWSettings::select_default_kb(gint kb)
{
	if (debug) g_print("CWSettings::select_default_kb(%d)\n",kb);
	// map default key string
	if (kb == kb_FR) { // FRench kb layout
		gchar *dflt_key_string = DFLT_KEY_BINDING_FR;
		for (gint i=0; i<key_NONE; i++)
			_settings->_keys[i] = dflt_key_string[i];
	} else { // default = US
		gchar *dflt_key_string = DFLT_KEY_BINDING;
		for (gint i=0; i<key_NONE; i++)
			_settings->_keys[i] = dflt_key_string[i];
	}
	// update labels of all buttons
	GtkWidget *lbl;
	gchar st[32];
	for (gint i=0; i<key_NONE; i++) {
		gtk_widget_destroy(GTK_BIN(_key_buttons[i])->child);
		lbl = gtk_label_new(stringify_key(_settings->_keys[i], st));
		gtk_container_add (GTK_CONTAINER (_key_buttons[i]), lbl);
		gtk_widget_show(lbl);
	}
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
}

void CWSettings::select_key(gint button)
{
	if (_key_buttons[button] == NULL) return;
	if (debug) g_print("CWSettings::select_key(%d)\n",button);
	if (GTK_TOGGLE_BUTTON(_key_buttons[button])->active) {
		// toggled on -> toggle off preceeding button
		if (_key != key_NONE && _key_buttons[_key] != NULL)
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(_key_buttons[_key]), FALSE);
		_key = button;
	} else {
		// toggled off
		_key = key_NONE;
}	}

gint CWSettings::show(GtkWidget *parent)
{
	if (debug) g_print("CWSettings::show()\n");
	gchar st[32];
	GtkWidget *tab;
	GtkWidget *box1,*box2;
	GtkWidget *table;
	GtkWidget *lbl;
	GtkWidget *scale;
	GtkWidget *button;
	GtkWidget *check_button;
	GtkWidget *spin_button;
	GtkWidget *sep;

	// init	window	
	if (_viewed) {
		if (debug) g_print("CWSettings::show: window already open, exiting\n");
		return 1;
	}
	_viewed = true;
	_parent = parent;
	
	// init settings
	*_settings = *settings;
	for (gint i=0; i<NUMBER_SOUND_EVENT+1; i++)
		_sounds[i] = sounds[i];
	_sound_list = NULL;
	for (gint i=0; i<NUMBER_SOUND_EVENT; i++)
		_sound_list = g_list_append(_sound_list, _sounds[i]._event);
	_current_sound = -1;

	// create window
	_window = gnome_property_box_new ();
	gnome_dialog_set_parent (GNOME_DIALOG(_window), GTK_WINDOW(_parent));
	gtk_signal_connect_object(GTK_OBJECT(_window), "destroy",
		GTK_SIGNAL_FUNC(cb_close_dialog), GTK_OBJECT(_window));

	// TABS: Gaming, View, Key Bindings, Sound, HOF, App
	
	// gaming tab //////////////////////////////////
	tab = gtk_label_new (_(" Gaming "));

	box1 = gtk_vbox_new(FALSE, 5);

	box2 = gtk_hbox_new(FALSE, 5);
	lbl = gtk_label_new(_("World size:"));
	gtk_box_pack_start(GTK_BOX(box2), lbl, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);
    
	table = gtk_table_new(3,2,FALSE);
	gtk_container_border_width (GTK_CONTAINER(table), GNOME_PAD_SMALL);
	
	// width
	lbl = gtk_label_new(_("World Width"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,0,1,0,1);
	_adj_width = gtk_adjustment_new(_settings->_w,
		MIN_WORLD_WIDTH, MAX_WORLD_WIDTH, 1.0, 1.0, 0);
	scale = gtk_hscale_new(GTK_ADJUSTMENT(_adj_width));
	gtk_scale_set_digits(GTK_SCALE(scale),0);
	gtk_scale_set_value_pos(GTK_SCALE(scale), GTK_POS_LEFT);
	gtk_table_attach_defaults(GTK_TABLE(table),scale,1,2,0,1);
	gtk_signal_connect(GTK_OBJECT(_adj_width),"value_changed",
		GTK_SIGNAL_FUNC(cb_width_value_changed), this);

	// height
	lbl = gtk_label_new(_("World Height"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,0,1,1,2);
	_adj_height = gtk_adjustment_new(_settings->_h,
		MIN_WORLD_HEIGHT, MAX_WORLD_HEIGHT, 1.0, 1.0, 0);
	scale = gtk_hscale_new(GTK_ADJUSTMENT(_adj_height));
	gtk_scale_set_digits(GTK_SCALE(scale),0);
	gtk_scale_set_value_pos(GTK_SCALE(scale), GTK_POS_LEFT);
	gtk_table_attach_defaults(GTK_TABLE(table),scale,1,2,1,2);
	gtk_signal_connect(GTK_OBJECT(_adj_height),"value_changed",
		GTK_SIGNAL_FUNC(cb_height_value_changed), this);

	// depth
	lbl = gtk_label_new(_("World Depth"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,0,1,2,3);
	_adj_depth = gtk_adjustment_new(_settings->_d,
		MIN_WORLD_DEPTH, MAX_WORLD_DEPTH, 1.0, 1.0, 0);
	scale = gtk_hscale_new(GTK_ADJUSTMENT(_adj_depth));
	gtk_scale_set_digits(GTK_SCALE(scale),0);
	gtk_scale_set_value_pos(GTK_SCALE(scale), GTK_POS_LEFT);
	gtk_table_attach_defaults(GTK_TABLE(table),scale,1,2,2,3);
	gtk_signal_connect(GTK_OBJECT(_adj_depth),"value_changed",
		GTK_SIGNAL_FUNC(cb_depth_value_changed), this);

	gtk_box_pack_start(GTK_BOX(box1), table, FALSE, FALSE, 0);

	// separator
	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(box1), sep, FALSE, FALSE, 5);
	box2 = gtk_hbox_new(FALSE, 5);
	lbl = gtk_label_new(_("Game difficulty:"));
	gtk_box_pack_start(GTK_BOX(box2), lbl, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);

	// difficulty buttons
	gchar *str = g_strdup_printf(_("The current set of pieces is %s"),
		CSettings::diff_descriptors[_settings->_difficulty]);
	_diff_label = gtk_label_new(str);
	g_free(str);
	gtk_box_pack_start(GTK_BOX(box1), _diff_label, TRUE, TRUE, 0);
	box2 = gtk_hbox_new(FALSE, 5);
	button = gtk_button_new_with_label(
		CSettings::diff_descriptors[CSettings::diff_EASY]);
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);  
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_easy_clicked), this);
	button = gtk_button_new_with_label(
		CSettings::diff_descriptors[CSettings::diff_MEDIUM]);
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);  
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_medium_clicked), this);
	button = gtk_button_new_with_label(
		CSettings::diff_descriptors[CSettings::diff_HARD]);
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);  
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_hard_clicked), this);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);

	// separator
	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(box1), sep, FALSE, FALSE, 5);
	box2 = gtk_hbox_new(FALSE, 5);
	lbl = gtk_label_new(_("Piece moving speed:"));
	gtk_box_pack_start(GTK_BOX(box2), lbl, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);

	// speed buttons
	str = g_strdup_printf(_("The current speed is set to %s"),
		CSettings::speed_descriptors[_settings->_speed]);
	_speed_label = gtk_label_new(str);
	g_free(str);
	gtk_box_pack_start(GTK_BOX(box1), _speed_label, TRUE, TRUE, 0);
	box2 = gtk_hbox_new(FALSE, 5);
	button = gtk_button_new_with_label(
		CSettings::speed_descriptors[CSettings::speed_NORMAL]);
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);  
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_normal_clicked), this);
	button = gtk_button_new_with_label(
		CSettings::speed_descriptors[CSettings::speed_FAST]);
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);  
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_fast_clicked), this);
	button = gtk_button_new_with_label(
		CSettings::speed_descriptors[CSettings::speed_INSTANT]);
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);  
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_instant_clicked), this);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);
	
	gnome_property_box_append_page(GNOME_PROPERTY_BOX(_window), box1, tab);

	// view tab //////////////////////////////////
	tab = gtk_label_new (_(" View "));

	box1 = gtk_vbox_new(FALSE, 5);

	_check_button_show_np =
		gtk_check_button_new_with_label(_("Show next piece"));
	GTK_TOGGLE_BUTTON(_check_button_show_np)->active = _settings->_show_np;
	gtk_box_pack_start(GTK_BOX(box1), _check_button_show_np, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(_check_button_show_np), "toggled",
		GTK_SIGNAL_FUNC(cb_check_button_show_np_toggled), this);

	_check_button_show_scale =
		gtk_check_button_new_with_label(_("Show depth scale"));
	GTK_TOGGLE_BUTTON(_check_button_show_scale)->active =
		_settings->_show_scale;
	gtk_box_pack_start(GTK_BOX(box1), _check_button_show_scale, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(_check_button_show_scale), "toggled",
		GTK_SIGNAL_FUNC(cb_check_button_show_scale_toggled), this);

	_check_button_show_score =
		gtk_check_button_new_with_label(_("Show score"));
	GTK_TOGGLE_BUTTON(_check_button_show_score)->active =
		_settings->_show_score;
	gtk_box_pack_start(GTK_BOX(box1), _check_button_show_score, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(_check_button_show_score), "toggled",
		GTK_SIGNAL_FUNC(cb_check_button_show_score_toggled), this);

	_check_button_show_stats =
		gtk_check_button_new_with_label(_("Show statistics"));
	GTK_TOGGLE_BUTTON(_check_button_show_stats)->active =
		_settings->_show_stats;
	gtk_box_pack_start(GTK_BOX(box1), _check_button_show_stats, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(_check_button_show_stats), "toggled",
		GTK_SIGNAL_FUNC(cb_check_button_show_stats_toggled), this);

	// buttons
	box2 = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);
	
	button = gtk_button_new_with_label(_("Show all"));
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);  
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_show_all_clicked), this);

	button = gtk_button_new_with_label(_("Hide all"));
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);  
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_hide_all_clicked), this);
		
	gnome_property_box_append_page(GNOME_PROPERTY_BOX(_window), box1, tab);
		
	// keys tab //////////////////////////////////
	tab = gtk_label_new (_(" Keys "));

	box1 = gtk_vbox_new(FALSE, 5);

	lbl = gtk_label_new(_("Click on the control you wish to change then hit a new key"));
	gtk_box_pack_start(GTK_BOX(box1), lbl, FALSE, FALSE, 0);

	table = gtk_table_new(7,3,FALSE);
	gtk_container_border_width (GTK_CONTAINER(table), GNOME_PAD_SMALL);

	lbl = gtk_label_new(_("-"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,1,2,0,1);
	lbl = gtk_label_new(_("+"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,2,3,0,1);

	lbl = gtk_label_new(_("X-axis rotation"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,0,1,1,2);
	_key_buttons[key_XNEG] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_XNEG], st));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_XNEG]), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table),_key_buttons[key_XNEG],1,2,1,2);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_XNEG]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_xneg_clicked), this);
	_key_buttons[key_XPOS] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_XPOS], st));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_XPOS]), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table),_key_buttons[key_XPOS],2,3,1,2);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_XPOS]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_xpos_clicked), this);

	lbl = gtk_label_new(_("Y-axis rotation"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,0,1,2,3);
	_key_buttons[key_YNEG] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_YNEG], st));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_YNEG]), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table),_key_buttons[key_YNEG],1,2,2,3);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_YNEG]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_yneg_clicked), this);
	_key_buttons[key_YPOS] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_YPOS], st));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_YPOS]), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table),_key_buttons[key_YPOS],2,3,2,3);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_YPOS]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_ypos_clicked), this);

	lbl = gtk_label_new(_("Z-axis rotation"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,0,1,3,4);
	_key_buttons[key_ZNEG] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_ZNEG], st));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_ZNEG]), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table),_key_buttons[key_ZNEG],1,2,3,4);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_ZNEG]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_zneg_clicked), this);
	_key_buttons[key_ZPOS] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_ZPOS], st));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_ZPOS]), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table),_key_buttons[key_ZPOS],2,3,3,4);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_ZPOS]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_zpos_clicked), this);

	lbl = gtk_label_new(_("X-axis translation (besides arrows)"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,0,1,4,5);
	_key_buttons[key_LEFT] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_LEFT], st));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_LEFT]), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table),_key_buttons[key_LEFT],1,2,4,5);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_LEFT]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_left_clicked), this);
	_key_buttons[key_RIGHT] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_RIGHT], st));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_RIGHT]), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table),_key_buttons[key_RIGHT],2,3,4,5);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_RIGHT]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_right_clicked), this);

	lbl = gtk_label_new(_("Y-axis translation (besides arrows)"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,0,1,5,6);
	_key_buttons[key_DOWN] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_DOWN], st));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_DOWN]), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table),_key_buttons[key_DOWN],1,2,5,6);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_DOWN]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_down_clicked), this);
	_key_buttons[key_UP] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_UP], st));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_UP]), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table),_key_buttons[key_UP],2,3,5,6);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_UP]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_up_clicked), this);

	gtk_box_pack_start(GTK_BOX(box1), table, FALSE, FALSE, 0);

	box2 = gtk_hbox_new(TRUE, 5);

	lbl = gtk_label_new(_("Drop"));
	gtk_box_pack_start(GTK_BOX(box2), lbl, TRUE, TRUE, 0);
	_key_buttons[key_DROP] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_DROP], st));
	gtk_box_pack_start(GTK_BOX(box2), _key_buttons[key_DROP], TRUE, TRUE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_DROP]), FALSE);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_DROP]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_drop_clicked), this);

	lbl = gtk_label_new(_("Pause"));
	gtk_box_pack_start(GTK_BOX(box2), lbl, TRUE, TRUE, 0);
	_key_buttons[key_PAUSE] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_PAUSE], st));
	gtk_box_pack_start(GTK_BOX(box2), _key_buttons[key_PAUSE], TRUE, TRUE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_PAUSE]), FALSE);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_PAUSE]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_pause_clicked), this);

	lbl = gtk_label_new(_("Next piece"));
	gtk_box_pack_start(GTK_BOX(box2), lbl, TRUE, TRUE, 0);
	_key_buttons[key_TOGGLENP] = gtk_toggle_button_new_with_label(stringify_key(_settings->_keys[key_TOGGLENP], st));
	gtk_box_pack_start(GTK_BOX(box2), _key_buttons[key_TOGGLENP], TRUE, TRUE, 5);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_key_buttons[key_TOGGLENP]), FALSE);
	gtk_signal_connect(GTK_OBJECT(_key_buttons[key_TOGGLENP]), "clicked",
		GTK_SIGNAL_FUNC(cb_button_key_togglenp_clicked), this);

	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);

	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(box1), sep, FALSE, FALSE, 5);

	box2 = gtk_hbox_new(TRUE, 5);
	lbl = gtk_label_new(_("Restore default bindings :"));
	gtk_box_pack_start(GTK_BOX(box2), lbl, FALSE, FALSE, 0);
	button = gtk_button_new_with_label(_("QWERTY keyboard"));
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_qwerty_kb_clicked), this);
	button = gtk_button_new_with_label(_("AZERTY keyboard"));
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_azerty_kb_clicked), this);
	
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);

	gnome_property_box_append_page(GNOME_PROPERTY_BOX(_window), box1, tab);

	// special binding for keys tab
	gtk_signal_connect(GTK_OBJECT(_window), "key_press_event",
		GTK_SIGNAL_FUNC(cb_key_press_event),this);

	// sounds tab //////////////////////////////////
	tab = gtk_label_new (_(" Sounds "));

	box1 = gtk_vbox_new(FALSE, 5);

	_check_button_sound_support =
		gtk_check_button_new_with_label(_("Enable sounds"));
	GTK_TOGGLE_BUTTON(_check_button_sound_support)->active =
		_settings->_sound_support;
	gtk_box_pack_start(GTK_BOX(box1),
		_check_button_sound_support, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(_check_button_sound_support), "toggled",
		GTK_SIGNAL_FUNC(cb_check_button_sound_support_toggled), this);

	// separator
	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(box1), sep, FALSE, FALSE, 5);

	// sound event + drop-down box + numbers
	box2 = gtk_hbox_new(FALSE, 5);
	lbl = gtk_label_new(_("Sound event : "));
	gtk_box_pack_start(GTK_BOX(box2), lbl, FALSE, FALSE, 0);
	_combo = gtk_combo_new();
	gtk_combo_set_popdown_strings(GTK_COMBO(_combo), _sound_list);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(_combo)->entry), "changed",
		GTK_SIGNAL_FUNC(cb_sound_event_combo_entry_changed), this);
	gtk_box_pack_start(GTK_BOX(box2), _combo, FALSE, FALSE, 0);
	_current_sound = csound_get_event_number(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(_combo)->entry)));
	str = g_strdup_printf("%d / %d",_current_sound+1, NUMBER_SOUND_EVENT);
	_sound_number_lbl = gtk_label_new(str);
	g_free(str);
	gtk_box_pack_start(GTK_BOX(box2), _sound_number_lbl, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);
	
	// notes label + note
	box2 = gtk_hbox_new(FALSE, 5);
	lbl = gtk_label_new(_("Sound played when"));
	gtk_box_pack_start(GTK_BOX(box2), lbl, FALSE, FALSE, 0);
	_sound_event_note = gtk_label_new(_sounds[_current_sound]._note);
	gtk_label_set_line_wrap(GTK_LABEL(_sound_event_note), TRUE);
	gtk_box_pack_start(GTK_BOX(box2), _sound_event_note, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);

	// check box + text entry
	box2 = gtk_hbox_new(FALSE, 5);
	_check_sound = gtk_check_button_new_with_label(_("Enable"));
	GTK_TOGGLE_BUTTON(_check_sound)->active = _sounds[_current_sound]._played;
	gtk_box_pack_start(GTK_BOX(box2), _check_sound, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(_check_sound), "toggled",
		GTK_SIGNAL_FUNC(cb_check_sound_toggled), this);
	_sound_entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(_sound_entry), sounds[_current_sound]._file);
	gtk_box_pack_start(GTK_BOX(box2), _sound_entry, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(_sound_entry), "changed",
		GTK_SIGNAL_FUNC(cb_sound_entry_changed), this);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);
	
	// buttons browse... + play + restore default
	box2 = gtk_hbox_new(TRUE, 5);
	_browse_sound_button = gtk_button_new_with_label(_("Browse..."));
	gtk_box_pack_start(GTK_BOX(box2), _browse_sound_button, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(_browse_sound_button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_browse_sounds_clicked), this);
	_play_sound_button = gtk_button_new_with_label(_("Play"));
	gtk_box_pack_start(GTK_BOX(box2), _play_sound_button, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(_play_sound_button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_play_sound_clicked), this);
	_default_sound_button = gtk_button_new_with_label(_("Restore default"));
	gtk_box_pack_start(GTK_BOX(box2), _default_sound_button, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(_default_sound_button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_default_sound_clicked), this);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);
		
	// separator
	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(box1), sep, FALSE, FALSE, 5);

	// disable all, enable all buttons
	box2 = gtk_hbox_new(TRUE, 5);
	button = gtk_button_new_with_label(_("Mute all sound events"));
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_disable_sounds_clicked), this);
	button = gtk_button_new_with_label(_("Play all sound events"));
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_enable_sounds_clicked), this);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);
	
	// restore all defaults button
	button = gtk_button_new_with_label(_("Restore all default sounds"));
	gtk_box_pack_start(GTK_BOX(box1), button, FALSE, FALSE, 5);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_default_sounds_clicked), this);

	gnome_property_box_append_page(GNOME_PROPERTY_BOX(_window), box1, tab);

	// HOF tab //////////////////////////////////
	tab = gtk_label_new (_(" HOF "));

	box1 = gtk_vbox_new(FALSE, 5);

	// number of lines
	box2 = gtk_hbox_new(FALSE, 5);
	lbl = gtk_label_new(_("View:"));
	gtk_box_pack_start(GTK_BOX(box2), lbl, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);

	box2 = gtk_hbox_new(FALSE, 5);
	lbl = gtk_label_new(_("Number of records in the Hall Of Fame:"));
	gtk_box_pack_start(GTK_BOX(box2), lbl, FALSE, FALSE, 10);
	_adj_nhofl = gtk_adjustment_new(_settings->_nhofl /*value*/,
		MIN_NHOFL/*lower*/, MAX_NHOFL/*upper*/,
		1.0/*step_increment*/, 5.0/*page_increment*/,
		0/*page_size unuzed*/);
	spin_button = gtk_spin_button_new(
		GTK_ADJUSTMENT(_adj_nhofl), 0.0/*accel*/,0/*digits*/);
	gtk_signal_connect(GTK_OBJECT(spin_button), "changed",
		GTK_SIGNAL_FUNC(cb_nhofl_sbutton_value_changed), this);
	gtk_box_pack_start(GTK_BOX(box2), spin_button, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);

	// separator
	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(box1), sep, FALSE, FALSE, 5);

	// old scores
	box2 = gtk_hbox_new(FALSE, 5);
	lbl = gtk_label_new(_("Old scores:"));
	gtk_box_pack_start(GTK_BOX(box2), lbl, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);

	// explanation
	lbl = gtk_label_new(_("Old scores are from games played with an incompatible version of 3D Tetris.\nRanks are displayed between parenthesis in the Hall Of Fame."));
	gtk_label_set_justify(GTK_LABEL(lbl), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(GTK_BOX(box1), lbl, FALSE, FALSE, 0);

	// check box
	check_button =
		gtk_check_button_new_with_label(_("Show old scores"));
	GTK_TOGGLE_BUTTON(check_button)->active = _settings->_show_old_score;
	gtk_box_pack_start(GTK_BOX(box1), check_button, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(check_button), "toggled",
		GTK_SIGNAL_FUNC(cb_check_button_show_old_score_toggled), this);

	// separator
	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(box1), sep, FALSE, FALSE, 5);

	// buttons
	box2 = gtk_hbox_new(TRUE, 5);
	button = gtk_button_new_with_label(_("Show/Refresh Hall Of Fame"));
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_show_hof_clicked), this);
	button = gtk_button_new_with_label(_("Clean all scores"));
	gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cb_button_clean_all_scores_clicked), this);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);
	
	
	gnome_property_box_append_page(GNOME_PROPERTY_BOX(_window), box1, tab);
	
	// app tab //////////////////////////////////
	tab = gtk_label_new (_(" App. "));

	box1 = gtk_vbox_new(FALSE, 5);

	box2 = gtk_hbox_new(FALSE, 5);
	lbl = gtk_label_new(_("Focus behavior:"));
	gtk_box_pack_start(GTK_BOX(box2), lbl, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);

	check_button = gtk_check_button_new_with_label(
		_("Pause game when the window loses focus"));
	GTK_TOGGLE_BUTTON(check_button)->active = _settings->_pause_on_nofocus;
	gtk_box_pack_start(GTK_BOX(box1), check_button, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(check_button), "toggled",
		GTK_SIGNAL_FUNC(cb_check_button_pause_on_nofocus_toggled), this);

	_check_button_resume = gtk_check_button_new_with_label(
		_("Resume game when the window grabs focus"));
	GTK_TOGGLE_BUTTON(_check_button_resume)->active =
		_settings->_resume_on_focus;
	gtk_widget_set_sensitive(_check_button_resume,
		_settings->_pause_on_nofocus);
	gtk_box_pack_start(GTK_BOX(box1), _check_button_resume, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(_check_button_resume), "toggled",
		GTK_SIGNAL_FUNC(cb_check_button_resume_on_focus_toggled), this);
		
	gnome_property_box_append_page(GNOME_PROPERTY_BOX(_window), box1, tab);
	
	// Prefs buttons signals ///////////////////////////
	gtk_signal_connect (GTK_OBJECT(_window), "apply",
		GTK_SIGNAL_FUNC(cb_apply_button_clicked), this);

	gtk_widget_show_all(_window);
	return 0;
}

void CWSettings::show_hof(void)
{
	if (debug > 1) g_print("CWSettings::show_hof\n");
	whof->hof(FALSE, _settings->_show_old_score, _settings->_nhofl);
}

gchar *CWSettings::stringify_key(gint key, gchar *str)
{
	if (debug > 1) g_print("CWSettings::stringify_key(%c=%d)\n",key,key);
	if (key >= 'a' && key <= 'z')
		sprintf(str, "%c",key-'a'+'A');
	else sprintf(str, "%s",gdk_keyval_name(key));
	return str;
}

void CWSettings::toggle_pause_on_nofocus(gboolean state)
{
	if (debug) g_print("CWSettings::toggle_pause_on_nofocus %d\n", state);
	_settings->_pause_on_nofocus = state;
	gtk_widget_set_sensitive(_check_button_resume, state);
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
}

void CWSettings::toggle_play_for_sound_event(void)
{
	if (debug) g_print("CWSettings::toggle_play_for_sound_event\n");
	if (_current_sound != -1) {
		_sounds[_current_sound]._played =
			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_check_sound));
		gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));			
	}
}

void CWSettings::toggle_resume_on_focus(gboolean state)
{
	if (debug) g_print("CWSettings::toggle_resume_nofocus %d\n", state);
	_settings->_resume_on_focus = state;
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
}

void CWSettings::toggle_show_np(gint action)
{
	if (debug) g_print("CWSettings::toggle_show_np\n");
	if (action == toggle_ON)
		GTK_TOGGLE_BUTTON(_check_button_show_np)->active = TRUE;
	else if (action == toggle_OFF)
		GTK_TOGGLE_BUTTON(_check_button_show_np)->active = FALSE;
	else if (action == toggle_TOGGLE)
		GTK_TOGGLE_BUTTON(_check_button_show_np)->active =
			!GTK_TOGGLE_BUTTON(_check_button_show_np)->active;
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
	_settings->_show_np = GTK_TOGGLE_BUTTON(_check_button_show_np)->active;
}

void CWSettings::toggle_show_old_score(gboolean state)
{
	if (debug) g_print("CWSettings::toggle_show_old_score %d\n", state);
	_settings->_show_old_score = state;
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
}

void CWSettings::toggle_show_scale(gint action)
{
	if (debug) g_print("CWSettings::toggle_show_scale\n");
	if (action == toggle_ON)
		GTK_TOGGLE_BUTTON(_check_button_show_scale)->active = TRUE;
	else if (action == toggle_OFF)
		GTK_TOGGLE_BUTTON(_check_button_show_scale)->active = FALSE;
	else if (action == toggle_TOGGLE)
		GTK_TOGGLE_BUTTON(_check_button_show_scale)->active =
			!GTK_TOGGLE_BUTTON(_check_button_show_scale)->active;
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
	_settings->_show_scale =
		GTK_TOGGLE_BUTTON(_check_button_show_scale)->active;
}

void CWSettings::toggle_show_score(gint action)
{
	if (debug) g_print("CWSettings::toggle_show_score\n");
	if (action == toggle_ON)
		GTK_TOGGLE_BUTTON(_check_button_show_score)->active = TRUE;
	else if (action == toggle_OFF)
		GTK_TOGGLE_BUTTON(_check_button_show_score)->active = FALSE;
	else if (action == toggle_TOGGLE)
		GTK_TOGGLE_BUTTON(_check_button_show_score)->active =
			!GTK_TOGGLE_BUTTON(_check_button_show_score)->active;
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
	_settings->_show_score =
		GTK_TOGGLE_BUTTON(_check_button_show_score)->active;
}

void CWSettings::toggle_show_stats(gint action)
{
	if (debug) g_print("CWSettings::toggle_show_stats\n");
	if (action == toggle_ON)
		GTK_TOGGLE_BUTTON(_check_button_show_stats)->active = TRUE;
	else if (action == toggle_OFF)
		GTK_TOGGLE_BUTTON(_check_button_show_stats)->active = FALSE;
	else if (action == toggle_TOGGLE)
		GTK_TOGGLE_BUTTON(_check_button_show_stats)->active =
			!GTK_TOGGLE_BUTTON(_check_button_show_stats)->active;
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
	_settings->_show_stats =
		GTK_TOGGLE_BUTTON(_check_button_show_stats)->active;
}

void CWSettings::toggle_sound_support(gint action)
{
	if (debug) g_print("CWSettings::toggle_sound_support\n");
	if (action == toggle_ON)
		GTK_TOGGLE_BUTTON(_check_button_sound_support)->active = TRUE;
	else if (action == toggle_OFF)
		GTK_TOGGLE_BUTTON(_check_button_sound_support)->active = FALSE;
	else if (action == toggle_TOGGLE)
		GTK_TOGGLE_BUTTON(_check_button_sound_support)->active =
			!GTK_TOGGLE_BUTTON(_check_button_sound_support)->active;
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
	_settings->_sound_support =
		GTK_TOGGLE_BUTTON(_check_button_sound_support)->active;
}

void CWSettings::update_difficulty_label(void)
{
	gchar *str = g_strdup_printf(_("The current set of pieces is %s"),
		CSettings::diff_descriptors[_settings->_difficulty]);
	gtk_label_set_text(GTK_LABEL(_diff_label), str);
	g_free(str);
}

void CWSettings::update_speed_label(void)
{
	gchar *str = g_strdup_printf(_("The current speed is set to %s"),
		CSettings::speed_descriptors[_settings->_speed]);
	gtk_label_set_text(GTK_LABEL(_speed_label), str);
	g_free(str);
}

void CWSettings::view_toggle_all(gboolean action)
{
	if (debug) g_print("CWSettings::view_toggle_all %d\n", action);
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(_check_button_show_np), action);
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(_check_button_show_score), action);
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(_check_button_show_scale), action);
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(_check_button_show_stats), action);
}

// Callbacks ////////////////////////////////////////////////////////////////

// all tabs

void CWSettings::cb_apply_button_clicked(GtkWidget *widget, gint page_num,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_apply_button_clicked(%d)\n",
		page_num);
	if (page_num == -1) ((CWSettings *)data)->apply();
}	

void CWSettings::cb_close_dialog(GtkObject *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_close_dialog\n");
	gnome_dialog_close(GNOME_DIALOG(widget));
	_viewed = false;
}

// Settings tab events

void CWSettings::cb_button_easy_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_easy_clicked\n");
	((CWSettings *)data)->change_difficulty(CSettings::diff_EASY);
}

void CWSettings::cb_button_hard_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_hard_clicked\n");
	((CWSettings *)data)->change_difficulty(CSettings::diff_HARD);
}

void CWSettings::cb_button_medium_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_medium_clicked\n");
	((CWSettings *)data)->change_difficulty(CSettings::diff_MEDIUM);
}

void CWSettings::cb_button_normal_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_normal_clicked\n");
	((CWSettings *)data)->change_speed(CSettings::speed_NORMAL);
}

void CWSettings::cb_button_fast_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_fast_clicked\n");
	((CWSettings *)data)->change_speed(CSettings::speed_FAST);
}

void CWSettings::cb_button_instant_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("wCWSettings::cb_button_instant_clicked\n");
	((CWSettings *)data)->change_speed(CSettings::speed_INSTANT);
}

void CWSettings::cb_depth_value_changed(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_depth_value_changed\n");
	((CWSettings *)data)->change_depth();
}

void CWSettings::cb_height_value_changed(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_height_value_changed\n");
	((CWSettings *)data)->change_height();
}

void CWSettings::cb_width_value_changed(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_width_value_changed\n");
	((CWSettings *)data)->change_width();
}

// View tab events

void CWSettings::cb_check_button_show_np_toggled(GtkWidget *widget,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_check_button_show_np_toggled\n");
	((CWSettings *)data)->toggle_show_np();
}

void CWSettings::cb_check_button_show_scale_toggled(GtkWidget *widget,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_check_button_show_scale_toggled\n");
	((CWSettings *)data)->toggle_show_scale();
}

void CWSettings::cb_check_button_show_score_toggled(GtkWidget *widget,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_check_button_show_score_toggled\n");
	((CWSettings *)data)->toggle_show_score();
}

void CWSettings::cb_check_button_show_stats_toggled(GtkWidget *widget,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_check_button_show_stats_toggled\n");
	((CWSettings *)data)->toggle_show_stats();
}

void CWSettings::cb_button_show_all_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_show_all_clicked\n");
	((CWSettings *)data)->view_toggle_all(TRUE);
}

void CWSettings::cb_button_hide_all_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_hide_all_clicked\n");
	((CWSettings *)data)->view_toggle_all(FALSE);
}

// Key bindings tab events

void CWSettings::cb_button_key_xneg_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_xneg_clicked\n");
	((CWSettings *)data)->select_key(key_XNEG);
}

void CWSettings::cb_button_key_xpos_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_xpos_clicked\n");
	((CWSettings *)data)->select_key(key_XPOS);
}

void CWSettings::cb_button_key_yneg_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_yneg_clicked\n");
	((CWSettings *)data)->select_key(key_YNEG);
}

void CWSettings::cb_button_key_ypos_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_ypos_clicked\n");
	((CWSettings *)data)->select_key(key_YPOS);
}

void CWSettings::cb_button_key_zneg_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_zneg_clicked\n");
	((CWSettings *)data)->select_key(key_ZNEG);
}

void CWSettings::cb_button_key_zpos_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_zpos_clicked\n");
	((CWSettings *)data)->select_key(key_ZPOS);
}

void CWSettings::cb_button_key_left_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_left_clicked\n");
	((CWSettings *)data)->select_key(key_LEFT);
}

void CWSettings::cb_button_key_right_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_right_clicked\n");
	((CWSettings *)data)->select_key(key_RIGHT);
}

void CWSettings::cb_button_key_down_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_down_clicked\n");
	((CWSettings *)data)->select_key(key_DOWN);
}

void CWSettings::cb_button_key_up_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_up_clicked\n");
	((CWSettings *)data)->select_key(key_UP);
}

void CWSettings::cb_button_key_drop_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_drop_clicked\n");
	((CWSettings *)data)->select_key(key_DROP);
}

void CWSettings::cb_button_key_pause_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_pause_clicked\n");
	((CWSettings *)data)->select_key(key_PAUSE);
}

void CWSettings::cb_button_key_togglenp_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_key_togglenp_clicked\n");
	((CWSettings *)data)->select_key(key_TOGGLENP);
}

void CWSettings::cb_button_qwerty_kb_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_qwerty_kb_clicked\n");
	((CWSettings *)data)->select_default_kb(kb_US);
}

void CWSettings::cb_button_azerty_kb_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_azerty_kb_clicked\n");
	((CWSettings *)data)->select_default_kb(kb_FR);
}

gint CWSettings::cb_key_press_event(GtkWidget *widget, GdkEvent *event,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_key_press_event\n");
	((CWSettings *)data)->key_pressed((gint)(((GdkEventKey *)event)->keyval));
}

// Sound tab events

void CWSettings::cb_check_button_sound_support_toggled(GtkWidget *widget,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_check_button_sound_support_toggled\n");
	((CWSettings *)data)->toggle_sound_support();
}

void CWSettings::cb_sound_event_combo_entry_changed(GtkWidget *widget,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_sound_event_combo_entry_changed\n");
	((CWSettings *)data)->change_sound_event();
}

void CWSettings::cb_check_sound_toggled(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_check_sound_toggled\n");
	((CWSettings *)data)->toggle_play_for_sound_event();
}

void CWSettings::cb_sound_entry_changed(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_sound_entry_changed\n");
	((CWSettings *)data)->change_sound_file();
}

void CWSettings::cb_button_browse_sounds_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_browse_sounds_clicked\n");
	((CWSettings *)data)->browse_sound_files();
}

void CWSettings::cb_button_play_sound_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_play_clicked\n");
	((CWSettings *)data)->play_sound();
}

void CWSettings::cb_button_default_sound_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_default_sound_clicked\n");
	((CWSettings *)data)->restore_default_sound();
}

void CWSettings::cb_button_disable_sounds_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_disable_sounds_clicked\n");
	((CWSettings *)data)->enable_sounds(FALSE);
}

void CWSettings::cb_button_enable_sounds_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_enable_sounds_clicked\n");
	((CWSettings *)data)->enable_sounds(TRUE);
}

void CWSettings::cb_button_default_sounds_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_default_sounds_clicked\n");
	((CWSettings *)data)->restore_default_sounds();
}

// HOF

void CWSettings::cb_check_button_show_old_score_toggled(
	GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print(
		"CWSettings::cb_check_button_show_old_score_toggled\n");
	((CWSettings *)data)->toggle_show_old_score(
		GTK_TOGGLE_BUTTON(widget)->active);
}

void CWSettings::cb_nhofl_sbutton_value_changed(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_nhofl_button_value_changed\n");
	((CWSettings *)data)->change_nhofl(
		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)));
}

void CWSettings::cb_button_show_hof_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_show_hof_clicked\n");
	((CWSettings *)data)->show_hof();
}

void CWSettings::cb_button_clean_all_scores_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_button_clean_all_scores_clicked\n");
	((CWSettings *)data)->clean_all_scores(FALSE);
}

/* cb_dialog_clean_all_scores_clicked
 * Dialog Window: Clean all scores ?
 * button 0 = YES, button 1 = NO
*/
void CWSettings::cb_dialog_clean_all_scores_clicked(GtkWidget *widget, gint button, gpointer data)
{
	if (debug > DBG_CB) g_print("CWMain::cb_dialog_clean_all_scores_clicked: button = %d\n", button);
	if (button == 0) ((CWSettings *)data)->clean_all_scores(TRUE);
}
		
// app.

void CWSettings::cb_check_button_pause_on_nofocus_toggled(
	GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print(
		"CWSettings::cb_check_button_pause_on_nofocus_toggled\n");
	((CWSettings *)data)->toggle_pause_on_nofocus(
		GTK_TOGGLE_BUTTON(widget)->active);
}

void CWSettings::cb_check_button_resume_on_focus_toggled(
	GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print(
		"CWSettings::cb_check_button_resume_on_focus_toggled\n");
	((CWSettings *)data)->toggle_resume_on_focus(
		GTK_TOGGLE_BUTTON(widget)->active);
}
