/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999, 2000, 2001 Sebastien Nicoud <seb@eseb.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#ifndef _CWSETTINGS
#define _CWSETTINGS

#include "common.h"

class CWSettings
{
private:
	enum {
		kb_US,
		kb_FR
	};
	enum {
		toggle_ON,
		toggle_OFF,
		toggle_TOGGLE,
		toggle_KEEP
	};
public:
	static gboolean _viewed;

private:
	GtkWidget *_window;
	GtkWidget *_parent;
	CSettings *_settings;
	// gaming
	GtkObject *_adj_width;
	GtkObject *_adj_height;
	GtkObject *_adj_depth;
	GtkWidget *_diff_label;
	GtkWidget *_speed_label;
	// view
	GtkWidget *_check_button_show_np;
	GtkWidget *_check_button_show_scale;
	GtkWidget *_check_button_show_score;
	GtkWidget *_check_button_show_stats;
	// keys
	gint	   _key; // the number of the key button currently on
	GtkWidget *_key_buttons[16];
	// sounds
	GtkWidget *_check_button_sound_support;
	GtkWidget *_combo;
	CSound	   _sounds[NUMBER_SOUND_EVENT+1];
	gint	   _current_sound;
	GList	  *_sound_list;
	GtkWidget *_sound_number_lbl;
	GtkWidget *_sound_event_note;
	GtkWidget *_check_sound;
	GtkWidget *_sound_entry;
	GtkWidget *_browse_sound_button;
	GtkWidget *_play_sound_button;
	GtkWidget *_default_sound_button;
	// HOF
	GtkObject *_adj_nhofl;
	// app.
	GtkWidget *_check_button_resume;

public:
	CWSettings();
	~CWSettings();
	
public:
	gint show(GtkWidget *parent); // return 0:ok, 1:window already open, action cancelled

private:
	void apply(void);
	void free_memory(void);
	// game settings
	void change_depth(void);
	void change_difficulty(gint d);
	void change_height(void);
	void change_speed(gint d);
	void change_width(void);
	void update_difficulty_label(void);
	void update_speed_label(void);
	// view settings
	void toggle_show_np(gint action = toggle_KEEP);
	void toggle_show_scale(gint action = toggle_KEEP);
	void toggle_show_score(gint action = toggle_KEEP);
	void toggle_show_stats(gint action = toggle_KEEP);
	void view_toggle_all(gboolean action);
	// key bindings
	void key_pressed(gint key);
	void select_default_kb(gint kb);
	void select_key(gint button);
	gchar *stringify_key(gint key, gchar *str);
	// sound settings
	void toggle_sound_support(gint action = toggle_KEEP);
	void change_sound_event(void);
	void toggle_play_for_sound_event(void);
	void change_sound_file(void);
	void browse_sound_files(void);
	void play_sound(void);
	void restore_default_sound(void);
	void enable_sounds(gboolean action = TRUE);
	void restore_default_sounds(void);
	// HOF settings
	void change_nhofl(gint value);
	void toggle_show_old_score(gboolean state);
	gint clean_all_scores(gboolean confirmed = FALSE);
	void show_hof(void);
	// app. settings
	void toggle_pause_on_nofocus(gboolean state);
	void toggle_resume_on_focus(gboolean state);

// callbacks
private:
	static void cb_apply_button_clicked(GtkWidget *widget, gint page_num,
		gpointer data);
	static void cb_close_dialog(GtkObject *widget, gpointer data);
	// world settings
	static void cb_button_easy_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_hard_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_medium_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_normal_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_fast_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_instant_clicked(GtkWidget *widget, gpointer data);
	static void cb_depth_value_changed(GtkWidget *widget, gpointer data);
	static void cb_height_value_changed(GtkWidget *widget, gpointer data);
	static void cb_width_value_changed(GtkWidget *widget, gpointer data);	
	// view settings
	static void cb_check_button_show_np_toggled(
		GtkWidget *widget, gpointer data);
	static void cb_check_button_show_scale_toggled(
		GtkWidget *widget, gpointer data);
	static void cb_check_button_show_score_toggled(
		GtkWidget *widget, gpointer data);
	static void cb_check_button_show_stats_toggled(
		GtkWidget *widget, gpointer data);
	static void cb_button_show_all_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_hide_all_clicked(GtkWidget *widget, gpointer data);
	// key bindings
	static void cb_button_key_xneg_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_key_xpos_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_key_yneg_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_key_ypos_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_key_zneg_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_key_zpos_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_key_down_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_key_left_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_key_right_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_key_up_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_key_drop_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_key_pause_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_key_togglenp_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_qwerty_kb_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_azerty_kb_clicked(GtkWidget *widget, gpointer data);
	static gint cb_key_press_event(GtkWidget *widget, GdkEvent *event,
		gpointer data);
	// sound settings
	static void cb_check_button_sound_support_toggled(
		GtkWidget *widget, gpointer data);
	static void cb_sound_event_combo_entry_changed(
		GtkWidget *widget, gpointer data);
	static void cb_check_sound_toggled(GtkWidget *widget, gpointer data);
	static void cb_button_browse_sounds_clicked(
		GtkWidget *widget, gpointer data);
	static void cb_button_play_sound_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_default_sound_clicked(
		GtkWidget *widget, gpointer data);
	static void cb_sound_entry_changed(GtkWidget *widget, gpointer data);
	static void cb_button_default_sounds_clicked(
		GtkWidget *widget, gpointer data);
	static void cb_button_disable_sounds_clicked(
		GtkWidget *widget, gpointer data);
	static void cb_button_enable_sounds_clicked(
		GtkWidget *widget, gpointer data);
	// HOF settings
	static void cb_nhofl_sbutton_value_changed(GtkWidget *widget, gpointer data);
	static void cb_check_button_show_old_score_toggled(
		GtkWidget *widget, gpointer data);
	static void cb_button_show_hof_clicked(GtkWidget *widget, gpointer data);
	static void cb_button_clean_all_scores_clicked(
		GtkWidget *widget, gpointer data);
	static void cb_dialog_clean_all_scores_clicked(
		GtkWidget *widget, gint button, gpointer data);
	// app. settings
	static void cb_check_button_pause_on_nofocus_toggled(
		GtkWidget *widget, gpointer data);
	static void cb_check_button_resume_on_focus_toggled(
		GtkWidget *widget, gpointer data);
};

#endif
